@@ -23,8 +23,10 @@ import com.google.common.base.Joiner;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Sets;
 import org.apache.druid.annotations.SubclassesMustOverrideEqualsAndHashCode;
+import org.apache.druid.java.util.common.Cacheable;
 import org.apache.druid.java.util.common.ISE;
 import org.apache.druid.math.expr.vector.ExprVectorProcessor;
+import org.apache.druid.query.cache.CacheKeyBuilder;
 
 import javax.annotation.Nullable;
 import java.util.ArrayList;
@@ -38,8 +40,9 @@ import java.util.Set;
  * immutable.
  */
 @SubclassesMustOverrideEqualsAndHashCode
-public interface Expr
+public interface Expr extends Cacheable
 {
+
   String NULL_LITERAL = "null";
   Joiner ARG_JOINER = Joiner.on(", ");
 
@@ -132,13 +135,20 @@ public interface Expr
   BindingAnalysis analyzeInputs();
 
   /**
-   * Given an {@link InputBindingTypes}, compute what the output {@link ExprType} will be for this expression. A return
-   * value of null indicates that the given type information was not enough to resolve the output type, so the
-   * expression must be evaluated using default {@link #eval} handling where types are only known after evaluation,
-   * through {@link ExprEval#type}.
+   * Given an {@link InputBindingInspector}, compute what the output {@link ExprType} will be for this expression.
+   *
+   * In the vectorized expression engine, if {@link #canVectorize(InputBindingInspector)} returns true, a return value
+   * of null MUST ONLY indicate that the expression has all null inputs (non-existent columns) or null constants for
+   * the entire expression. Otherwise, all vectorizable expressions must produce an output type to correctly operate
+   * with the vectorized engine.
+   *
+   * Outside the context of vectorized expressions, a return value of null can also indicate that the given type
+   * information was not enough to resolve the output type, so the expression must be evaluated using default
+   * {@link #eval} handling where types are only known after evaluation, through {@link ExprEval#type}, such as
+   * transform expressions at ingestion time
    */
   @Nullable
-  default ExprType getOutputType(InputBindingTypes inputTypes)
+  default ExprType getOutputType(InputBindingInspector inspector)
   {
     return null;
   }
@@ -147,8 +157,9 @@ public interface Expr
    * Check if an expression can be 'vectorized', for a given set of inputs. If this method returns true,
    * {@link #buildVectorized} is expected to produce a {@link ExprVectorProcessor} which can evaluate values in batches
    * to use with vectorized query engines.
+   * @param inspector
    */
-  default boolean canVectorize(InputBindingTypes inputTypes)
+  default boolean canVectorize(InputBindingInspector inspector)
   {
     return false;
   }
@@ -156,19 +167,30 @@ public interface Expr
   /**
    * Builds a 'vectorized' expression processor, that can operate on batches of input values for use in vectorized
    * query engines.
+   * @param inspector
    */
-  default <T> ExprVectorProcessor<T> buildVectorized(VectorInputBindingTypes inputTypes)
+  default <T> ExprVectorProcessor<T> buildVectorized(VectorInputBindingInspector inspector)
   {
     throw Exprs.cannotVectorize(this);
   }
 
+  @Override
+  default byte[] getCacheKey()
+  {
+    return new CacheKeyBuilder(Exprs.EXPR_CACHE_KEY).appendString(stringify()).build();
+  }
+
   /**
    * Mechanism to supply input types for the bindings which will back {@link IdentifierExpr}, to use in the aid of
    * inferring the output type of an expression with {@link #getOutputType}. A null value means that either the binding
    * doesn't exist, or, that the type information is unavailable.
    */
-  interface InputBindingTypes
+  interface InputBindingInspector
   {
+    /**
+     * Get the {@link ExprType} from the backing store for a given identifier (this is likely a column, but could be other
+     * things depending on the backing adapter)
+     */
     @Nullable
     ExprType getType(String name);
 
@@ -179,12 +201,11 @@ public interface Expr
      */
     default boolean areNumeric(List<Expr> args)
     {
-      boolean numeric = args.size() > 0;
+      boolean numeric = true;
       for (Expr arg : args) {
         ExprType argType = arg.getOutputType(this);
         if (argType == null) {
-          numeric = false;
-          break;
+          continue;
         }
         numeric &= argType.isNumeric();
       }
@@ -202,7 +223,37 @@ public interface Expr
     }
 
     /**
-     * Check if every provided {@link Expr} computes {@link Expr#canVectorize(InputBindingTypes)} to a value of true
+     * Check if all provided {@link Expr} can infer the output type as {@link ExprType#isScalar()} (non-array) with a
+     * value of true.
+     *
+     * There must be at least one expression with a computable scalar output type for this method to return true.
+     */
+    default boolean areScalar(List<Expr> args)
+    {
+      boolean scalar = true;
+      for (Expr arg : args) {
+        ExprType argType = arg.getOutputType(this);
+        if (argType == null) {
+          continue;
+        }
+        scalar &= argType.isScalar();
+      }
+      return scalar;
+    }
+
+    /**
+     * Check if all provided {@link Expr} can infer the output type as {@link ExprType#isScalar()} (non-array) with a
+     * value of true.
+     *
+     * There must be at least one expression with a computable scalar output type for this method to return true.
+     */
+    default boolean areScalar(Expr... args)
+    {
+      return areScalar(Arrays.asList(args));
+    }
+
+    /**
+     * Check if every provided {@link Expr} computes {@link Expr#canVectorize(InputBindingInspector)} to a value of true
      */
     default boolean canVectorize(List<Expr> args)
     {
@@ -214,7 +265,7 @@ public interface Expr
     }
 
     /**
-     * Check if every provided {@link Expr} computes {@link Expr#canVectorize(InputBindingTypes)} to a value of true
+     * Check if every provided {@link Expr} computes {@link Expr#canVectorize(InputBindingInspector)} to a value of true
      */
     default boolean canVectorize(Expr... args)
     {
@@ -223,9 +274,9 @@ public interface Expr
   }
 
   /**
-   * {@link InputBindingTypes} + vectorizations stuff for {@link #buildVectorized}
+   * {@link InputBindingInspector} + vectorizations stuff for {@link #buildVectorized}
    */
-  interface VectorInputBindingTypes extends InputBindingTypes
+  interface VectorInputBindingInspector extends InputBindingInspector
   {
     int getMaxVectorSize();
   }
@@ -247,7 +298,7 @@ public interface Expr
    * the vectorized column selector interfaces, and includes {@link ExprType} information about all input bindings
    * which exist
    */
-  interface VectorInputBinding extends VectorInputBindingTypes
+  interface VectorInputBinding extends VectorInputBindingInspector
   {
     <T> T[] getObjectVector(String name);
 
@@ -259,6 +310,13 @@ public interface Expr
     boolean[] getNullVector(String name);
 
     int getCurrentVectorSize();
+
+    /**
+     * Returns an integer that uniquely identifies the current position of the underlying vector offset, if this
+     * binding is backed by a segment. This is useful for caching: it is safe to assume nothing has changed in the
+     * offset so long as the id remains the same. See also: ReadableVectorOffset (in druid-processing)
+     */
+    int getCurrentVectorId();
   }
 
   /**
