@@ -22,6 +22,7 @@ package org.apache.druid.math.expr;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Supplier;
+import com.google.common.base.Suppliers;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Sets;
@@ -35,12 +36,14 @@ import org.apache.druid.java.util.common.logger.Logger;
 import org.apache.druid.math.expr.antlr.ExprLexer;
 import org.apache.druid.math.expr.antlr.ExprParser;
 
+import javax.annotation.Nullable;
 import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.function.UnaryOperator;
 import java.util.stream.Collectors;
 
 public class Parser
@@ -96,17 +99,33 @@ public class Parser
   }
 
   /**
+   * Create a memoized lazy supplier to parse a string into a flattened {@link Expr}. There is some overhead to this,
+   * and these objects are all immutable, so this assists in the goal of re-using instead of re-creating whenever
+   * possible.
+   *
+   * Lazy form of {@link #parse(String, ExprMacroTable)}
+   *
+   * @param in expression to parse
+   * @param macroTable additional extensions to expression language
+   */
+  public static Supplier<Expr> lazyParse(@Nullable String in, ExprMacroTable macroTable)
+  {
+    return Suppliers.memoize(() -> in == null ? null : Parser.parse(in, macroTable));
+  }
+
+  /**
    * Parse a string into a flattened {@link Expr}. There is some overhead to this, and these objects are all immutable,
    * so re-use instead of re-creating whenever possible.
+   *
    * @param in expression to parse
    * @param macroTable additional extensions to expression language
-   * @return
    */
   public static Expr parse(String in, ExprMacroTable macroTable)
   {
     return parse(in, macroTable, true);
   }
 
+
   @VisibleForTesting
   public static Expr parse(String in, ExprMacroTable macroTable, boolean withFlatten)
   {
@@ -164,47 +183,43 @@ public class Parser
   /**
    * Applies a transformation to an {@link Expr} given a list of known (or uknown) multi-value input columns that are
    * used in a scalar manner, walking the {@link Expr} tree and lifting array variables into the {@link LambdaExpr} of
-   * {@link ApplyFunctionExpr} and transforming the arguments of {@link FunctionExpr}
-   * @param expr expression to visit and rewrite
-   * @param bindingsToApply
-   * @return
+   * {@link ApplyFunctionExpr} and transforming the arguments of {@link FunctionExpr} as necessary.
+   *
+   * This function applies a transformation for "map" style uses, such as column selectors, where the supplied
+   * expression will be transformed to return an array of results instead of the scalar result (or appropriately
+   * rewritten into existing apply expressions to produce correct results when referenced from a scalar context).
+   *
+   * This function and {@link #foldUnappliedBindings(Expr, Expr.BindingAnalysis, List, String)} exist to handle
+   * "multi-valued" string dimensions, which exist in a superposition of both single and multi-valued during realtime
+   * ingestion, until they are written to a segment and become locked into either single or multi-valued. This also
+   * means that multi-valued-ness can vary for a column from segment to segment, so this family of transformation
+   * functions exist so that multi-valued strings can be expressed in either and array or scalar context, which is
+   * important because the writer of the query might not actually know if the column is definitively always single or
+   * multi-valued (and it might in fact not be).
+   *
+   * @see #foldUnappliedBindings(Expr, Expr.BindingAnalysis, List, String)
    */
-  public static Expr applyUnappliedBindings(Expr expr, Expr.BindingAnalysis bindingAnalysis, List<String> bindingsToApply)
+  public static Expr applyUnappliedBindings(
+      Expr expr,
+      Expr.BindingAnalysis bindingAnalysis,
+      List<String> bindingsToApply
+  )
   {
     if (bindingsToApply.isEmpty()) {
       // nothing to do, expression is fine as is
       return expr;
     }
     // filter the list of bindings to those which are used in this expression
-    List<String> unappliedBindingsInExpression = bindingsToApply.stream()
-                                     .filter(x -> bindingAnalysis.getRequiredBindings().contains(x))
-                                     .collect(Collectors.toList());
+    List<String> unappliedBindingsInExpression =
+        bindingsToApply.stream()
+                       .filter(x -> bindingAnalysis.getRequiredBindings().contains(x))
+                       .collect(Collectors.toList());
 
     // any unapplied bindings that are inside a lambda expression need that lambda expression to be rewritten
-    Expr newExpr = expr.visit(
-        childExpr -> {
-          if (childExpr instanceof ApplyFunctionExpr) {
-            // try to lift unapplied arguments into the apply function lambda
-            return liftApplyLambda((ApplyFunctionExpr) childExpr, unappliedBindingsInExpression);
-          } else if (childExpr instanceof FunctionExpr) {
-            // check array function arguments for unapplied identifiers to transform if necessary
-            FunctionExpr fnExpr = (FunctionExpr) childExpr;
-            Set<Expr> arrayInputs = fnExpr.function.getArrayInputs(fnExpr.args);
-            List<Expr> newArgs = new ArrayList<>();
-            for (Expr arg : fnExpr.args) {
-              if (arg.getIdentifierIfIdentifier() == null && arrayInputs.contains(arg)) {
-                Expr newArg = applyUnappliedBindings(arg, bindingAnalysis, unappliedBindingsInExpression);
-                newArgs.add(newArg);
-              } else {
-                newArgs.add(arg);
-              }
-            }
-
-            FunctionExpr newFnExpr = new FunctionExpr(fnExpr.function, fnExpr.function.name(), newArgs);
-            return newFnExpr;
-          }
-          return childExpr;
-        }
+    Expr newExpr = rewriteUnappliedSubExpressions(
+        expr,
+        unappliedBindingsInExpression,
+        (arg) -> applyUnappliedBindings(arg, bindingAnalysis, bindingsToApply)
     );
 
     Expr.BindingAnalysis newExprBindings = newExpr.analyzeInputs();
@@ -221,9 +236,123 @@ public class Parser
     return applyUnapplied(newExpr, remainingUnappliedBindings);
   }
 
+
+  /**
+   * Applies a transformation to an {@link Expr} given a list of known (or uknown) multi-value input columns that are
+   * used in a scalar manner, walking the {@link Expr} tree and lifting array variables into the {@link LambdaExpr} of
+   * {@link ApplyFunctionExpr} and transforming the arguments of {@link FunctionExpr} as necessary.
+   *
+   * This function applies a transformation for "fold" style uses, such as aggregators, where the supplied
+   * expression will be transformed to accumulate the result of applying the expression to each value of the unapplied
+   * input (or appropriately rewritten into existing apply expressions to produce correct results when referenced from
+   * a scalar context). This rewriting assumes that there exists some accumulator variable, which is re-used as the
+   * accumulator for this fold rewrite, so that evaluating each expression can be accumulated into the larger external
+   * fold operation that an aggregator might be performing.
+   *
+   * This function and {@link #applyUnappliedBindings(Expr, Expr.BindingAnalysis, List)} exist to handle
+   * "multi-valued" string dimensions, which exist in a superposition of both single and multi-valued during realtime
+   * ingestion, until they are written to a segment and become locked into either single or multi-valued. This also
+   * means that multi-valued-ness can vary for a column from segment to segment, so this family of transformation
+   * functions exist so that multi-valued strings can be expressed in either and array or scalar context, which is
+   * important because the writer of the query might not actually know if the column is definitively always single or
+   * multi-valued (and it might in fact not be).
+   *
+   * @see #applyUnappliedBindings(Expr, Expr.BindingAnalysis, List)
+   */
+  public static Expr foldUnappliedBindings(Expr expr, Expr.BindingAnalysis bindingAnalysis, List<String> bindingsToApply, String accumulatorId)
+  {
+    if (bindingsToApply.isEmpty()) {
+      // nothing to do, expression is fine as is
+      return expr;
+    }
+
+    // filter the list of bindings to those which are used in this expression
+    List<String> unappliedBindingsInExpression =
+        bindingsToApply.stream()
+                       .filter(x -> bindingAnalysis.getRequiredBindings().contains(x))
+                       .collect(Collectors.toList());
+
+    Expr newExpr = rewriteUnappliedSubExpressions(
+        expr,
+        unappliedBindingsInExpression,
+        (arg) -> foldUnappliedBindings(arg, bindingAnalysis, bindingsToApply, accumulatorId)
+    );
+
+    Expr.BindingAnalysis newExprBindings = newExpr.analyzeInputs();
+    final Set<String> expectedArrays = newExprBindings.getArrayVariables();
+
+    List<String> remainingUnappliedBindings =
+        unappliedBindingsInExpression.stream().filter(x -> !expectedArrays.contains(x)).collect(Collectors.toList());
+
+    // if lifting the lambdas got rid of all missing bindings, return the transformed expression
+    if (remainingUnappliedBindings.isEmpty()) {
+      return newExpr;
+    }
+
+    return foldUnapplied(newExpr, remainingUnappliedBindings, accumulatorId);
+  }
+
+  /**
+   * Any unapplied bindings that are inside a lambda expression need that lambda expression to be rewritten to "lift"
+   * the identifier variables and transform the function.
+   *
+   * For example:
+   * if "y" is unapplied:
+   *  map((x) -> x + y, x) => cartesian_map((x,y) -> x + y, x, y)
+   *
+   * @see #liftApplyLambda(ApplyFunctionExpr, List)
+   *
+   * Array functions on expressions using unapplied identifiers might also need transformed, so we recursively call the
+   * unapplied binding transformation function (supplied to this method) on that expression to ensure proper
+   * transformation and rewrite of these array expressions.
+   *
+   * For example:
+   * if "y" is unapplied:
+   *  array_length(filter((x) -> x > y, x))
+   */
+  private static Expr rewriteUnappliedSubExpressions(
+      Expr expr,
+      List<String> unappliedBindingsInExpression,
+      UnaryOperator<Expr> applyUnappliedFn
+  )
+  {
+    // any unapplied bindings that are inside a lambda expression need that lambda expression to be rewritten
+    return expr.visit(
+        childExpr -> {
+          if (childExpr instanceof ApplyFunctionExpr) {
+            // try to lift unapplied arguments into the apply function lambda
+            return liftApplyLambda((ApplyFunctionExpr) childExpr, unappliedBindingsInExpression);
+          } else if (childExpr instanceof FunctionExpr) {
+            // check array function arguments for unapplied identifiers to transform if necessary
+            FunctionExpr fnExpr = (FunctionExpr) childExpr;
+            Set<Expr> arrayInputs = fnExpr.function.getArrayInputs(fnExpr.args);
+            List<Expr> newArgs = new ArrayList<>();
+            for (Expr arg : fnExpr.args) {
+              if (arg.getIdentifierIfIdentifier() == null && arrayInputs.contains(arg)) {
+                Expr newArg = applyUnappliedFn.apply(arg);
+                newArgs.add(newArg);
+              } else {
+                newArgs.add(arg);
+              }
+            }
+
+            FunctionExpr newFnExpr = new FunctionExpr(fnExpr.function, fnExpr.function.name(), newArgs);
+            return newFnExpr;
+          }
+          return childExpr;
+        }
+    );
+  }
+
   /**
    * translate an {@link Expr} into an {@link ApplyFunctionExpr} for {@link ApplyFunction.MapFunction} or
-   * {@link ApplyFunction.CartesianMapFunction} if there are multiple unbound arguments to be applied
+   * {@link ApplyFunction.CartesianMapFunction} if there are multiple unbound arguments to be applied.
+   *
+   * For example:
+   * if "x" is unapplied:
+   *  x + y => map((x) -> x + y, x)
+   * if "x" and "y" are unapplied:
+   *  x + y => cartesian_map((x, y) -> x + y, x, y)
    */
   private static Expr applyUnapplied(Expr expr, List<String> unappliedBindings)
   {
@@ -274,6 +403,72 @@ public class Parser
     return magic;
   }
 
+  /**
+   * translate an {@link Expr} into an {@link ApplyFunctionExpr} for {@link ApplyFunction.FoldFunction} or
+   * {@link ApplyFunction.CartesianFoldFunction} if there are multiple unbound arguments to be applied.
+   *
+   * This assumes a known {@link IdentifierExpr} is an "accumulator", which is re-used as the accumulator variable and
+   * input for the translated fold.
+   *
+   * For example given an accumulator "__acc":
+   *  if "x" is unapplied:
+   *    __acc + x => fold((x, __acc) -> x + __acc, x, __acc)
+   *  if "x" and "y" are unapplied:
+   *    __acc + x + y => cartesian_fold((x, y, __acc) -> __acc + x + y, x, y, __acc)
+   *
+   */
+  private static Expr foldUnapplied(Expr expr, List<String> unappliedBindings, String accumulatorId)
+  {
+
+    // filter to get list of IdentifierExpr that are backed by the unapplied bindings
+    final List<IdentifierExpr> args = expr.analyzeInputs()
+                                          .getFreeVariables()
+                                          .stream()
+                                          .filter(x -> unappliedBindings.contains(x.getBinding()))
+                                          .collect(Collectors.toList());
+
+    final List<IdentifierExpr> lambdaArgs = new ArrayList<>();
+
+    // construct lambda args from list of args to apply. Identifiers in a lambda body have artificial 'binding' values
+    // that is the same as the 'identifier', because the bindings are supplied by the wrapping apply function
+    // replacements are done by binding rather than identifier because repeats of the same input should not result
+    // in a cartesian product
+    final Map<String, IdentifierExpr> toReplace = new HashMap<>();
+    for (IdentifierExpr applyFnArg : args) {
+      if (!toReplace.containsKey(applyFnArg.getBinding())) {
+        IdentifierExpr lambdaRewrite = new IdentifierExpr(applyFnArg.getBinding());
+        lambdaArgs.add(lambdaRewrite);
+        toReplace.put(applyFnArg.getBinding(), lambdaRewrite);
+      }
+    }
+
+    lambdaArgs.add(new IdentifierExpr(accumulatorId));
+
+    // rewrite identifiers in the expression which will become the lambda body, so they match the lambda identifiers we
+    // are constructing
+    Expr newExpr = expr.visit(childExpr -> {
+      if (childExpr instanceof IdentifierExpr) {
+        if (toReplace.containsKey(((IdentifierExpr) childExpr).getBinding())) {
+          return toReplace.get(((IdentifierExpr) childExpr).getBinding());
+        }
+      }
+      return childExpr;
+    });
+
+
+    // wrap an expression in either fold or cartesian_fold to apply any unapplied identifiers
+    final LambdaExpr lambdaExpr = new LambdaExpr(lambdaArgs, newExpr);
+    final ApplyFunction fn;
+    if (lambdaArgs.size() == 2) {
+      fn = new ApplyFunction.FoldFunction();
+    } else {
+      fn = new ApplyFunction.CartesianFoldFunction();
+    }
+
+    final Expr magic = new ApplyFunctionExpr(fn, fn.name(), lambdaExpr, ImmutableList.copyOf(lambdaArgs));
+    return magic;
+  }
+
   /**
    * Performs partial lifting of free identifiers of the lambda expression of an {@link ApplyFunctionExpr}, constrained
    * by a list of "unapplied" identifiers, and translating them into arguments of a new {@link LambdaExpr} and
@@ -406,23 +601,4 @@ public class Parser
     }
   }
 
-  /**
-   * Create {@link Expr.ObjectBinding} backed by {@link Map} to provide values for identifiers to evaluate {@link Expr}
-   */
-  public static Expr.ObjectBinding withMap(final Map<String, ?> bindings)
-  {
-    return bindings::get;
-  }
-
-  /**
-   * Create {@link Expr.ObjectBinding} backed by map of {@link Supplier} to provide values for identifiers to evaluate
-   * {@link Expr}
-   */
-  public static Expr.ObjectBinding withSuppliers(final Map<String, Supplier<Object>> bindings)
-  {
-    return (String name) -> {
-      Supplier<Object> supplier = bindings.get(name);
-      return supplier == null ? null : supplier.get();
-    };
-  }
 }
