@@ -21,7 +21,6 @@ package org.apache.druid.security.basic.authorization.db.updater;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
 import com.google.common.base.Preconditions;
-import com.google.common.collect.Lists;
 import com.google.inject.Inject;
 import org.apache.druid.common.config.ConfigManager;
 import org.apache.druid.concurrent.LifecycleLock;
@@ -52,12 +51,10 @@ import org.apache.druid.security.basic.authorization.entity.BasicAuthorizerUser;
 import org.apache.druid.security.basic.authorization.entity.BasicAuthorizerUserMapBundle;
 import org.apache.druid.security.basic.authorization.entity.GroupMappingAndRoleMap;
 import org.apache.druid.security.basic.authorization.entity.UserAndRoleMap;
-import org.apache.druid.server.security.Action;
+import org.apache.druid.server.security.AuthorizationUtils;
 import org.apache.druid.server.security.Authorizer;
 import org.apache.druid.server.security.AuthorizerMapper;
-import org.apache.druid.server.security.Resource;
 import org.apache.druid.server.security.ResourceAction;
-import org.apache.druid.server.security.ResourceType;
 import org.joda.time.Duration;
 
 import javax.annotation.Nonnull;
@@ -86,7 +83,7 @@ public class CoordinatorBasicAuthorizerMetadataStorageUpdater implements BasicAu
   private static final String GROUP_MAPPINGS = "groupMappings";
   private static final String ROLES = "roles";
 
-  public static final List<ResourceAction> SUPERUSER_PERMISSIONS = makeSuperUserPermissions();
+  public static final List<ResourceAction> SUPERUSER_PERMISSIONS = AuthorizationUtils.makeSuperUserPermissions();
 
   private final AuthorizerMapper authorizerMapper;
   private final MetadataStorageConnector connector;
@@ -143,42 +140,52 @@ public class CoordinatorBasicAuthorizerMetadataStorageUpdater implements BasicAu
 
     try {
       LOG.info("Starting CoordinatorBasicAuthorizerMetadataStorageUpdater");
-      for (Map.Entry<String, Authorizer> entry : authorizerMapper.getAuthorizerMap().entrySet()) {
-        Authorizer authorizer = entry.getValue();
-        if (authorizer instanceof BasicRoleBasedAuthorizer) {
-          BasicRoleBasedAuthorizer basicRoleBasedAuthorizer = (BasicRoleBasedAuthorizer) authorizer;
-          BasicAuthDBConfig dbConfig = basicRoleBasedAuthorizer.getDbConfig();
-          String authorizerName = entry.getKey();
-          authorizerNames.add(authorizerName);
+      BasicAuthUtils.maybeInitialize(
+          () -> {
+            for (Map.Entry<String, Authorizer> entry : authorizerMapper.getAuthorizerMap().entrySet()) {
+              Authorizer authorizer = entry.getValue();
+              if (authorizer instanceof BasicRoleBasedAuthorizer) {
+                BasicRoleBasedAuthorizer basicRoleBasedAuthorizer = (BasicRoleBasedAuthorizer) authorizer;
+                BasicAuthDBConfig dbConfig = basicRoleBasedAuthorizer.getDbConfig();
+                String authorizerName = entry.getKey();
+                authorizerNames.add(authorizerName);
 
-          byte[] userMapBytes = getCurrentUserMapBytes(authorizerName);
-          Map<String, BasicAuthorizerUser> userMap = BasicAuthUtils.deserializeAuthorizerUserMap(
-              objectMapper,
-              userMapBytes
-          );
-          cachedUserMaps.put(authorizerName, new BasicAuthorizerUserMapBundle(userMap, userMapBytes));
+                byte[] userMapBytes = getCurrentUserMapBytes(authorizerName);
+                Map<String, BasicAuthorizerUser> userMap = BasicAuthUtils.deserializeAuthorizerUserMap(
+                    objectMapper,
+                    userMapBytes
+                );
+                cachedUserMaps.put(authorizerName, new BasicAuthorizerUserMapBundle(userMap, userMapBytes));
 
-          byte[] groupMappingMapBytes = getCurrentGroupMappingMapBytes(authorizerName);
-          Map<String, BasicAuthorizerGroupMapping> groupMappingMap = BasicAuthUtils.deserializeAuthorizerGroupMappingMap(
-              objectMapper,
-              groupMappingMapBytes
-          );
-          cachedGroupMappingMaps.put(authorizerName, new BasicAuthorizerGroupMappingMapBundle(groupMappingMap, groupMappingMapBytes));
+                byte[] groupMappingMapBytes = getCurrentGroupMappingMapBytes(authorizerName);
+                Map<String, BasicAuthorizerGroupMapping> groupMappingMap = BasicAuthUtils.deserializeAuthorizerGroupMappingMap(
+                    objectMapper,
+                    groupMappingMapBytes
+                );
+                cachedGroupMappingMaps.put(
+                    authorizerName,
+                    new BasicAuthorizerGroupMappingMapBundle(
+                        groupMappingMap,
+                        groupMappingMapBytes
+                    )
+                );
 
-          byte[] roleMapBytes = getCurrentRoleMapBytes(authorizerName);
-          Map<String, BasicAuthorizerRole> roleMap = BasicAuthUtils.deserializeAuthorizerRoleMap(
-              objectMapper,
-              roleMapBytes
-          );
-          cachedRoleMaps.put(authorizerName, new BasicAuthorizerRoleMapBundle(roleMap, roleMapBytes));
+                byte[] roleMapBytes = getCurrentRoleMapBytes(authorizerName);
+                Map<String, BasicAuthorizerRole> roleMap = BasicAuthUtils.deserializeAuthorizerRoleMap(
+                    objectMapper,
+                    roleMapBytes
+                );
+                cachedRoleMaps.put(authorizerName, new BasicAuthorizerRoleMapBundle(roleMap, roleMapBytes));
 
-          initSuperUsersAndGroupMapping(authorizerName, userMap, roleMap, groupMappingMap,
-                                        dbConfig.getInitialAdminUser(),
-                                        dbConfig.getInitialAdminRole(),
-                                        dbConfig.getInitialAdminGroupMapping()
-          );
-        }
-      }
+                initSuperUsersAndGroupMapping(authorizerName, userMap, roleMap, groupMappingMap,
+                                              dbConfig.getInitialAdminUser(),
+                                              dbConfig.getInitialAdminRole(),
+                                              dbConfig.getInitialAdminGroupMapping()
+                );
+              }
+            }
+            return true;
+          });
 
       ScheduledExecutors.scheduleWithFixedDelay(
           exec,
@@ -1185,39 +1192,4 @@ public class CoordinatorBasicAuthorizerMetadataStorageUpdater implements BasicAu
       createGroupMappingInternal(authorizerName, groupMapping);
     }
   }
-
-  private static List<ResourceAction> makeSuperUserPermissions()
-  {
-    ResourceAction datasourceR = new ResourceAction(
-        new Resource(".*", ResourceType.DATASOURCE),
-        Action.READ
-    );
-
-    ResourceAction datasourceW = new ResourceAction(
-        new Resource(".*", ResourceType.DATASOURCE),
-        Action.WRITE
-    );
-
-    ResourceAction configR = new ResourceAction(
-        new Resource(".*", ResourceType.CONFIG),
-        Action.READ
-    );
-
-    ResourceAction configW = new ResourceAction(
-        new Resource(".*", ResourceType.CONFIG),
-        Action.WRITE
-    );
-
-    ResourceAction stateR = new ResourceAction(
-        new Resource(".*", ResourceType.STATE),
-        Action.READ
-    );
-
-    ResourceAction stateW = new ResourceAction(
-        new Resource(".*", ResourceType.STATE),
-        Action.WRITE
-    );
-
-    return Lists.newArrayList(datasourceR, datasourceW, configR, configW, stateR, stateW);
-  }
 }
