@@ -30,6 +30,8 @@ import com.google.api.services.storage.Storage;
 import com.google.common.collect.ImmutableList;
 import com.google.inject.Binder;
 import com.google.inject.Provides;
+import com.google.inject.multibindings.MapBinder;
+import org.apache.druid.data.SearchableVersionedDataFinder;
 import org.apache.druid.data.input.google.GoogleCloudStorageInputSource;
 import org.apache.druid.firehose.google.StaticGoogleBlobStoreFirehoseFactory;
 import org.apache.druid.guice.Binders;
@@ -42,7 +44,8 @@ import java.util.List;
 
 public class GoogleStorageDruidModule implements DruidModule
 {
-  static final String SCHEME = "google";
+  public static final String SCHEME = "google";
+  public static final String SCHEME_GS = "gs";
   private static final Logger LOG = new Logger(GoogleStorageDruidModule.class);
   private static final String APPLICATION_NAME = "druid-google-extensions";
 
@@ -94,6 +97,10 @@ public class GoogleStorageDruidModule implements DruidModule
     Binders.taskLogsBinder(binder).addBinding(SCHEME).to(GoogleTaskLogs.class);
     JsonConfigProvider.bind(binder, "druid.indexer.logs", GoogleTaskLogsConfig.class);
     binder.bind(GoogleTaskLogs.class).in(LazySingleton.class);
+    MapBinder.newMapBinder(binder, String.class, SearchableVersionedDataFinder.class)
+        .addBinding(SCHEME_GS)
+        .to(GoogleTimestampVersionedDataFinder.class)
+        .in(LazySingleton.class);
   }
 
   @Provides
