@@ -76,10 +76,9 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
-import java.util.Set;
-import java.util.SortedSet;
 
 /**
+ *
  */
 public class HadoopDruidIndexerConfig
 {
@@ -99,7 +98,7 @@ public class HadoopDruidIndexerConfig
   /**
    * Hadoop tasks running in an Indexer process need a reference to the Properties instance created
    * in PropertiesModule so that the task sees properties that were specified in Druid's config files.
-   *
+   * <p>
    * This is not strictly necessary for Peon-based tasks which have all properties, including config file properties,
    * specified on their command line by ForkingTaskRunner (so they could use System.getProperties() only),
    * but we always use the injected Properties for consistency.
@@ -314,9 +313,9 @@ public class HadoopDruidIndexerConfig
 
   public Optional<List<Interval>> getIntervals()
   {
-    Optional<SortedSet<Interval>> setOptional = schema.getDataSchema().getGranularitySpec().bucketIntervals();
-    if (setOptional.isPresent()) {
-      return Optional.of(JodaUtils.condenseIntervals(setOptional.get()));
+    Iterable<Interval> bucketIntervals = schema.getDataSchema().getGranularitySpec().sortedBucketIntervals();
+    if (bucketIntervals.iterator().hasNext()) {
+      return Optional.of(JodaUtils.condenseIntervals(bucketIntervals));
     } else {
       return Optional.absent();
     }
@@ -426,7 +425,6 @@ public class HadoopDruidIndexerConfig
    * Get the proper bucket for some input row.
    *
    * @param inputRow an InputRow
-   *
    * @return the Bucket that this row belongs to
    */
   Optional<Bucket> getBucket(InputRow inputRow)
@@ -455,14 +453,12 @@ public class HadoopDruidIndexerConfig
 
   }
 
-  Optional<Set<Interval>> getSegmentGranularIntervals()
+  Iterable<Interval> getSegmentGranularIntervals()
   {
-    return Optional.fromNullable(
+    return
         schema.getDataSchema()
               .getGranularitySpec()
-              .bucketIntervals()
-              .orNull()
-    );
+              .sortedBucketIntervals();
   }
 
   public List<Interval> getInputIntervals()
@@ -474,15 +470,17 @@ public class HadoopDruidIndexerConfig
 
   Optional<Iterable<Bucket>> getAllBuckets()
   {
-    Optional<Set<Interval>> intervals = getSegmentGranularIntervals();
-    if (intervals.isPresent()) {
+    Iterable<Interval> intervals = getSegmentGranularIntervals();
+    if (intervals.iterator().hasNext()) {
       return Optional.of(
           FunctionalIterable
-              .create(intervals.get())
+              .create(intervals)
               .transformCat(
                   input -> {
                     final DateTime bucketTime = input.getStart();
-                    final List<HadoopyShardSpec> specs = schema.getTuningConfig().getShardSpecs().get(bucketTime.getMillis());
+                    final List<HadoopyShardSpec> specs = schema.getTuningConfig()
+                                                               .getShardSpecs()
+                                                               .get(bucketTime.getMillis());
                     if (specs == null) {
                       return ImmutableList.of();
                     }
