@@ -22,6 +22,8 @@ package org.apache.druid.indexing.common.config;
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonProperty;
 import com.google.common.collect.ImmutableList;
+import org.apache.commons.lang3.EnumUtils;
+import org.apache.druid.java.util.common.logger.Logger;
 import org.apache.druid.segment.loading.StorageLocationConfig;
 import org.joda.time.Period;
 
@@ -31,12 +33,30 @@ import java.nio.file.Paths;
 import java.util.Collections;
 import java.util.List;
 
+/**
+ * Configurations for ingestion tasks. These configurations can be applied per middleManager, indexer, or overlord.
+ *
+ * See {@link org.apache.druid.indexing.overlord.config.DefaultTaskConfig} if you want to apply the same configuration
+ * to all tasks submitted to the overlord.
+ */
 public class TaskConfig
 {
+  private static final Logger log = new Logger(TaskConfig.class);
+
   public static final List<String> DEFAULT_DEFAULT_HADOOP_COORDINATES = ImmutableList.of(
       "org.apache.hadoop:hadoop-client:2.8.5"
   );
 
+  // This enum controls processing mode of batch ingestion "segment creation" phase (i.e. appenderator logic)
+  public enum BatchProcessingMode
+  {
+    OPEN_SEGMENTS, /* mmap segments, legacy code */
+    CLOSED_SEGMENTS, /* Do not mmap segments but keep most other legacy code */
+    CLOSED_SEGMENTS_SINKS /* Most aggressive memory optimization, do not mmap segments and eliminate sinks, etc. */
+  }
+
+  public static final BatchProcessingMode BATCH_PROCESSING_MODE_DEFAULT = BatchProcessingMode.CLOSED_SEGMENTS;
+
   private static final Period DEFAULT_DIRECTORY_LOCK_TIMEOUT = new Period("PT10M");
   private static final Period DEFAULT_GRACEFUL_SHUTDOWN_TIMEOUT = new Period("PT5M");
 
@@ -67,6 +87,15 @@ public class TaskConfig
   @JsonProperty
   private final List<StorageLocationConfig> shuffleDataLocations;
 
+  @JsonProperty
+  private final boolean ignoreTimestampSpecForDruidInputSource;
+
+  @JsonProperty
+  private final boolean batchMemoryMappedIndex;
+
+  @JsonProperty
+  private final BatchProcessingMode batchProcessingMode;
+
   @JsonCreator
   public TaskConfig(
       @JsonProperty("baseDir") String baseDir,
@@ -77,7 +106,10 @@ public class TaskConfig
       @JsonProperty("restoreTasksOnRestart") boolean restoreTasksOnRestart,
       @JsonProperty("gracefulShutdownTimeout") Period gracefulShutdownTimeout,
       @JsonProperty("directoryLockTimeout") Period directoryLockTimeout,
-      @JsonProperty("shuffleDataLocations") List<StorageLocationConfig> shuffleDataLocations
+      @JsonProperty("shuffleDataLocations") List<StorageLocationConfig> shuffleDataLocations,
+      @JsonProperty("ignoreTimestampSpecForDruidInputSource") boolean ignoreTimestampSpecForDruidInputSource,
+      @JsonProperty("batchMemoryMappedIndex") boolean batchMemoryMappedIndex, // deprecated, only set to true to fall back to older behavior
+      @JsonProperty("batchProcessingMode") String batchProcessingMode
   )
   {
     this.baseDir = baseDir == null ? System.getProperty("java.io.tmpdir") : baseDir;
@@ -102,6 +134,24 @@ public class TaskConfig
     } else {
       this.shuffleDataLocations = shuffleDataLocations;
     }
+    this.ignoreTimestampSpecForDruidInputSource = ignoreTimestampSpecForDruidInputSource;
+
+    this.batchMemoryMappedIndex = batchMemoryMappedIndex;
+    // Conflict resolution. Assume that if batchMemoryMappedIndex is set (since false is the default) that
+    // the user changed it intentionally to use legacy, in this case oveeride batchProcessingMode and also
+    // set it to legacy else just use batchProcessingMode and don't pay attention to batchMemoryMappedIndexMode:
+    if (batchMemoryMappedIndex) {
+      this.batchProcessingMode = BatchProcessingMode.OPEN_SEGMENTS;
+    } else if (EnumUtils.isValidEnum(BatchProcessingMode.class, batchProcessingMode)) {
+      this.batchProcessingMode = BatchProcessingMode.valueOf(batchProcessingMode);
+    } else {
+      // batchProcessingMode input string is invalid, log & use the default.
+      this.batchProcessingMode = BatchProcessingMode.CLOSED_SEGMENTS; // Default
+      log.warn("Batch processing mode argument value is null or not valid:[%s], defaulting to[%s] ",
+               batchProcessingMode, this.batchProcessingMode
+      );
+    }
+    log.info("Batch processing mode:[%s]", this.batchProcessingMode);
   }
 
   @JsonProperty
@@ -178,6 +228,29 @@ public class TaskConfig
     return shuffleDataLocations;
   }
 
+  @JsonProperty
+  public boolean isIgnoreTimestampSpecForDruidInputSource()
+  {
+    return ignoreTimestampSpecForDruidInputSource;
+  }
+
+  @JsonProperty
+  public BatchProcessingMode getBatchProcessingMode()
+  {
+    return batchProcessingMode;
+  }
+
+  /**
+   * Do not use in code! use {@link TaskConfig#getBatchProcessingMode() instead}
+   */
+  @Deprecated
+  @JsonProperty
+  public boolean getbatchMemoryMappedIndex()
+  {
+    return batchMemoryMappedIndex;
+  }
+
+
   private String defaultDir(@Nullable String configParameter, final String defaultVal)
   {
     if (configParameter == null) {
