@@ -22,10 +22,10 @@ package org.apache.druid.indexing.input;
 import com.fasterxml.jackson.annotation.JacksonInject;
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonInclude;
-import com.fasterxml.jackson.annotation.JsonInclude.Include;
 import com.fasterxml.jackson.annotation.JsonProperty;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.FluentIterable;
+import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Iterators;
 import org.apache.druid.client.coordinator.CoordinatorClient;
 import org.apache.druid.data.input.AbstractInputSource;
@@ -39,10 +39,11 @@ import org.apache.druid.data.input.SegmentsSplitHintSpec;
 import org.apache.druid.data.input.SplitHintSpec;
 import org.apache.druid.data.input.impl.InputEntityIteratingReader;
 import org.apache.druid.data.input.impl.SplittableInputSource;
-import org.apache.druid.indexing.common.ReingestionTimelineUtils;
+import org.apache.druid.data.input.impl.TimestampSpec;
 import org.apache.druid.indexing.common.RetryPolicy;
 import org.apache.druid.indexing.common.RetryPolicyFactory;
-import org.apache.druid.indexing.common.SegmentLoaderFactory;
+import org.apache.druid.indexing.common.SegmentCacheManagerFactory;
+import org.apache.druid.indexing.common.config.TaskConfig;
 import org.apache.druid.indexing.firehose.WindowedSegmentId;
 import org.apache.druid.java.util.common.IAE;
 import org.apache.druid.java.util.common.ISE;
@@ -50,7 +51,8 @@ import org.apache.druid.java.util.common.guava.Comparators;
 import org.apache.druid.java.util.common.logger.Logger;
 import org.apache.druid.query.filter.DimFilter;
 import org.apache.druid.segment.IndexIO;
-import org.apache.druid.segment.loading.SegmentLoader;
+import org.apache.druid.segment.column.ColumnHolder;
+import org.apache.druid.segment.loading.SegmentCacheManager;
 import org.apache.druid.timeline.DataSegment;
 import org.apache.druid.timeline.TimelineObjectHolder;
 import org.apache.druid.timeline.VersionedIntervalTimeline;
@@ -65,19 +67,57 @@ import java.io.File;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.SortedMap;
 import java.util.TreeMap;
 import java.util.concurrent.ThreadLocalRandom;
 import java.util.stream.Stream;
 
+/**
+ * An {@link org.apache.druid.data.input.InputSource} that allows reading from Druid segments.
+ *
+ * Used internally by {@link org.apache.druid.indexing.common.task.CompactionTask}, and can also be used directly.
+ */
+@JsonInclude(JsonInclude.Include.NON_NULL)
 public class DruidInputSource extends AbstractInputSource implements SplittableInputSource<List<WindowedSegmentId>>
 {
   private static final Logger LOG = new Logger(DruidInputSource.class);
 
+  /**
+   * Timestamp formats that the standard __time column can be parsed with.
+   */
+  private static final List<String> STANDARD_TIME_COLUMN_FORMATS = ImmutableList.of("millis", "auto");
+
+  /**
+   * A Comparator that orders {@link WindowedSegmentId} mainly by segmentId (which is important), and then by intervals
+   * (which is arbitrary, and only here for totality of ordering).
+   */
+  private static final Comparator<WindowedSegmentId> WINDOWED_SEGMENT_ID_COMPARATOR =
+      Comparator.comparing(WindowedSegmentId::getSegmentId)
+                .thenComparing(windowedSegmentId -> windowedSegmentId.getIntervals().size())
+                .thenComparing(
+                    (WindowedSegmentId a, WindowedSegmentId b) -> {
+                      // Same segmentId, same intervals list size. Compare each interval.
+                      int cmp = 0;
+
+                      for (int i = 0; i < a.getIntervals().size(); i++) {
+                        cmp = Comparators.intervalsByStartThenEnd()
+                                         .compare(a.getIntervals().get(i), b.getIntervals().get(i));
+
+                        if (cmp != 0) {
+                          return cmp;
+                        }
+                      }
+
+                      return cmp;
+                    }
+                );
+
   private final String dataSource;
   // Exactly one of interval and segmentIds should be non-null. Typically 'interval' is specified directly
   // by the user creating this firehose and 'segmentIds' is used for sub-tasks if it is split for parallel
@@ -87,12 +127,21 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
   @Nullable
   private final List<WindowedSegmentId> segmentIds;
   private final DimFilter dimFilter;
-  private final List<String> dimensions;
-  private final List<String> metrics;
   private final IndexIO indexIO;
   private final CoordinatorClient coordinatorClient;
-  private final SegmentLoaderFactory segmentLoaderFactory;
+  private final SegmentCacheManagerFactory segmentCacheManagerFactory;
   private final RetryPolicyFactory retryPolicyFactory;
+  private final TaskConfig taskConfig;
+
+  /**
+   * Included for serde backwards-compatibility only. Not used.
+   */
+  private final List<String> dimensions;
+
+  /**
+   * Included for serde backwards-compatibility only. Not used.
+   */
+  private final List<String> metrics;
 
   @JsonCreator
   public DruidInputSource(
@@ -106,8 +155,9 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
       @Nullable @JsonProperty("metrics") List<String> metrics,
       @JacksonInject IndexIO indexIO,
       @JacksonInject CoordinatorClient coordinatorClient,
-      @JacksonInject SegmentLoaderFactory segmentLoaderFactory,
-      @JacksonInject RetryPolicyFactory retryPolicyFactory
+      @JacksonInject SegmentCacheManagerFactory segmentCacheManagerFactory,
+      @JacksonInject RetryPolicyFactory retryPolicyFactory,
+      @JacksonInject TaskConfig taskConfig
   )
   {
     Preconditions.checkNotNull(dataSource, "dataSource");
@@ -122,8 +172,9 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
     this.metrics = metrics;
     this.indexIO = Preconditions.checkNotNull(indexIO, "null IndexIO");
     this.coordinatorClient = Preconditions.checkNotNull(coordinatorClient, "null CoordinatorClient");
-    this.segmentLoaderFactory = Preconditions.checkNotNull(segmentLoaderFactory, "null SegmentLoaderFactory");
+    this.segmentCacheManagerFactory = Preconditions.checkNotNull(segmentCacheManagerFactory, "null segmentCacheManagerFactory");
     this.retryPolicyFactory = Preconditions.checkNotNull(retryPolicyFactory, "null RetryPolicyFactory");
+    this.taskConfig = Preconditions.checkNotNull(taskConfig, "null taskConfig");
   }
 
   @JsonProperty
@@ -141,7 +192,6 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
 
   @Nullable
   @JsonProperty("segments")
-  @JsonInclude(Include.NON_NULL)
   public List<WindowedSegmentId> getSegmentIds()
   {
     return segmentIds;
@@ -153,12 +203,18 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
     return dimFilter;
   }
 
+  /**
+   * Included for serde backwards-compatibility only. Not used.
+   */
   @JsonProperty
   public List<String> getDimensions()
   {
     return dimensions;
   }
 
+  /**
+   * Included for serde backwards-compatibility only. Not used.
+   */
   @JsonProperty
   public List<String> getMetrics()
   {
@@ -168,7 +224,7 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
   @Override
   protected InputSourceReader fixedFormatReader(InputRowSchema inputRowSchema, @Nullable File temporaryDirectory)
   {
-    final SegmentLoader segmentLoader = segmentLoaderFactory.manufacturate(temporaryDirectory);
+    final SegmentCacheManager segmentCacheManager = segmentCacheManagerFactory.manufacturate(temporaryDirectory);
 
     final List<TimelineObjectHolder<String, DataSegment>> timeline = createTimeline();
     final Iterator<DruidSegmentInputEntity> entityIterator = FluentIterable
@@ -179,30 +235,40 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
           //noinspection ConstantConditions
           return FluentIterable
               .from(partitionHolder)
-              .transform(chunk -> new DruidSegmentInputEntity(segmentLoader, chunk.getObject(), holder.getInterval()));
+              .transform(chunk -> new DruidSegmentInputEntity(segmentCacheManager, chunk.getObject(), holder.getInterval()));
         }).iterator();
-    final List<String> effectiveDimensions = ReingestionTimelineUtils.getDimensionsToReingest(
-        dimensions,
-        inputRowSchema.getDimensionsSpec(),
-        timeline
-    );
 
-    List<String> effectiveMetrics;
-    if (metrics == null) {
-      effectiveMetrics = ReingestionTimelineUtils.getUniqueMetrics(timeline);
+    final DruidSegmentInputFormat inputFormat = new DruidSegmentInputFormat(indexIO, dimFilter);
+
+    final InputRowSchema inputRowSchemaToUse;
+
+    if (taskConfig.isIgnoreTimestampSpecForDruidInputSource()) {
+      // Legacy compatibility mode; see https://github.com/apache/druid/pull/10267.
+      LOG.warn("Ignoring the provided timestampSpec and reading the __time column instead. To use timestampSpecs with "
+               + "the 'druid' input source, set druid.indexer.task.ignoreTimestampSpecForDruidInputSource to false.");
+
+      inputRowSchemaToUse = new InputRowSchema(
+          new TimestampSpec(ColumnHolder.TIME_COLUMN_NAME, STANDARD_TIME_COLUMN_FORMATS.iterator().next(), null),
+          inputRowSchema.getDimensionsSpec(),
+          inputRowSchema.getColumnsFilter().plus(ColumnHolder.TIME_COLUMN_NAME)
+      );
     } else {
-      effectiveMetrics = metrics;
+      inputRowSchemaToUse = inputRowSchema;
     }
 
-    final DruidSegmentInputFormat inputFormat = new DruidSegmentInputFormat(
-        indexIO,
-        dimFilter,
-        effectiveDimensions,
-        effectiveMetrics
-    );
+    if (ColumnHolder.TIME_COLUMN_NAME.equals(inputRowSchemaToUse.getTimestampSpec().getTimestampColumn())
+        && !STANDARD_TIME_COLUMN_FORMATS.contains(inputRowSchemaToUse.getTimestampSpec().getTimestampFormat())) {
+      // Slight chance the user did this intentionally, but not likely. Log a warning.
+      LOG.warn(
+          "The provided timestampSpec refers to the %s column without using format %s. If you wanted to read the "
+          + "column as-is, switch formats.",
+          inputRowSchemaToUse.getTimestampSpec().getTimestampColumn(),
+          STANDARD_TIME_COLUMN_FORMATS
+      );
+    }
 
     return new InputEntityIteratingReader(
-        inputRowSchema,
+        inputRowSchemaToUse,
         inputFormat,
         entityIterator,
         temporaryDirectory
@@ -273,8 +339,9 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
         metrics,
         indexIO,
         coordinatorClient,
-        segmentLoaderFactory,
-        retryPolicyFactory
+        segmentCacheManagerFactory,
+        retryPolicyFactory,
+        taskConfig
     );
   }
 
@@ -284,6 +351,43 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
     return false;
   }
 
+  @Override
+  public boolean equals(Object o)
+  {
+    if (this == o) {
+      return true;
+    }
+    if (o == null || getClass() != o.getClass()) {
+      return false;
+    }
+    DruidInputSource that = (DruidInputSource) o;
+    return Objects.equals(dataSource, that.dataSource)
+           && Objects.equals(interval, that.interval)
+           && Objects.equals(segmentIds, that.segmentIds)
+           && Objects.equals(dimFilter, that.dimFilter)
+           && Objects.equals(dimensions, that.dimensions)
+           && Objects.equals(metrics, that.metrics);
+  }
+
+  @Override
+  public int hashCode()
+  {
+    return Objects.hash(dataSource, interval, segmentIds, dimFilter, dimensions, metrics);
+  }
+
+  @Override
+  public String toString()
+  {
+    return "DruidInputSource{" +
+           "dataSource='" + dataSource + '\'' +
+           ", interval=" + interval +
+           ", segmentIds=" + segmentIds +
+           ", dimFilter=" + dimFilter +
+           (dimensions != null ? ", dimensions=" + dimensions : "") +
+           (metrics != null ? ", metrics=" + metrics : "") +
+           '}';
+  }
+
   public static Iterator<InputSplit<List<WindowedSegmentId>>> createSplits(
       CoordinatorClient coordinatorClient,
       RetryPolicyFactory retryPolicyFactory,
@@ -313,6 +417,8 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
     //noinspection ConstantConditions
     return Iterators.transform(
         convertedSplitHintSpec.split(
+            // segmentIdToSize is sorted by segment ID; useful for grouping up segments from the same time chunk into
+            // the same input split.
             segmentIdToSize.keySet().iterator(),
             segmentId -> new InputFileAttribute(
                 Preconditions.checkNotNull(segmentIdToSize.get(segmentId), "segment size for [%s]", segmentId)
@@ -322,7 +428,10 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
     );
   }
 
-  private static Map<WindowedSegmentId, Long> createWindowedSegmentIdFromTimeline(
+  /**
+   * Returns a map of {@link WindowedSegmentId} to size, sorted by {@link WindowedSegmentId#getSegmentId()}.
+   */
+  private static SortedMap<WindowedSegmentId, Long> createWindowedSegmentIdFromTimeline(
       List<TimelineObjectHolder<String, DataSegment>> timelineHolders
   )
   {
@@ -335,9 +444,9 @@ public class DruidInputSource extends AbstractInputSource implements SplittableI
         ).addInterval(holder.getInterval());
       }
     }
-    // It is important to create this map after windowedSegmentIds is completely filled
-    // because WindowedSegmentId can be updated.
-    Map<WindowedSegmentId, Long> segmentSizeMap = new HashMap<>();
+    // It is important to create this map after windowedSegmentIds is completely filled, because WindowedSegmentIds
+    // can be updated while being constructed. (Intervals are added.)
+    SortedMap<WindowedSegmentId, Long> segmentSizeMap = new TreeMap<>(WINDOWED_SEGMENT_ID_COMPARATOR);
     windowedSegmentIds.forEach((segment, segmentId) -> segmentSizeMap.put(segmentId, segment.getSize()));
     return segmentSizeMap;
   }
