@@ -674,6 +674,67 @@ public class TaskLockbox
     }
   }
 
+  /**
+   * Gets a List of Intervals locked by higher priority tasks for each datasource.
+   * Here, Segment Locks are being treated the same as Time Chunk Locks i.e.
+   * a Task with a Segment Lock is assumed to lock a whole Interval and not just
+   * the corresponding Segment.
+   *
+   * @param minTaskPriority Minimum task priority for each datasource. Only the
+   *                        Intervals that are locked by Tasks with equal or
+   *                        higher priority than this are returned. Locked intervals
+   *                        for datasources that are not present in this Map are
+   *                        not returned.
+   * @return Map from Datasource to List of Intervals locked by Tasks that have
+   * priority greater than or equal to the {@code minTaskPriority} for that datasource.
+   */
+  public Map<String, List<Interval>> getLockedIntervals(Map<String, Integer> minTaskPriority)
+  {
+    final Map<String, Set<Interval>> datasourceToIntervals = new HashMap<>();
+
+    // Take a lock and populate the maps
+    giant.lock();
+    try {
+      running.forEach(
+          (datasource, datasourceLocks) -> {
+            // If this datasource is not requested, do not proceed
+            if (!minTaskPriority.containsKey(datasource)) {
+              return;
+            }
+
+            datasourceLocks.forEach(
+                (startTime, startTimeLocks) -> startTimeLocks.forEach(
+                    (interval, taskLockPosses) -> taskLockPosses.forEach(
+                        taskLockPosse -> {
+                          if (taskLockPosse.getTaskLock().isRevoked()) {
+                            // Do not proceed if the lock is revoked
+                            return;
+                          } else if (taskLockPosse.getTaskLock().getPriority() == null
+                                     || taskLockPosse.getTaskLock().getPriority() < minTaskPriority.get(datasource)) {
+                            // Do not proceed if the lock has a priority strictly less than the minimum
+                            return;
+                          }
+
+                          datasourceToIntervals
+                              .computeIfAbsent(datasource, k -> new HashSet<>())
+                              .add(interval);
+                        })
+                )
+            );
+          }
+      );
+    }
+    finally {
+      giant.unlock();
+    }
+
+    return datasourceToIntervals.entrySet().stream()
+                                .collect(Collectors.toMap(
+                                    Map.Entry::getKey,
+                                    entry -> new ArrayList<>(entry.getValue())
+                                ));
+  }
+
   public void unlock(final Task task, final Interval interval)
   {
     unlock(task, interval, null);
@@ -775,6 +836,25 @@ public class TaskLockbox
     }
   }
 
+  public void unlockAll(Task task)
+  {
+    giant.lock();
+    try {
+      for (final TaskLockPosse taskLockPosse : findLockPossesForTask(task)) {
+        unlock(
+            task,
+            taskLockPosse.getTaskLock().getInterval(),
+            taskLockPosse.getTaskLock().getGranularity() == LockGranularity.SEGMENT
+            ? ((SegmentLock) taskLockPosse.taskLock).getPartitionId()
+            : null
+        );
+      }
+    }
+    finally {
+      giant.unlock();
+    }
+  }
+
   public void add(Task task)
   {
     giant.lock();
@@ -798,15 +878,7 @@ public class TaskLockbox
     try {
       try {
         log.info("Removing task[%s] from activeTasks", task.getId());
-        for (final TaskLockPosse taskLockPosse : findLockPossesForTask(task)) {
-          unlock(
-              task,
-              taskLockPosse.getTaskLock().getInterval(),
-              taskLockPosse.getTaskLock().getGranularity() == LockGranularity.SEGMENT
-              ? ((SegmentLock) taskLockPosse.taskLock).getPartitionId()
-              : null
-          );
-        }
+        unlockAll(task);
       }
       finally {
         activeTasks.remove(task.getId());
