@@ -245,13 +245,15 @@ public class TaskSerdeTest
                 ),
                 null
             ),
-            new IndexIOConfig(null, new LocalInputSource(new File("lol"), "rofl"), new NoopInputFormat(), true),
+            new IndexIOConfig(null, new LocalInputSource(new File("lol"), "rofl"), new NoopInputFormat(), true, false),
             new IndexTuningConfig(
+                null,
                 null,
                 null,
                 10,
                 null,
                 null,
+                null,
                 9999,
                 null,
                 null,
@@ -266,7 +268,9 @@ public class TaskSerdeTest
                 null,
                 null,
                 null,
-                null
+                null,
+                null,
+                1L
             )
         ),
         null
@@ -304,6 +308,7 @@ public class TaskSerdeTest
     Assert.assertEquals(taskTuningConfig.getNumShards(), task2TuningConfig.getNumShards());
     Assert.assertEquals(taskTuningConfig.getMaxRowsPerSegment(), task2TuningConfig.getMaxRowsPerSegment());
     Assert.assertEquals(taskTuningConfig.isReportParseExceptions(), task2TuningConfig.isReportParseExceptions());
+    Assert.assertEquals(taskTuningConfig.getAwaitSegmentAvailabilityTimeoutMillis(), task2TuningConfig.getAwaitSegmentAvailabilityTimeoutMillis());
   }
 
   @Test
@@ -325,8 +330,9 @@ public class TaskSerdeTest
                 ),
                 null
             ),
-            new IndexIOConfig(null, new LocalInputSource(new File("lol"), "rofl"), new NoopInputFormat(), true),
+            new IndexIOConfig(null, new LocalInputSource(new File("lol"), "rofl"), new NoopInputFormat(), true, false),
             new IndexTuningConfig(
+                null,
                 null,
                 null,
                 10,
@@ -335,6 +341,7 @@ public class TaskSerdeTest
                 null,
                 null,
                 null,
+                null,
                 new DynamicPartitionsSpec(10000, null),
                 indexSpec,
                 null,
@@ -346,6 +353,8 @@ public class TaskSerdeTest
                 null,
                 null,
                 null,
+                null,
+                null,
                 null
             )
         ),
@@ -396,8 +405,10 @@ public class TaskSerdeTest
             ),
 
             new RealtimeTuningConfig(
+                null,
                 1,
                 10L,
+                null,
                 new Period("PT10M"),
                 null,
                 null,
@@ -407,7 +418,6 @@ public class TaskSerdeTest
                 NoneShardSpec.instance(),
                 indexSpec,
                 null,
-                null,
                 0,
                 0,
                 true,
