@@ -108,6 +108,12 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
 
   private TestSupervisorTask task;
 
+  public ParallelIndexSupervisorTaskResourceTest()
+  {
+    // We don't need to emulate transient failures for this test.
+    super(0.0, 0.0);
+  }
+
   @After
   public void teardown()
   {
@@ -123,7 +129,8 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
             null,
             new TestInputSource(IntStream.range(0, NUM_SUB_TASKS).boxed().collect(Collectors.toList())),
             new NoopInputFormat(),
-            false
+            false,
+            null
         )
     );
     getIndexingServiceClient().runTask(task.getId(), task);
@@ -427,6 +434,8 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
             null,
             null,
             null,
+            null,
+            null,
             NUM_SUB_TASKS,
             null,
             null,
@@ -436,6 +445,8 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
             null,
             null,
             null,
+            null,
+            null,
             null
         )
     );
@@ -545,7 +556,8 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
                   null,
                   baseInputSource.withSplit(split),
                   getIngestionSchema().getIOConfig().getInputFormat(),
-                  getIngestionSchema().getIOConfig().isAppendToExisting()
+                  getIngestionSchema().getIOConfig().isAppendToExisting(),
+                  null
               ),
               getIngestionSchema().getTuningConfig()
           ),
@@ -584,6 +596,7 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
       final TestSubTask subTask = new TestSubTask(
           getGroupId(),
           getSupervisorTaskId(),
+          getId(),
           numAttempts,
           getIngestionSpec(),
           getContext()
@@ -627,6 +640,7 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
     TestSubTask(
         String groupId,
         String supervisorTaskId,
+        String subtaskSpecId,
         int numAttempts,
         ParallelIndexIngestionSpec ingestionSchema,
         Map<String, Object> context
@@ -637,6 +651,7 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
           groupId,
           null,
           supervisorTaskId,
+          subtaskSpecId,
           numAttempts,
           ingestionSchema,
           context
@@ -663,19 +678,20 @@ public class ParallelIndexSupervisorTaskResourceTest extends AbstractParallelInd
           .getGivenOrDefaultPartitionsSpec();
       final SegmentAllocator segmentAllocator = SegmentAllocators.forLinearPartitioning(
           toolbox,
-          getId(),
+          getSubtaskSpecId(),
           new SupervisorTaskAccess(getSupervisorTaskId(), taskClient),
           getIngestionSchema().getDataSchema(),
           getTaskLockHelper(),
           getIngestionSchema().getIOConfig().isAppendToExisting(),
-          partitionsSpec
+          partitionsSpec,
+          true
       );
 
       final SegmentIdWithShardSpec segmentIdentifier = segmentAllocator.allocate(
           new MapBasedInputRow(DateTimes.of("2017-01-01"), Collections.emptyList(), Collections.emptyMap()),
-          getId(),
+          getSubtaskSpecId(),
           null,
-          true
+          false
       );
 
       final DataSegment segment = new DataSegment(
