@@ -53,6 +53,7 @@ import org.joda.time.Interval;
 import javax.annotation.Nullable;
 import java.io.File;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
@@ -66,6 +67,7 @@ class ParallelIndexTestingFactory
   static final String GROUP_ID = "group-id";
   static final TaskResource TASK_RESOURCE = null;
   static final String SUPERVISOR_TASK_ID = "supervisor-task-id";
+  static final String SUBTASK_SPEC_ID = "subtask-spec-id";
   static final int NUM_ATTEMPTS = 1;
   static final Map<String, Object> CONTEXT = Collections.emptyMap();
   static final IndexingServiceClient INDEXING_SERVICE_CLIENT = TestUtils.INDEXING_SERVICE_CLIENT;
@@ -75,10 +77,10 @@ class ParallelIndexTestingFactory
   static final ShuffleClient SHUFFLE_CLIENT = new ShuffleClient()
   {
     @Override
-    public <T, P extends PartitionLocation<T>> File fetchSegmentFile(
+    public File fetchSegmentFile(
         File partitionDir,
         String supervisorTaskId,
-        P location
+        PartitionLocation location
     )
     {
       return null;
@@ -158,8 +160,10 @@ class ParallelIndexTestingFactory
       return new ParallelIndexTuningConfig(
           1,
           null,
+          null,
           3,
           4L,
+          null,
           5L,
           6,
           null,
@@ -181,7 +185,9 @@ class ParallelIndexTestingFactory
           22,
           logParseExceptions,
           maxParseExceptions,
-          25
+          25,
+          null,
+          null
       );
     }
   }
@@ -215,7 +221,7 @@ class ParallelIndexTestingFactory
       DataSchema dataSchema
   )
   {
-    ParallelIndexIOConfig ioConfig = new ParallelIndexIOConfig(null, inputSource, inputFormat, false);
+    ParallelIndexIOConfig ioConfig = new ParallelIndexIOConfig(null, inputSource, inputFormat, false, false);
 
     return new ParallelIndexIngestionSpec(dataSchema, ioConfig, tuningConfig);
   }
@@ -274,6 +280,18 @@ class ParallelIndexTestingFactory
     }
   }
 
+  static String createRowFromMap(long timestamp, Map<String, Object> fields)
+  {
+    HashMap<String, Object> row = new HashMap<>(fields);
+    row.put(SCHEMA_TIME, timestamp);
+    try {
+      return NESTED_OBJECT_MAPPER.writeValueAsString(row);
+    }
+    catch (JsonProcessingException e) {
+      throw new RuntimeException(e);
+    }
+  }
+
   static InputFormat getInputFormat()
   {
     return new JsonInputFormat(null, null, null);
