@@ -111,19 +111,6 @@ public class PartialDimensionDistributionTaskTest
           .build();
     }
 
-    @Test
-    public void requiresGranularitySpecInputIntervals()
-    {
-      exception.expect(IllegalArgumentException.class);
-      exception.expectMessage("Missing intervals in granularitySpec");
-
-      DataSchema dataSchema = ParallelIndexTestingFactory.createDataSchema(Collections.emptyList());
-
-      new PartialDimensionDistributionTaskBuilder()
-          .dataSchema(dataSchema)
-          .build();
-    }
-
     @Test
     public void serializesDeserializes()
     {
@@ -484,6 +471,7 @@ public class PartialDimensionDistributionTaskTest
           ParallelIndexTestingFactory.GROUP_ID,
           ParallelIndexTestingFactory.TASK_RESOURCE,
           ParallelIndexTestingFactory.SUPERVISOR_TASK_ID,
+          ParallelIndexTestingFactory.SUBTASK_SPEC_ID,
           ParallelIndexTestingFactory.NUM_ATTEMPTS,
           ingestionSpec,
           ParallelIndexTestingFactory.CONTEXT,
