@@ -20,18 +20,30 @@
 package org.apache.druid.indexing.overlord.config;
 
 import com.fasterxml.jackson.databind.ObjectMapper;
+import com.google.common.collect.ImmutableList;
+import com.google.inject.Injector;
+import com.google.inject.ProvisionException;
+import org.apache.druid.guice.GuiceInjectors;
+import org.apache.druid.guice.IndexingServiceModuleHelper;
+import org.apache.druid.guice.JsonConfigProvider;
 import org.apache.druid.guice.JsonConfigurator;
 import org.apache.druid.jackson.DefaultObjectMapper;
 import org.joda.time.Period;
 import org.junit.Assert;
+import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.ExpectedException;
 
 import java.io.IOException;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.Properties;
 
 public class RemoteTaskRunnerConfigTest
 {
+  @Rule
+  public ExpectedException expectedException = ExpectedException.none();
+
   private static final ObjectMapper MAPPER = new DefaultObjectMapper();
   private static final Period DEFAULT_TIMEOUT = Period.ZERO;
   private static final String DEFAULT_VERSION = "";
@@ -793,6 +805,49 @@ public class RemoteTaskRunnerConfigTest
     );
   }
 
+  @Test
+  public void testMaxZnodeBytesLowerThanExpected()
+  {
+    final Injector injector = GuiceInjectors.makeStartupInjectorWithModules(ImmutableList.of(
+        binder -> IndexingServiceModuleHelper.configureTaskRunnerConfigs(binder))
+    );
+
+    this.expectedException.expect(ProvisionException.class);
+    this.expectedException.expectMessage("maxZnodeBytes must be in the range of [10KiB, 2GiB)");
+
+    Properties props = new Properties();
+    props.put(IndexingServiceModuleHelper.INDEXER_RUNNER_PROPERTY_PREFIX + ".maxZnodeBytes", "9KiB");
+
+    JsonConfigProvider<RemoteTaskRunnerConfig> configProvider = JsonConfigProvider.of(
+        IndexingServiceModuleHelper.INDEXER_RUNNER_PROPERTY_PREFIX,
+        RemoteTaskRunnerConfig.class
+    );
+    configProvider.inject(props, injector.getBinding(JsonConfigurator.class).getProvider().get());
+    configProvider.get().get();
+  }
+
+  @Test
+  public void testMaxZnodeBytesGreaterThanExpected()
+  {
+    final Injector injector = GuiceInjectors.makeStartupInjectorWithModules(ImmutableList.of(
+        binder -> IndexingServiceModuleHelper.configureTaskRunnerConfigs(binder))
+    );
+
+    this.expectedException.expect(ProvisionException.class);
+    this.expectedException.expectMessage("maxZnodeBytes must be in the range of [10KiB, 2GiB)");
+
+    Properties props = new Properties();
+    props.put(IndexingServiceModuleHelper.INDEXER_RUNNER_PROPERTY_PREFIX + ".maxZnodeBytes", "2GiB");
+
+    JsonConfigProvider<RemoteTaskRunnerConfig> configProvider = JsonConfigProvider.of(
+        IndexingServiceModuleHelper.INDEXER_RUNNER_PROPERTY_PREFIX,
+        RemoteTaskRunnerConfig.class
+    );
+    configProvider.inject(props, injector.getBinding(JsonConfigurator.class).getProvider().get());
+    configProvider.get().get();
+  }
+
+
   private RemoteTaskRunnerConfig reflect(RemoteTaskRunnerConfig config) throws IOException
   {
     return MAPPER.readValue(MAPPER.writeValueAsString(config), RemoteTaskRunnerConfig.class);
