@@ -26,6 +26,7 @@ import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
 import org.apache.druid.indexing.overlord.DataSourceMetadata;
 import org.apache.druid.indexing.overlord.TaskMaster;
+import org.apache.druid.indexing.overlord.supervisor.autoscaler.SupervisorTaskAutoScaler;
 import org.apache.druid.java.util.common.DateTimes;
 import org.apache.druid.segment.TestHelper;
 import org.apache.druid.server.security.Access;
@@ -1155,6 +1156,12 @@ public class SupervisorResourceTest extends EasyMockSupport
       return supervisor;
     }
 
+    @Override
+    public SupervisorTaskAutoScaler createAutoscaler(Supervisor supervisor)
+    {
+      return null;
+    }
+
     @Override
     public List<String> getDataSources()
     {
