@@ -21,6 +21,7 @@ package org.apache.druid.indexing.seekablestream;
 
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.ObjectMapper;
+import com.fasterxml.jackson.databind.SerializationFeature;
 import com.fasterxml.jackson.databind.jsontype.NamedType;
 import com.google.common.base.Predicates;
 import com.google.common.base.Throwables;
@@ -32,6 +33,7 @@ import com.google.common.util.concurrent.ListenableFuture;
 import com.google.common.util.concurrent.ListeningExecutorService;
 import org.apache.druid.common.config.NullHandling;
 import org.apache.druid.data.input.InputFormat;
+import org.apache.druid.data.input.impl.ByteEntity;
 import org.apache.druid.data.input.impl.DimensionsSpec;
 import org.apache.druid.data.input.impl.FloatDimensionSchema;
 import org.apache.druid.data.input.impl.JSONParseSpec;
@@ -77,7 +79,7 @@ import org.apache.druid.segment.QueryableIndex;
 import org.apache.druid.segment.column.DictionaryEncodedColumn;
 import org.apache.druid.segment.indexing.DataSchema;
 import org.apache.druid.segment.indexing.granularity.UniformGranularitySpec;
-import org.apache.druid.segment.realtime.appenderator.AppenderatorImpl;
+import org.apache.druid.segment.realtime.appenderator.StreamAppenderator;
 import org.apache.druid.timeline.DataSegment;
 import org.apache.druid.utils.CompressionUtils;
 import org.assertj.core.api.Assertions;
@@ -190,7 +192,7 @@ public class SeekableStreamIndexTaskTestBase extends EasyMockSupport
     this.lockGranularity = lockGranularity;
   }
 
-  protected static byte[] jb(
+  protected static ByteEntity jb(
       String timestamp,
       String dim1,
       String dim2,
@@ -198,9 +200,82 @@ public class SeekableStreamIndexTaskTestBase extends EasyMockSupport
       String dimFloat,
       String met1
   )
+  {
+    return jb(false, timestamp, dim1, dim2, dimLong, dimFloat, met1);
+  }
+
+  protected static byte[] jbb(
+      String timestamp,
+      String dim1,
+      String dim2,
+      String dimLong,
+      String dimFloat,
+      String met1
+  )
+  {
+    return jbb(false, timestamp, dim1, dim2, dimLong, dimFloat, met1);
+  }
+
+  protected static ByteEntity jb(boolean prettyPrint,
+      String timestamp,
+      String dim1,
+      String dim2,
+      String dimLong,
+      String dimFloat,
+      String met1
+  )
+  {
+    return new ByteEntity(jbb(prettyPrint, timestamp, dim1, dim2, dimLong, dimFloat, met1));
+  }
+
+  protected static byte[] jbb(
+      boolean prettyPrint,
+      String timestamp,
+      String dim1,
+      String dim2,
+      String dimLong,
+      String dimFloat,
+      String met1
+  )
+  {
+    return StringUtils.toUtf8(toJsonString(
+        prettyPrint,
+        timestamp,
+        dim1,
+        dim2,
+        dimLong,
+        dimFloat,
+        met1
+    ));
+  }
+
+  protected static List<ByteEntity> jbl(
+      String timestamp,
+      String dim1,
+      String dim2,
+      String dimLong,
+      String dimFloat,
+      String met1
+  )
+  {
+    return Collections.singletonList(jb(timestamp, dim1, dim2, dimLong, dimFloat, met1));
+  }
+
+  protected static String toJsonString(boolean prettyPrint,
+                             String timestamp,
+                             String dim1,
+                             String dim2,
+                             String dimLong,
+                             String dimFloat,
+                             String met1
+  )
   {
     try {
-      return new ObjectMapper().writeValueAsBytes(
+      ObjectMapper mapper = new ObjectMapper();
+      if (prettyPrint) {
+        mapper.enable(SerializationFeature.INDENT_OUTPUT);
+      }
+      return mapper.writeValueAsString(
           ImmutableMap.builder()
                       .put("timestamp", timestamp)
                       .put("dim1", dim1)
@@ -216,18 +291,6 @@ public class SeekableStreamIndexTaskTestBase extends EasyMockSupport
     }
   }
 
-  protected static List<byte[]> jbl(
-      String timestamp,
-      String dim1,
-      String dim2,
-      String dimLong,
-      String dimFloat,
-      String met1
-  )
-  {
-    return Collections.singletonList(jb(timestamp, dim1, dim2, dimLong, dimFloat, met1));
-  }
-
   protected File getSegmentDirectory()
   {
     return new File(directory, "segments");
@@ -398,7 +461,7 @@ public class SeekableStreamIndexTaskTestBase extends EasyMockSupport
   protected void unlockAppenderatorBasePersistDirForTask(SeekableStreamIndexTask task)
       throws NoSuchMethodException, InvocationTargetException, IllegalAccessException
   {
-    Method unlockBasePersistDir = ((AppenderatorImpl) task.getAppenderator())
+    Method unlockBasePersistDir = ((StreamAppenderator) task.getAppenderator())
         .getClass()
         .getDeclaredMethod("unlockBasePersistDirectory");
     unlockBasePersistDir.setAccessible(true);
