@@ -30,6 +30,7 @@ import org.apache.druid.query.aggregation.PostAggregator;
 import org.apache.druid.query.aggregation.post.ArithmeticPostAggregator;
 import org.apache.druid.query.aggregation.post.ConstantPostAggregator;
 import org.apache.druid.query.aggregation.post.FieldAccessPostAggregator;
+import org.apache.druid.query.filter.TrueDimFilter;
 import org.apache.druid.query.spec.MultipleSpecificSegmentSpec;
 import org.apache.druid.query.timeseries.TimeseriesQuery;
 import org.apache.druid.query.timeseries.TimeseriesResultValue;
@@ -423,6 +424,7 @@ public class QueriesTest
                                                   "j0.",
                                                   "\"foo.x\" == \"bar.x\"",
                                                   JoinType.INNER,
+                                                  null,
                                                   ExprMacroTable.nil()
                                               )
                                           )
@@ -459,6 +461,7 @@ public class QueriesTest
                                                       "j0.",
                                                       "\"foo.x\" == \"bar.x\"",
                                                       JoinType.INNER,
+                                                      null,
                                                       ExprMacroTable.nil()
                                                   )
                                               )
@@ -479,4 +482,78 @@ public class QueriesTest
         )
     );
   }
+
+  @Test
+  public void testWithBaseDataSourcedBaseFilterWithMultiJoin()
+  {
+    Assert.assertEquals(
+        Druids.newTimeseriesQueryBuilder()
+              .dataSource(
+                  new QueryDataSource(
+                      Druids.newTimeseriesQueryBuilder()
+                            .dataSource(
+                                JoinDataSource.create(
+                                    JoinDataSource.create(
+                                        new TableDataSource("foo"),
+                                        new TableDataSource("bar"),
+                                        "j1.",
+                                        "\"foo.x\" == \"bar.x\"",
+                                        JoinType.INNER,
+                                        TrueDimFilter.instance(),
+                                        ExprMacroTable.nil()
+                                    ),
+                                    new TableDataSource("foo_outer"),
+                                    "j0.",
+                                    "\"foo_outer.x\" == \"bar.x\"",
+                                    JoinType.INNER,
+                                    null,
+                                    ExprMacroTable.nil()
+                                )
+
+                            )
+                            .intervals("2000/3000")
+                            .granularity(Granularities.ALL)
+                            .build()
+                  )
+              )
+              .intervals("2000/3000")
+              .granularity(Granularities.ALL)
+              .build(),
+        Queries.withBaseDataSource(
+            Druids.newTimeseriesQueryBuilder()
+                  .dataSource(
+                      new QueryDataSource(
+                          Druids.newTimeseriesQueryBuilder()
+                                .dataSource(
+                                    JoinDataSource.create(
+                                        JoinDataSource.create(
+                                            new TableDataSource("foo_inner"),
+                                            new TableDataSource("bar"),
+                                            "j1.",
+                                            "\"foo.x\" == \"bar.x\"",
+                                            JoinType.INNER,
+                                            TrueDimFilter.instance(),
+                                            ExprMacroTable.nil()
+                                        ),
+                                        new TableDataSource("foo_outer"),
+                                        "j0.",
+                                        "\"foo_outer.x\" == \"bar.x\"",
+                                        JoinType.INNER,
+                                        null,
+                                        ExprMacroTable.nil()
+                                    )
+
+                                )
+                                .intervals("2000/3000")
+                                .granularity(Granularities.ALL)
+                                .build()
+                      )
+                  )
+                  .intervals("2000/3000")
+                  .granularity(Granularities.ALL)
+                  .build(),
+            new TableDataSource("foo")
+        )
+    );
+  }
 }
