@@ -45,6 +45,9 @@ import org.apache.druid.java.util.common.StringUtils;
 import org.apache.druid.java.util.common.granularity.Granularities;
 import org.apache.druid.java.util.common.guava.Sequence;
 import org.apache.druid.java.util.common.guava.Sequences;
+import org.apache.druid.math.expr.Expr;
+import org.apache.druid.math.expr.ExprType;
+import org.apache.druid.math.expr.Parser;
 import org.apache.druid.query.BitmapResultFactory;
 import org.apache.druid.query.aggregation.Aggregator;
 import org.apache.druid.query.aggregation.CountAggregatorFactory;
@@ -57,6 +60,7 @@ import org.apache.druid.query.filter.DimFilter;
 import org.apache.druid.query.filter.Filter;
 import org.apache.druid.query.filter.ValueMatcher;
 import org.apache.druid.query.filter.vector.VectorValueMatcher;
+import org.apache.druid.segment.ColumnInspector;
 import org.apache.druid.segment.ColumnSelector;
 import org.apache.druid.segment.ColumnSelectorFactory;
 import org.apache.druid.segment.Cursor;
@@ -82,6 +86,8 @@ import org.apache.druid.segment.incremental.IncrementalIndexStorageAdapter;
 import org.apache.druid.segment.vector.SingleValueDimensionVectorSelector;
 import org.apache.druid.segment.vector.VectorColumnSelectorFactory;
 import org.apache.druid.segment.vector.VectorCursor;
+import org.apache.druid.segment.vector.VectorObjectSelector;
+import org.apache.druid.segment.vector.VectorValueSelector;
 import org.apache.druid.segment.virtual.ExpressionVirtualColumn;
 import org.apache.druid.segment.writeout.OffHeapMemorySegmentWriteOutMediumFactory;
 import org.apache.druid.segment.writeout.SegmentWriteOutMediumFactory;
@@ -113,7 +119,12 @@ public abstract class BaseFilterTest extends InitializedNullHandlingTest
       ImmutableList.of(
           new ExpressionVirtualColumn("expr", "1.0 + 0.1", ValueType.FLOAT, TestExprMacroTable.INSTANCE),
           new ExpressionVirtualColumn("exprDouble", "1.0 + 1.1", ValueType.DOUBLE, TestExprMacroTable.INSTANCE),
-          new ExpressionVirtualColumn("exprLong", "1 + 2", ValueType.LONG, TestExprMacroTable.INSTANCE)
+          new ExpressionVirtualColumn("exprLong", "1 + 2", ValueType.LONG, TestExprMacroTable.INSTANCE),
+          new ExpressionVirtualColumn("vdim0", "dim0", ValueType.STRING, TestExprMacroTable.INSTANCE),
+          new ExpressionVirtualColumn("vdim1", "dim1", ValueType.STRING, TestExprMacroTable.INSTANCE),
+          new ExpressionVirtualColumn("vd0", "d0", ValueType.DOUBLE, TestExprMacroTable.INSTANCE),
+          new ExpressionVirtualColumn("vf0", "f0", ValueType.FLOAT, TestExprMacroTable.INSTANCE),
+          new ExpressionVirtualColumn("vl0", "l0", ValueType.LONG, TestExprMacroTable.INSTANCE)
       )
   );
 
@@ -382,12 +393,11 @@ public abstract class BaseFilterTest extends InitializedNullHandlingTest
 
   private VectorCursor makeVectorCursor(final Filter filter)
   {
+
     return adapter.makeVectorCursor(
         filter,
         Intervals.ETERNITY,
-        // VirtualColumns do not support vectorization yet. Avoid passing them in, and any tests that need virtual
-        // columns should skip vectorization tests.
-        VirtualColumns.EMPTY,
+        VIRTUAL_COLUMNS,
         false,
         3, // Vector size smaller than the number of rows, to ensure we use more than one.
         null
@@ -445,7 +455,11 @@ public abstract class BaseFilterTest extends InitializedNullHandlingTest
 
   private long selectCountUsingVectorizedFilteredAggregator(final DimFilter dimFilter)
   {
-    Preconditions.checkState(makeFilter(dimFilter).canVectorizeMatcher(), "Cannot vectorize filter: %s", dimFilter);
+    Preconditions.checkState(
+        makeFilter(dimFilter).canVectorizeMatcher(adapter),
+        "Cannot vectorize filter: %s",
+        dimFilter
+    );
 
     try (final VectorCursor cursor = makeVectorCursor(null)) {
       final FilteredAggregatorFactory aggregatorFactory = new FilteredAggregatorFactory(
@@ -597,9 +611,9 @@ public abstract class BaseFilterTest extends InitializedNullHandlingTest
       }
 
       @Override
-      public boolean canVectorizeMatcher()
+      public boolean canVectorizeMatcher(ColumnInspector inspector)
       {
-        return theFilter.canVectorizeMatcher();
+        return theFilter.canVectorizeMatcher(inspector);
       }
 
       @Override
@@ -664,6 +678,63 @@ public abstract class BaseFilterTest extends InitializedNullHandlingTest
     }
   }
 
+  private List<String> selectColumnValuesMatchingFilterUsingVectorVirtualColumnCursor(
+      final DimFilter filter,
+      final String virtualColumn,
+      final String selectColumn
+  )
+  {
+    final Expr parsedIdentifier = Parser.parse(selectColumn, TestExprMacroTable.INSTANCE);
+    try (final VectorCursor cursor = makeVectorCursor(makeFilter(filter))) {
+
+      final ExprType outputType = parsedIdentifier.getOutputType(cursor.getColumnSelectorFactory());
+      final List<String> values = new ArrayList<>();
+
+      if (ExprType.STRING.equals(outputType)) {
+        final VectorObjectSelector objectSelector = cursor.getColumnSelectorFactory().makeObjectSelector(
+            virtualColumn
+        );
+        while (!cursor.isDone()) {
+          final Object[] rowVector = objectSelector.getObjectVector();
+          for (int i = 0; i < cursor.getCurrentVectorSize(); i++) {
+            values.add((String) rowVector[i]);
+          }
+          cursor.advance();
+        }
+      } else {
+        final VectorValueSelector valueSelector = cursor.getColumnSelectorFactory().makeValueSelector(virtualColumn);
+        while (!cursor.isDone()) {
+          final boolean[] nulls = valueSelector.getNullVector();
+          if (ExprType.DOUBLE.equals(outputType)) {
+            final double[] doubles = valueSelector.getDoubleVector();
+            for (int i = 0; i < cursor.getCurrentVectorSize(); i++) {
+              if (nulls != null && nulls[i]) {
+                values.add(null);
+              } else {
+                values.add(String.valueOf(doubles[i]));
+              }
+            }
+          } else {
+            final long[] longs = valueSelector.getLongVector();
+            for (int i = 0; i < cursor.getCurrentVectorSize(); i++) {
+              if (nulls != null && nulls[i]) {
+                values.add(null);
+              } else {
+                values.add(String.valueOf(longs[i]));
+              }
+            }
+          }
+
+          cursor.advance();
+        }
+      }
+
+
+
+      return values;
+    }
+  }
+
   private List<String> selectColumnValuesMatchingFilterUsingRowBasedColumnSelectorFactory(
       final DimFilter filter,
       final String selectColumn
@@ -734,6 +805,12 @@ public abstract class BaseFilterTest extends InitializedNullHandlingTest
           expectedRows,
           selectColumnValuesMatchingFilterUsingVectorCursor(filter, "dim0")
       );
+
+      Assert.assertEquals(
+          "Cursor Virtual Column (vectorized): " + filter,
+          expectedRows,
+          selectColumnValuesMatchingFilterUsingVectorVirtualColumnCursor(filter, "vdim0", "dim0")
+      );
     }
 
     Assert.assertEquals(
