@@ -26,6 +26,7 @@ import org.apache.druid.client.cache.CachePopulatorStats;
 import org.apache.druid.java.util.common.ISE;
 import org.apache.druid.java.util.emitter.service.ServiceEmitter;
 import org.apache.druid.query.Query;
+import org.apache.druid.query.QueryProcessingPool;
 import org.apache.druid.query.QueryRunner;
 import org.apache.druid.query.QueryRunnerFactoryConglomerate;
 import org.apache.druid.query.SegmentDescriptor;
@@ -40,8 +41,6 @@ import org.apache.druid.segment.realtime.FireDepartmentMetrics;
 import org.apache.druid.server.coordination.DataSegmentAnnouncer;
 import org.joda.time.Interval;
 
-import java.util.concurrent.ExecutorService;
-
 /**
  * Manages Appenderators for tasks running within a CliPeon process.
  *
@@ -73,7 +72,7 @@ public class PeonAppenderatorsManager implements AppenderatorsManager
       QueryRunnerFactoryConglomerate conglomerate,
       DataSegmentAnnouncer segmentAnnouncer,
       ServiceEmitter emitter,
-      ExecutorService queryExecutorService,
+      QueryProcessingPool queryProcessingPool,
       JoinableFactory joinableFactory,
       Cache cache,
       CacheConfig cacheConfig,
@@ -99,7 +98,7 @@ public class PeonAppenderatorsManager implements AppenderatorsManager
           conglomerate,
           segmentAnnouncer,
           emitter,
-          queryExecutorService,
+          queryProcessingPool,
           joinableFactory,
           cache,
           cacheConfig,
@@ -145,6 +144,73 @@ public class PeonAppenderatorsManager implements AppenderatorsManager
     }
   }
 
+  @Override
+  public Appenderator createOpenSegmentsOfflineAppenderatorForTask(
+      String taskId,
+      DataSchema schema,
+      AppenderatorConfig config,
+      FireDepartmentMetrics metrics,
+      DataSegmentPusher dataSegmentPusher,
+      ObjectMapper objectMapper,
+      IndexIO indexIO,
+      IndexMerger indexMerger,
+      RowIngestionMeters rowIngestionMeters,
+      ParseExceptionHandler parseExceptionHandler
+  )
+  {
+    // CompactionTask does run multiple sub-IndexTasks, so we allow multiple batch appenderators
+    if (realtimeAppenderator != null) {
+      throw new ISE("A realtime appenderator was already created for this peon's task.");
+    } else {
+      batchAppenderator = Appenderators.createOpenSegmentsOffline(
+          taskId,
+          schema,
+          config,
+          metrics,
+          dataSegmentPusher,
+          objectMapper,
+          indexIO,
+          indexMerger,
+          rowIngestionMeters,
+          parseExceptionHandler
+      );
+      return batchAppenderator;
+    }
+  }
+
+  @Override
+  public Appenderator createClosedSegmentsOfflineAppenderatorForTask(
+      String taskId,
+      DataSchema schema,
+      AppenderatorConfig config,
+      FireDepartmentMetrics metrics,
+      DataSegmentPusher dataSegmentPusher,
+      ObjectMapper objectMapper,
+      IndexIO indexIO,
+      IndexMerger indexMerger,
+      RowIngestionMeters rowIngestionMeters,
+      ParseExceptionHandler parseExceptionHandler
+  )
+  {
+    // CompactionTask does run multiple sub-IndexTasks, so we allow multiple batch appenderators
+    if (realtimeAppenderator != null) {
+      throw new ISE("A realtime appenderator was already created for this peon's task.");
+    } else {
+      batchAppenderator = Appenderators.createClosedSegmentsOffline(
+          taskId,
+          schema,
+          config,
+          metrics,
+          dataSegmentPusher,
+          objectMapper,
+          indexIO,
+          indexMerger,
+          rowIngestionMeters,
+          parseExceptionHandler
+      );
+      return batchAppenderator;
+    }
+  }
   @Override
   public void removeAppenderatorsForTask(String taskId, String dataSource)
   {
