@@ -29,6 +29,7 @@ import org.apache.druid.indexing.overlord.ObjectMetadata;
 import org.apache.druid.indexing.overlord.SegmentPublishResult;
 import org.apache.druid.indexing.overlord.Segments;
 import org.apache.druid.java.util.common.DateTimes;
+import org.apache.druid.java.util.common.ISE;
 import org.apache.druid.java.util.common.Intervals;
 import org.apache.druid.java.util.common.StringUtils;
 import org.apache.druid.segment.TestHelper;
@@ -54,11 +55,13 @@ import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.ExpectedException;
 import org.skife.jdbi.v2.Handle;
+import org.skife.jdbi.v2.PreparedBatch;
 import org.skife.jdbi.v2.tweak.HandleCallback;
 import org.skife.jdbi.v2.util.StringMapper;
 
 import java.io.IOException;
 import java.nio.charset.StandardCharsets;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
@@ -70,6 +73,8 @@ import java.util.stream.Collectors;
 
 public class IndexerSQLMetadataStorageCoordinatorTest
 {
+  private static final int MAX_SQL_MEATADATA_RETRY_FOR_TEST = 2;
+
   @Rule
   public final TestDerbyConnector.DerbyConnectorRule derbyConnectorRule = new TestDerbyConnector.DerbyConnectorRule();
 
@@ -187,8 +192,34 @@ public class IndexerSQLMetadataStorageCoordinatorTest
       100
   );
 
+  private final DataSegment existingSegment1 = new DataSegment(
+      "fooDataSource",
+      Intervals.of("1994-01-01T00Z/1994-01-02T00Z"),
+      "zversion",
+      ImmutableMap.of(),
+      ImmutableList.of("dim1"),
+      ImmutableList.of("m1"),
+      new NumberedShardSpec(1, 1),
+      9,
+      100
+  );
+
+  private final DataSegment existingSegment2 = new DataSegment(
+      "fooDataSource",
+      Intervals.of("1994-01-02T00Z/1994-01-03T00Z"),
+      "zversion",
+      ImmutableMap.of(),
+      ImmutableList.of("dim1"),
+      ImmutableList.of("m1"),
+      new NumberedShardSpec(1, 1),
+      9,
+      100
+  );
+
   private final Set<DataSegment> SEGMENTS = ImmutableSet.of(defaultSegment, defaultSegment2);
   private final AtomicLong metadataUpdateCounter = new AtomicLong();
+  private final AtomicLong segmentTableDropUpdateCounter = new AtomicLong();
+
   private IndexerSQLMetadataStorageCoordinator coordinator;
   private TestDerbyConnector derbyConnector;
 
@@ -202,6 +233,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     derbyConnector.createSegmentTable();
     derbyConnector.createPendingSegmentsTable();
     metadataUpdateCounter.set(0);
+    segmentTableDropUpdateCounter.set(0);
     coordinator = new IndexerSQLMetadataStorageCoordinator(
         mapper,
         derbyConnectorRule.metadataTablesConfigSupplier().get(),
@@ -209,7 +241,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     )
     {
       @Override
-      protected DataSourceMetadataUpdateResult updateDataSourceMetadataWithHandle(
+      protected DataStoreMetadataUpdateResult updateDataSourceMetadataWithHandle(
           Handle handle,
           String dataSource,
           DataSourceMetadata startMetadata,
@@ -220,6 +252,24 @@ public class IndexerSQLMetadataStorageCoordinatorTest
         metadataUpdateCounter.getAndIncrement();
         return super.updateDataSourceMetadataWithHandle(handle, dataSource, startMetadata, endMetadata);
       }
+
+      @Override
+      protected DataStoreMetadataUpdateResult dropSegmentsWithHandle(
+          final Handle handle,
+          final Set<DataSegment> segmentsToDrop,
+          final String dataSource
+      )
+      {
+        // Count number of times this method is called.
+        segmentTableDropUpdateCounter.getAndIncrement();
+        return super.dropSegmentsWithHandle(handle, segmentsToDrop, dataSource);
+      }
+
+      @Override
+      public int getSqlMetadataMaxRetry()
+      {
+        return MAX_SQL_MEATADATA_RETRY_FOR_TEST;
+      }
     };
   }
 
@@ -263,6 +313,47 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     );
   }
 
+  private Boolean insertUsedSegments(Set<DataSegment> dataSegments)
+  {
+    final String table = derbyConnectorRule.metadataTablesConfigSupplier().get().getSegmentsTable();
+    return derbyConnector.retryWithHandle(
+        new HandleCallback<Boolean>()
+        {
+          @Override
+          public Boolean withHandle(Handle handle) throws Exception
+          {
+            PreparedBatch preparedBatch = handle.prepareBatch(
+                StringUtils.format(
+                    "INSERT INTO %1$s (id, dataSource, created_date, start, %2$send%2$s, partitioned, version, used, payload) "
+                    + "VALUES (:id, :dataSource, :created_date, :start, :end, :partitioned, :version, :used, :payload)",
+                    table,
+                    derbyConnector.getQuoteString()
+                )
+            );
+            for (DataSegment segment : dataSegments) {
+              preparedBatch.add()
+                           .bind("id", segment.getId().toString())
+                           .bind("dataSource", segment.getDataSource())
+                           .bind("created_date", DateTimes.nowUtc().toString())
+                           .bind("start", segment.getInterval().getStart().toString())
+                           .bind("end", segment.getInterval().getEnd().toString())
+                           .bind("partitioned", (segment.getShardSpec() instanceof NoneShardSpec) ? false : true)
+                           .bind("version", segment.getVersion())
+                           .bind("used", true)
+                           .bind("payload", mapper.writeValueAsBytes(segment));
+            }
+
+            final int[] affectedRows = preparedBatch.execute();
+            final boolean succeeded = Arrays.stream(affectedRows).allMatch(eachAffectedRows -> eachAffectedRows == 1);
+            if (!succeeded) {
+              throw new ISE("Failed to publish segments to DB");
+            }
+            return true;
+          }
+        }
+    );
+  }
+
   @Test
   public void testSimpleAnnounce() throws IOException
   {
@@ -360,6 +451,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     // Insert first segment.
     final SegmentPublishResult result1 = coordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
         new ObjectMetadata(null),
         new ObjectMetadata(ImmutableMap.of("foo", "bar"))
     );
@@ -378,6 +470,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     // Insert second segment.
     final SegmentPublishResult result2 = coordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment2),
+        ImmutableSet.of(),
         new ObjectMetadata(ImmutableMap.of("foo", "bar")),
         new ObjectMetadata(ImmutableMap.of("foo", "baz"))
     );
@@ -415,7 +508,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     )
     {
       @Override
-      protected DataSourceMetadataUpdateResult updateDataSourceMetadataWithHandle(
+      protected DataStoreMetadataUpdateResult updateDataSourceMetadataWithHandle(
           Handle handle,
           String dataSource,
           DataSourceMetadata startMetadata,
@@ -424,7 +517,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
       {
         metadataUpdateCounter.getAndIncrement();
         if (attemptCounter.getAndIncrement() == 0) {
-          return DataSourceMetadataUpdateResult.TRY_AGAIN;
+          return DataStoreMetadataUpdateResult.TRY_AGAIN;
         } else {
           return super.updateDataSourceMetadataWithHandle(handle, dataSource, startMetadata, endMetadata);
         }
@@ -434,6 +527,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     // Insert first segment.
     final SegmentPublishResult result1 = failOnceCoordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
         new ObjectMetadata(null),
         new ObjectMetadata(ImmutableMap.of("foo", "bar"))
     );
@@ -455,6 +549,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     // Insert second segment.
     final SegmentPublishResult result2 = failOnceCoordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment2),
+        ImmutableSet.of(),
         new ObjectMetadata(ImmutableMap.of("foo", "bar")),
         new ObjectMetadata(ImmutableMap.of("foo", "baz"))
     );
@@ -485,6 +580,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
   {
     final SegmentPublishResult result1 = coordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
         new ObjectMetadata(ImmutableMap.of("foo", "bar")),
         new ObjectMetadata(ImmutableMap.of("foo", "baz"))
     );
@@ -494,11 +590,116 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     Assert.assertEquals(1, metadataUpdateCounter.get());
   }
 
+  @Test
+  public void testTransactionalAnnounceFailSegmentDropFailWithoutRetry() throws IOException
+  {
+    insertUsedSegments(ImmutableSet.of(existingSegment1, existingSegment2));
+
+    Assert.assertEquals(
+        ImmutableList.of(existingSegment1.getId().toString(), existingSegment2.getId().toString()),
+        retrieveUsedSegmentIds()
+    );
+
+    DataSegment dataSegmentBar = DataSegment.builder()
+                                            .dataSource("bar")
+                                            .interval(Intervals.of("2001/P1D"))
+                                            .shardSpec(new LinearShardSpec(1))
+                                            .version("b")
+                                            .size(0)
+                                            .build();
+    Set<DataSegment> dropSegments = ImmutableSet.of(existingSegment1, existingSegment2, dataSegmentBar);
+
+    final SegmentPublishResult result1 = coordinator.announceHistoricalSegments(
+        SEGMENTS,
+        dropSegments,
+        null,
+        null
+    );
+    Assert.assertEquals(SegmentPublishResult.fail("java.lang.RuntimeException: Aborting transaction!"), result1);
+
+    // Should only be tried once. Since dropSegmentsWithHandle will return FAILURE (not TRY_AGAIN) as set of
+    // segments to drop contains more than one datasource.
+    Assert.assertEquals(1, segmentTableDropUpdateCounter.get());
+
+    Assert.assertEquals(
+        ImmutableList.of(existingSegment1.getId().toString(), existingSegment2.getId().toString()),
+        retrieveUsedSegmentIds()
+    );
+  }
+
+  @Test
+  public void testTransactionalAnnounceSucceedWithSegmentDrop() throws IOException
+  {
+    insertUsedSegments(ImmutableSet.of(existingSegment1, existingSegment2));
+
+    Assert.assertEquals(
+        ImmutableList.of(existingSegment1.getId().toString(), existingSegment2.getId().toString()),
+        retrieveUsedSegmentIds()
+    );
+
+    final SegmentPublishResult result1 = coordinator.announceHistoricalSegments(
+        SEGMENTS,
+        ImmutableSet.of(existingSegment1, existingSegment2),
+        null,
+        null
+    );
+
+    Assert.assertEquals(SegmentPublishResult.ok(SEGMENTS), result1);
+
+    for (DataSegment segment : SEGMENTS) {
+      Assert.assertArrayEquals(
+          mapper.writeValueAsString(segment).getBytes(StandardCharsets.UTF_8),
+          derbyConnector.lookup(
+              derbyConnectorRule.metadataTablesConfigSupplier().get().getSegmentsTable(),
+              "id",
+              "payload",
+              segment.getId().toString()
+          )
+      );
+    }
+
+    Assert.assertEquals(
+        ImmutableList.of(defaultSegment.getId().toString(), defaultSegment2.getId().toString()),
+        retrieveUsedSegmentIds()
+    );
+  }
+
+  @Test
+  public void testTransactionalAnnounceFailSegmentDropFailWithRetry() throws IOException
+  {
+    insertUsedSegments(ImmutableSet.of(existingSegment1, existingSegment2));
+
+    Assert.assertEquals(
+        ImmutableList.of(existingSegment1.getId().toString(), existingSegment2.getId().toString()),
+        retrieveUsedSegmentIds()
+    );
+
+    DataSegment nonExistingSegment = defaultSegment4;
+
+    Set<DataSegment> dropSegments = ImmutableSet.of(existingSegment1, nonExistingSegment);
+
+    final SegmentPublishResult result1 = coordinator.announceHistoricalSegments(
+        SEGMENTS,
+        dropSegments,
+        null,
+        null
+    );
+    Assert.assertEquals(SegmentPublishResult.fail("org.apache.druid.metadata.RetryTransactionException: Aborting transaction!"), result1);
+
+    Assert.assertEquals(MAX_SQL_MEATADATA_RETRY_FOR_TEST, segmentTableDropUpdateCounter.get());
+
+    Assert.assertEquals(
+        ImmutableList.of(existingSegment1.getId().toString(), existingSegment2.getId().toString()),
+        retrieveUsedSegmentIds()
+    );
+  }
+
   @Test
   public void testTransactionalAnnounceFailDbNotNullWantNull() throws IOException
   {
     final SegmentPublishResult result1 = coordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
         new ObjectMetadata(null),
         new ObjectMetadata(ImmutableMap.of("foo", "baz"))
     );
@@ -506,6 +707,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
 
     final SegmentPublishResult result2 = coordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment2),
+        ImmutableSet.of(),
         new ObjectMetadata(null),
         new ObjectMetadata(ImmutableMap.of("foo", "baz"))
     );
@@ -520,6 +722,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
   {
     final SegmentPublishResult result1 = coordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
         new ObjectMetadata(null),
         new ObjectMetadata(ImmutableMap.of("foo", "baz"))
     );
@@ -527,6 +730,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
 
     final SegmentPublishResult result2 = coordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment2),
+        ImmutableSet.of(),
         new ObjectMetadata(ImmutableMap.of("foo", "qux")),
         new ObjectMetadata(ImmutableMap.of("foo", "baz"))
     );
@@ -833,6 +1037,7 @@ public class IndexerSQLMetadataStorageCoordinatorTest
   {
     coordinator.announceHistoricalSegments(
         ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
         new ObjectMetadata(null),
         new ObjectMetadata(ImmutableMap.of("foo", "bar"))
     );
@@ -848,6 +1053,39 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     Assert.assertNull("getDataSourceMetadataNullAfterDelete", coordinator.retrieveDataSourceMetadata("fooDataSource"));
   }
 
+  @Test
+  public void testDeleteSegmentsInMetaDataStorage() throws IOException
+  {
+    // Published segments to MetaDataStorage
+    coordinator.announceHistoricalSegments(SEGMENTS);
+
+    // check segments Published
+    Assert.assertEquals(
+            SEGMENTS,
+            ImmutableSet.copyOf(
+                    coordinator.retrieveUsedSegmentsForInterval(
+                            defaultSegment.getDataSource(),
+                            defaultSegment.getInterval(),
+                            Segments.ONLY_VISIBLE
+                    )
+            )
+    );
+    // remove segments in MetaDataStorage
+    coordinator.deleteSegments(SEGMENTS);
+
+    // check segments removed
+    Assert.assertEquals(
+            0,
+            ImmutableSet.copyOf(
+                    coordinator.retrieveUsedSegmentsForInterval(
+                            defaultSegment.getDataSource(),
+                            defaultSegment.getInterval(),
+                            Segments.ONLY_VISIBLE
+                    )
+            ).size()
+    );
+  }
+
   @Test
   public void testSingleAdditionalNumberedShardWithNoCorePartitions() throws IOException
   {
@@ -1205,4 +1443,142 @@ public class IndexerSQLMetadataStorageCoordinatorTest
     );
     Assert.assertNull(id);
   }
+
+  @Test
+  public void testDropSegmentsWithHandleForSegmentThatExist()
+  {
+    try (Handle handle = derbyConnector.getDBI().open()) {
+      Assert.assertTrue(insertUsedSegments(ImmutableSet.of(defaultSegment)));
+      List<String> usedSegments = retrieveUsedSegmentIds();
+      Assert.assertEquals(1, usedSegments.size());
+      Assert.assertEquals(defaultSegment.getId().toString(), usedSegments.get(0));
+
+      // Try drop segment
+      IndexerSQLMetadataStorageCoordinator.DataStoreMetadataUpdateResult result = coordinator.dropSegmentsWithHandle(handle, ImmutableSet.of(defaultSegment), defaultSegment.getDataSource());
+
+      Assert.assertEquals(IndexerSQLMetadataStorageCoordinator.DataStoreMetadataUpdateResult.SUCCESS, result);
+      usedSegments = retrieveUsedSegmentIds();
+      Assert.assertEquals(0, usedSegments.size());
+    }
+  }
+
+  @Test
+  public void testDropSegmentsWithHandleForSegmentThatDoesNotExist()
+  {
+    try (Handle handle = derbyConnector.getDBI().open()) {
+      // Try drop segment
+      IndexerSQLMetadataStorageCoordinator.DataStoreMetadataUpdateResult result = coordinator.dropSegmentsWithHandle(handle, ImmutableSet.of(defaultSegment), defaultSegment.getDataSource());
+      Assert.assertEquals(IndexerSQLMetadataStorageCoordinator.DataStoreMetadataUpdateResult.TRY_AGAIN, result);
+    }
+  }
+
+  @Test
+  public void testRemoveDataSourceMetadataOlderThanDatasourceActiveShouldNotBeDeleted() throws Exception
+  {
+    coordinator.announceHistoricalSegments(
+        ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
+        new ObjectMetadata(null),
+        new ObjectMetadata(ImmutableMap.of("foo", "bar"))
+    );
+
+    Assert.assertEquals(
+        new ObjectMetadata(ImmutableMap.of("foo", "bar")),
+        coordinator.retrieveDataSourceMetadata("fooDataSource")
+    );
+
+    // Try delete. Datasource should not be deleted as it is in excluded set
+    int deletedCount = coordinator.removeDataSourceMetadataOlderThan(System.currentTimeMillis(), ImmutableSet.of("fooDataSource"));
+
+    // Datasource should not be deleted
+    Assert.assertEquals(
+        new ObjectMetadata(ImmutableMap.of("foo", "bar")),
+        coordinator.retrieveDataSourceMetadata("fooDataSource")
+    );
+    Assert.assertEquals(0, deletedCount);
+  }
+
+  @Test
+  public void testRemoveDataSourceMetadataOlderThanDatasourceNotActiveAndOlderThanTimeShouldBeDeleted() throws Exception
+  {
+    coordinator.announceHistoricalSegments(
+        ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
+        new ObjectMetadata(null),
+        new ObjectMetadata(ImmutableMap.of("foo", "bar"))
+    );
+
+    Assert.assertEquals(
+        new ObjectMetadata(ImmutableMap.of("foo", "bar")),
+        coordinator.retrieveDataSourceMetadata("fooDataSource")
+    );
+
+    // Try delete. Datasource should be deleted as it is not in excluded set and created time older than given time
+    int deletedCount = coordinator.removeDataSourceMetadataOlderThan(System.currentTimeMillis(), ImmutableSet.of());
+
+    // Datasource should be deleted
+    Assert.assertNull(
+        coordinator.retrieveDataSourceMetadata("fooDataSource")
+    );
+    Assert.assertEquals(1, deletedCount);
+  }
+
+  @Test
+  public void testRemoveDataSourceMetadataOlderThanDatasourceNotActiveButNotOlderThanTimeShouldNotBeDeleted() throws Exception
+  {
+    coordinator.announceHistoricalSegments(
+        ImmutableSet.of(defaultSegment),
+        ImmutableSet.of(),
+        new ObjectMetadata(null),
+        new ObjectMetadata(ImmutableMap.of("foo", "bar"))
+    );
+
+    Assert.assertEquals(
+        new ObjectMetadata(ImmutableMap.of("foo", "bar")),
+        coordinator.retrieveDataSourceMetadata("fooDataSource")
+    );
+
+    // Do delete. Datasource metadata should not be deleted. Datasource is not active but it was created just now so it's
+    // created timestamp will be later than the timestamp 2012-01-01T00:00:00Z
+    int deletedCount = coordinator.removeDataSourceMetadataOlderThan(DateTimes.of("2012-01-01T00:00:00Z").getMillis(), ImmutableSet.of());
+
+    // Datasource should not be deleted
+    Assert.assertEquals(
+        new ObjectMetadata(ImmutableMap.of("foo", "bar")),
+        coordinator.retrieveDataSourceMetadata("fooDataSource")
+    );
+    Assert.assertEquals(0, deletedCount);
+  }
+
+  @Test
+  public void testMarkSegmentsAsUnusedWithinInterval() throws IOException
+  {
+    coordinator.announceHistoricalSegments(ImmutableSet.of(existingSegment1, existingSegment2));
+
+    // interval covers existingSegment1 and partially overlaps existingSegment2,
+    // only existingSegment1 will be dropped
+    coordinator.markSegmentsAsUnusedWithinInterval(
+        existingSegment1.getDataSource(),
+        Intervals.of("1993-12-31T12Z/1994-01-02T12Z")
+    );
+
+    Assert.assertEquals(
+        ImmutableSet.of(existingSegment1),
+        ImmutableSet.copyOf(
+            coordinator.retrieveUnusedSegmentsForInterval(
+                existingSegment1.getDataSource(),
+                existingSegment1.getInterval().withEnd(existingSegment1.getInterval().getEnd().plus(1))
+            )
+        )
+    );
+    Assert.assertEquals(
+        ImmutableSet.of(),
+        ImmutableSet.copyOf(
+            coordinator.retrieveUnusedSegmentsForInterval(
+                existingSegment2.getDataSource(),
+                existingSegment2.getInterval().withEnd(existingSegment2.getInterval().getEnd().plusYears(1))
+            )
+        )
+    );
+  }
 }
