@@ -28,21 +28,30 @@ import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.util.concurrent.ListenableFuture;
 import com.google.common.util.concurrent.MoreExecutors;
+import com.google.inject.Injector;
+import com.google.inject.Key;
+import org.apache.druid.guice.GuiceInjectors;
+import org.apache.druid.guice.annotations.Smile;
 import org.apache.druid.jackson.DefaultObjectMapper;
 import org.apache.druid.java.util.common.concurrent.Execs;
 import org.apache.druid.java.util.common.guava.LazySequence;
 import org.apache.druid.java.util.common.guava.Sequences;
 import org.apache.druid.java.util.emitter.EmittingLogger;
 import org.apache.druid.java.util.emitter.service.ServiceEmitter;
+import org.apache.druid.query.BadJsonQueryException;
 import org.apache.druid.query.DefaultGenericQueryMetricsFactory;
 import org.apache.druid.query.DefaultQueryConfig;
 import org.apache.druid.query.MapQueryToolChestWarehouse;
 import org.apache.druid.query.Query;
+import org.apache.druid.query.QueryCapacityExceededException;
+import org.apache.druid.query.QueryException;
 import org.apache.druid.query.QueryInterruptedException;
 import org.apache.druid.query.QueryRunner;
 import org.apache.druid.query.QuerySegmentWalker;
+import org.apache.druid.query.QueryTimeoutException;
 import org.apache.druid.query.QueryToolChestWarehouse;
 import org.apache.druid.query.QueryUnsupportedException;
+import org.apache.druid.query.ResourceLimitExceededException;
 import org.apache.druid.query.Result;
 import org.apache.druid.query.SegmentDescriptor;
 import org.apache.druid.query.TruncatedResponseContextException;
@@ -75,6 +84,7 @@ import org.junit.Test;
 import javax.servlet.http.HttpServletRequest;
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.Response;
+import javax.ws.rs.core.Response.Status;
 import javax.ws.rs.core.StreamingOutput;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
@@ -90,7 +100,6 @@ import java.util.function.Consumer;
 public class QueryResourceTest
 {
   private static final QueryToolChestWarehouse WAREHOUSE = new MapQueryToolChestWarehouse(ImmutableMap.of());
-  private static final ObjectMapper JSON_MAPPER = new DefaultObjectMapper();
   private static final AuthenticationResult AUTHENTICATION_RESULT =
       new AuthenticationResult("druid", "druid", null, null);
 
@@ -172,6 +181,8 @@ public class QueryResourceTest
       false
   );
 
+  private ObjectMapper jsonMapper;
+  private ObjectMapper smileMapper;
   private QueryResource queryResource;
   private QueryScheduler queryScheduler;
   private TestRequestLogger testRequestLogger;
@@ -185,6 +196,10 @@ public class QueryResourceTest
   @Before
   public void setup()
   {
+    Injector injector = GuiceInjectors.makeStartupInjector();
+    jsonMapper = injector.getInstance(ObjectMapper.class);
+    smileMapper = injector.getInstance(Key.get(ObjectMapper.class, Smile.class));
+
     EasyMock.expect(testServletRequest.getContentType()).andReturn(MediaType.APPLICATION_JSON).anyTimes();
     EasyMock.expect(testServletRequest.getHeader("Accept")).andReturn(MediaType.APPLICATION_JSON).anyTimes();
     EasyMock.expect(testServletRequest.getHeader(QueryResource.HEADER_IF_NONE_MATCH)).andReturn(null).anyTimes();
@@ -207,8 +222,8 @@ public class QueryResourceTest
             AuthTestUtils.TEST_AUTHORIZER_MAPPER,
             Suppliers.ofInstance(new DefaultQueryConfig(ImmutableMap.of()))
         ),
-        JSON_MAPPER,
-        JSON_MAPPER,
+        jsonMapper,
+        smileMapper,
         queryScheduler,
         new AuthConfig(),
         null,
@@ -253,8 +268,8 @@ public class QueryResourceTest
             AuthTestUtils.TEST_AUTHORIZER_MAPPER,
             Suppliers.ofInstance(overrideConfig)
         ),
-        JSON_MAPPER,
-        JSON_MAPPER,
+        jsonMapper,
+        smileMapper,
         queryScheduler,
         new AuthConfig(),
         null,
@@ -273,7 +288,7 @@ public class QueryResourceTest
 
     final ByteArrayOutputStream baos = new ByteArrayOutputStream();
     ((StreamingOutput) response.getEntity()).write(baos);
-    final List<Result<TimeBoundaryResultValue>> responses = JSON_MAPPER.readValue(
+    final List<Result<TimeBoundaryResultValue>> responses = jsonMapper.readValue(
         baos.toByteArray(),
         new TypeReference<List<Result<TimeBoundaryResultValue>>>() {}
     );
@@ -305,8 +320,8 @@ public class QueryResourceTest
             AuthTestUtils.TEST_AUTHORIZER_MAPPER,
             Suppliers.ofInstance(overrideConfig)
         ),
-        JSON_MAPPER,
-        JSON_MAPPER,
+        jsonMapper,
+        smileMapper,
         queryScheduler,
         new AuthConfig(),
         null,
@@ -326,7 +341,7 @@ public class QueryResourceTest
 
     final ByteArrayOutputStream baos = new ByteArrayOutputStream();
     ((StreamingOutput) response.getEntity()).write(baos);
-    final List<Result<TimeBoundaryResultValue>> responses = JSON_MAPPER.readValue(
+    final List<Result<TimeBoundaryResultValue>> responses = jsonMapper.readValue(
         baos.toByteArray(),
         new TypeReference<List<Result<TimeBoundaryResultValue>>>() {}
     );
@@ -361,7 +376,7 @@ public class QueryResourceTest
     ).toString();
     Assert.assertEquals(
         expectedException,
-        JSON_MAPPER.readValue((byte[]) response.getEntity(), QueryInterruptedException.class).toString()
+        jsonMapper.readValue((byte[]) response.getEntity(), QueryInterruptedException.class).toString()
     );
   }
 
@@ -451,7 +466,7 @@ public class QueryResourceTest
   }
 
   @Test
-  public void testGoodQueryWithSmileAcceptHeader() throws IOException
+  public void testGoodQueryWithJsonRequestAndSmileAcceptHeader() throws IOException
   {
     //Doing a replay of testServletRequest for teardown to succeed.
     //We dont use testServletRequest in this testcase
@@ -460,6 +475,8 @@ public class QueryResourceTest
     //Creating our own Smile Servlet request, as to not disturb the remaining tests.
     // else refactoring required for this class. i know this kinda makes the class somewhat Dirty.
     final HttpServletRequest smileRequest = EasyMock.createMock(HttpServletRequest.class);
+
+    // Set Content-Type to JSON
     EasyMock.expect(smileRequest.getContentType()).andReturn(MediaType.APPLICATION_JSON).anyTimes();
 
     EasyMock.expect(smileRequest.getAttribute(AuthConfig.DRUID_AUTHORIZATION_CHECKED))
@@ -473,6 +490,7 @@ public class QueryResourceTest
 
     smileRequest.setAttribute(AuthConfig.DRUID_AUTHORIZATION_CHECKED, true);
 
+    // Set Accept to Smile
     EasyMock.expect(smileRequest.getHeader("Accept")).andReturn(SmileMediaTypes.APPLICATION_JACKSON_SMILE).anyTimes();
     EasyMock.expect(smileRequest.getHeader(QueryResource.HEADER_IF_NONE_MATCH)).andReturn(null).anyTimes();
     EasyMock.expect(smileRequest.getRemoteAddr()).andReturn("localhost").anyTimes();
@@ -484,11 +502,102 @@ public class QueryResourceTest
         smileRequest
     );
     Assert.assertEquals(HttpStatus.SC_OK, response.getStatus());
+
+    // Content-Type in response should be Smile
     Assert.assertEquals(SmileMediaTypes.APPLICATION_JACKSON_SMILE, (response.getMetadata().get("Content-Type").get(0)).toString());
     Assert.assertNotNull(response);
     EasyMock.verify(smileRequest);
   }
 
+  @Test
+  public void testGoodQueryWithSmileRequestAndSmileAcceptHeader() throws IOException
+  {
+    //Doing a replay of testServletRequest for teardown to succeed.
+    //We dont use testServletRequest in this testcase
+    EasyMock.replay(testServletRequest);
+
+    //Creating our own Smile Servlet request, as to not disturb the remaining tests.
+    // else refactoring required for this class. i know this kinda makes the class somewhat Dirty.
+    final HttpServletRequest smileRequest = EasyMock.createMock(HttpServletRequest.class);
+
+    // Set Content-Type to Smile
+    EasyMock.expect(smileRequest.getContentType()).andReturn(SmileMediaTypes.APPLICATION_JACKSON_SMILE).anyTimes();
+
+    EasyMock.expect(smileRequest.getAttribute(AuthConfig.DRUID_AUTHORIZATION_CHECKED))
+            .andReturn(null)
+            .anyTimes();
+    EasyMock.expect(smileRequest.getAttribute(AuthConfig.DRUID_ALLOW_UNSECURED_PATH)).andReturn(null).anyTimes();
+
+    EasyMock.expect(smileRequest.getAttribute(AuthConfig.DRUID_AUTHENTICATION_RESULT))
+            .andReturn(AUTHENTICATION_RESULT)
+            .anyTimes();
+
+    smileRequest.setAttribute(AuthConfig.DRUID_AUTHORIZATION_CHECKED, true);
+
+    // Set Accept to Smile
+    EasyMock.expect(smileRequest.getHeader("Accept")).andReturn(SmileMediaTypes.APPLICATION_JACKSON_SMILE).anyTimes();
+    EasyMock.expect(smileRequest.getHeader(QueryResource.HEADER_IF_NONE_MATCH)).andReturn(null).anyTimes();
+    EasyMock.expect(smileRequest.getRemoteAddr()).andReturn("localhost").anyTimes();
+
+    EasyMock.replay(smileRequest);
+    Response response = queryResource.doPost(
+        // Write input in Smile encoding
+        new ByteArrayInputStream(smileMapper.writeValueAsBytes(jsonMapper.readTree(SIMPLE_TIMESERIES_QUERY))),
+        null /*pretty*/,
+        smileRequest
+    );
+    Assert.assertEquals(HttpStatus.SC_OK, response.getStatus());
+
+    // Content-Type in response should be Smile
+    Assert.assertEquals(SmileMediaTypes.APPLICATION_JACKSON_SMILE, (response.getMetadata().get("Content-Type").get(0)).toString());
+    Assert.assertNotNull(response);
+    EasyMock.verify(smileRequest);
+  }
+
+  @Test
+  public void testGoodQueryWithSmileRequestNoSmileAcceptHeader() throws IOException
+  {
+    //Doing a replay of testServletRequest for teardown to succeed.
+    //We dont use testServletRequest in this testcase
+    EasyMock.replay(testServletRequest);
+
+    //Creating our own Smile Servlet request, as to not disturb the remaining tests.
+    // else refactoring required for this class. i know this kinda makes the class somewhat Dirty.
+    final HttpServletRequest smileRequest = EasyMock.createMock(HttpServletRequest.class);
+
+    // Set Content-Type to Smile
+    EasyMock.expect(smileRequest.getContentType()).andReturn(SmileMediaTypes.APPLICATION_JACKSON_SMILE).anyTimes();
+
+    EasyMock.expect(smileRequest.getAttribute(AuthConfig.DRUID_AUTHORIZATION_CHECKED))
+            .andReturn(null)
+            .anyTimes();
+    EasyMock.expect(smileRequest.getAttribute(AuthConfig.DRUID_ALLOW_UNSECURED_PATH)).andReturn(null).anyTimes();
+
+    EasyMock.expect(smileRequest.getAttribute(AuthConfig.DRUID_AUTHENTICATION_RESULT))
+            .andReturn(AUTHENTICATION_RESULT)
+            .anyTimes();
+
+    smileRequest.setAttribute(AuthConfig.DRUID_AUTHORIZATION_CHECKED, true);
+
+    // DO NOT set Accept to Smile, Content-Type in response will be default to Content-Type in request
+    EasyMock.expect(smileRequest.getHeader("Accept")).andReturn(null).anyTimes();
+    EasyMock.expect(smileRequest.getHeader(QueryResource.HEADER_IF_NONE_MATCH)).andReturn(null).anyTimes();
+    EasyMock.expect(smileRequest.getRemoteAddr()).andReturn("localhost").anyTimes();
+
+    EasyMock.replay(smileRequest);
+    Response response = queryResource.doPost(
+        // Write input in Smile encoding
+        new ByteArrayInputStream(smileMapper.writeValueAsBytes(jsonMapper.readTree(SIMPLE_TIMESERIES_QUERY))),
+        null /*pretty*/,
+        smileRequest
+    );
+    Assert.assertEquals(HttpStatus.SC_OK, response.getStatus());
+
+    // Content-Type in response will be default to Content-Type in request
+    Assert.assertEquals(SmileMediaTypes.APPLICATION_JACKSON_SMILE, (response.getMetadata().get("Content-Type").get(0)).toString());
+    Assert.assertNotNull(response);
+    EasyMock.verify(smileRequest);
+  }
 
   @Test
   public void testBadQuery() throws IOException
@@ -500,7 +609,29 @@ public class QueryResourceTest
         testServletRequest
     );
     Assert.assertNotNull(response);
-    Assert.assertEquals(Response.Status.INTERNAL_SERVER_ERROR.getStatusCode(), response.getStatus());
+    Assert.assertEquals(Status.BAD_REQUEST.getStatusCode(), response.getStatus());
+    QueryException e = jsonMapper.readValue((byte[]) response.getEntity(), QueryException.class);
+    Assert.assertEquals(BadJsonQueryException.ERROR_CODE, e.getErrorCode());
+    Assert.assertEquals(BadJsonQueryException.ERROR_CLASS, e.getErrorClass());
+  }
+
+  @Test
+  public void testResourceLimitExceeded() throws IOException
+  {
+    ByteArrayInputStream badQuery = EasyMock.createMock(ByteArrayInputStream.class);
+    EasyMock.expect(badQuery.read(EasyMock.anyObject(), EasyMock.anyInt(), EasyMock.anyInt()))
+            .andThrow(new ResourceLimitExceededException("You require too much of something"));
+    EasyMock.replay(badQuery, testServletRequest);
+    Response response = queryResource.doPost(
+        badQuery,
+        null /*pretty*/,
+        testServletRequest
+    );
+    Assert.assertNotNull(response);
+    Assert.assertEquals(Status.BAD_REQUEST.getStatusCode(), response.getStatus());
+    QueryException e = jsonMapper.readValue((byte[]) response.getEntity(), QueryException.class);
+    Assert.assertEquals(ResourceLimitExceededException.ERROR_CODE, e.getErrorCode());
+    Assert.assertEquals(ResourceLimitExceededException.class.getName(), e.getErrorClass());
   }
 
   @Test
@@ -519,13 +650,7 @@ public class QueryResourceTest
     );
     Assert.assertNotNull(response);
     Assert.assertEquals(QueryUnsupportedException.STATUS_CODE, response.getStatus());
-    QueryUnsupportedException ex;
-    try {
-      ex = JSON_MAPPER.readValue((byte[]) response.getEntity(), QueryUnsupportedException.class);
-    }
-    catch (IOException e) {
-      throw new RuntimeException(e);
-    }
+    QueryException ex = jsonMapper.readValue((byte[]) response.getEntity(), QueryException.class);
     Assert.assertEquals(errorMessage, ex.getMessage());
     Assert.assertEquals(QueryUnsupportedException.ERROR_CODE, ex.getErrorCode());
   }
@@ -582,8 +707,8 @@ public class QueryResourceTest
             authMapper,
             Suppliers.ofInstance(new DefaultQueryConfig(ImmutableMap.of()))
         ),
-        JSON_MAPPER,
-        JSON_MAPPER,
+        jsonMapper,
+        smileMapper,
         queryScheduler,
         new AuthConfig(),
         authMapper,
@@ -611,7 +736,7 @@ public class QueryResourceTest
 
     final ByteArrayOutputStream baos = new ByteArrayOutputStream();
     ((StreamingOutput) response.getEntity()).write(baos);
-    final List<Result<TimeBoundaryResultValue>> responses = JSON_MAPPER.readValue(
+    final List<Result<TimeBoundaryResultValue>> responses = jsonMapper.readValue(
         baos.toByteArray(),
         new TypeReference<List<Result<TimeBoundaryResultValue>>>() {}
     );
@@ -629,6 +754,64 @@ public class QueryResourceTest
     );
   }
 
+  @Test
+  public void testQueryTimeoutException() throws Exception
+  {
+    final QuerySegmentWalker timeoutSegmentWalker = new QuerySegmentWalker()
+    {
+      @Override
+      public <T> QueryRunner<T> getQueryRunnerForIntervals(Query<T> query, Iterable<Interval> intervals)
+      {
+        throw new QueryTimeoutException();
+      }
+
+      @Override
+      public <T> QueryRunner<T> getQueryRunnerForSegments(Query<T> query, Iterable<SegmentDescriptor> specs)
+      {
+        return getQueryRunnerForIntervals(null, null);
+      }
+    };
+
+    final QueryResource timeoutQueryResource = new QueryResource(
+        new QueryLifecycleFactory(
+            WAREHOUSE,
+            timeoutSegmentWalker,
+            new DefaultGenericQueryMetricsFactory(),
+            new NoopServiceEmitter(),
+            testRequestLogger,
+            new AuthConfig(),
+            AuthTestUtils.TEST_AUTHORIZER_MAPPER,
+            Suppliers.ofInstance(new DefaultQueryConfig(ImmutableMap.of()))
+        ),
+        jsonMapper,
+        jsonMapper,
+        queryScheduler,
+        new AuthConfig(),
+        null,
+        ResponseContextConfig.newConfig(true),
+        DRUID_NODE
+    );
+    expectPermissiveHappyPathAuth();
+    Response response = timeoutQueryResource.doPost(
+        new ByteArrayInputStream(SIMPLE_TIMESERIES_QUERY.getBytes(StandardCharsets.UTF_8)),
+        null /*pretty*/,
+        testServletRequest
+    );
+    Assert.assertNotNull(response);
+    Assert.assertEquals(QueryTimeoutException.STATUS_CODE, response.getStatus());
+    QueryTimeoutException ex;
+    try {
+      ex = jsonMapper.readValue((byte[]) response.getEntity(), QueryTimeoutException.class);
+    }
+    catch (IOException e) {
+      throw new RuntimeException(e);
+    }
+    Assert.assertEquals("Query Timed Out!", ex.getMessage());
+    Assert.assertEquals(QueryTimeoutException.ERROR_CODE, ex.getErrorCode());
+    Assert.assertEquals(1, timeoutQueryResource.getTimedOutQueryCount());
+
+  }
+
   @Test(timeout = 60_000L)
   public void testSecuredCancelQuery() throws Exception
   {
@@ -698,8 +881,8 @@ public class QueryResourceTest
             authMapper,
             Suppliers.ofInstance(new DefaultQueryConfig(ImmutableMap.of()))
         ),
-        JSON_MAPPER,
-        JSON_MAPPER,
+        jsonMapper,
+        smileMapper,
         queryScheduler,
         new AuthConfig(),
         authMapper,
@@ -822,8 +1005,8 @@ public class QueryResourceTest
             authMapper,
             Suppliers.ofInstance(new DefaultQueryConfig(ImmutableMap.of()))
         ),
-        JSON_MAPPER,
-        JSON_MAPPER,
+        jsonMapper,
+        smileMapper,
         queryScheduler,
         new AuthConfig(),
         authMapper,
@@ -916,7 +1099,7 @@ public class QueryResourceTest
           Assert.assertEquals(QueryCapacityExceededException.STATUS_CODE, response.getStatus());
           QueryCapacityExceededException ex;
           try {
-            ex = JSON_MAPPER.readValue((byte[]) response.getEntity(), QueryCapacityExceededException.class);
+            ex = jsonMapper.readValue((byte[]) response.getEntity(), QueryCapacityExceededException.class);
           }
           catch (IOException e) {
             throw new RuntimeException(e);
@@ -957,7 +1140,7 @@ public class QueryResourceTest
           Assert.assertEquals(QueryCapacityExceededException.STATUS_CODE, response.getStatus());
           QueryCapacityExceededException ex;
           try {
-            ex = JSON_MAPPER.readValue((byte[]) response.getEntity(), QueryCapacityExceededException.class);
+            ex = jsonMapper.readValue((byte[]) response.getEntity(), QueryCapacityExceededException.class);
           }
           catch (IOException e) {
             throw new RuntimeException(e);
@@ -1009,7 +1192,7 @@ public class QueryResourceTest
           Assert.assertEquals(QueryCapacityExceededException.STATUS_CODE, response.getStatus());
           QueryCapacityExceededException ex;
           try {
-            ex = JSON_MAPPER.readValue((byte[]) response.getEntity(), QueryCapacityExceededException.class);
+            ex = jsonMapper.readValue((byte[]) response.getEntity(), QueryCapacityExceededException.class);
           }
           catch (IOException e) {
             throw new RuntimeException(e);
@@ -1081,8 +1264,8 @@ public class QueryResourceTest
             AuthTestUtils.TEST_AUTHORIZER_MAPPER,
             Suppliers.ofInstance(new DefaultQueryConfig(ImmutableMap.of()))
         ),
-        JSON_MAPPER,
-        JSON_MAPPER,
+        jsonMapper,
+        smileMapper,
         scheduler,
         new AuthConfig(),
         null,
