@@ -28,16 +28,20 @@ import org.apache.druid.guice.ServerTypeConfig;
 import org.apache.druid.java.util.common.Intervals;
 import org.apache.druid.java.util.common.concurrent.Execs;
 import org.apache.druid.java.util.common.concurrent.ScheduledExecutorFactory;
-import org.apache.druid.java.util.common.logger.Logger;
+import org.apache.druid.java.util.emitter.EmittingLogger;
 import org.apache.druid.segment.IndexIO;
 import org.apache.druid.segment.TestHelper;
+import org.apache.druid.segment.loading.CacheTestSegmentCacheManager;
 import org.apache.druid.segment.loading.CacheTestSegmentLoader;
+import org.apache.druid.segment.loading.SegmentLoader;
 import org.apache.druid.segment.loading.SegmentLoaderConfig;
 import org.apache.druid.segment.loading.StorageLocationConfig;
 import org.apache.druid.server.SegmentManager;
+import org.apache.druid.server.coordination.SegmentLoadDropHandler.DataSegmentChangeRequestAndStatus;
+import org.apache.druid.server.coordination.SegmentLoadDropHandler.Status.STATE;
+import org.apache.druid.server.metrics.NoopServiceEmitter;
 import org.apache.druid.timeline.DataSegment;
 import org.apache.druid.timeline.partition.NoneShardSpec;
-import org.easymock.EasyMock;
 import org.joda.time.Interval;
 import org.junit.Assert;
 import org.junit.Before;
@@ -45,6 +49,8 @@ import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.ExpectedException;
 import org.junit.rules.TemporaryFolder;
+import org.mockito.ArgumentMatchers;
+import org.mockito.Mockito;
 
 import java.io.File;
 import java.io.IOException;
@@ -60,7 +66,6 @@ import java.util.concurrent.ScheduledFuture;
 import java.util.concurrent.ScheduledThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
-import java.util.stream.Collectors;
 
 /**
  */
@@ -68,17 +73,17 @@ public class SegmentLoadDropHandlerTest
 {
   public static final int COUNT = 50;
 
-  private static final Logger log = new Logger(ZkCoordinatorTest.class);
-
   private final ObjectMapper jsonMapper = TestHelper.makeJsonMapper();
 
   private SegmentLoadDropHandler segmentLoadDropHandler;
 
   private DataSegmentAnnouncer announcer;
   private File infoDir;
+  private TestStorageLocation testStorageLocation;
   private AtomicInteger announceCount;
   private ConcurrentSkipListSet<DataSegment> segmentsAnnouncedByMe;
-  private CacheTestSegmentLoader segmentLoader;
+  private CacheTestSegmentCacheManager segmentCacheManager;
+  private SegmentLoader segmentLoader;
   private SegmentManager segmentManager;
   private List<Runnable> scheduledRunnable;
   private SegmentLoaderConfig segmentLoaderConfig;
@@ -92,27 +97,29 @@ public class SegmentLoadDropHandlerTest
   @Rule
   public TemporaryFolder temporaryFolder = new TemporaryFolder();
 
+  public SegmentLoadDropHandlerTest()
+  {
+    EmittingLogger.registerEmitter(new NoopServiceEmitter());
+  }
+
   @Before
   public void setUp()
   {
     try {
-      infoDir = temporaryFolder.newFolder();
-      log.info("Creating tmp test files in [%s]", infoDir);
+      testStorageLocation = new TestStorageLocation(temporaryFolder);
+      infoDir = testStorageLocation.getInfoDir();
     }
     catch (IOException e) {
       throw new RuntimeException(e);
     }
 
     locations = Collections.singletonList(
-        new StorageLocationConfig(
-            infoDir,
-            100L,
-            100d
-        )
+        testStorageLocation.toStorageLocationConfig()
     );
 
     scheduledRunnable = new ArrayList<>();
 
+    segmentCacheManager = new CacheTestSegmentCacheManager();
     segmentLoader = new CacheTestSegmentLoader();
     segmentManager = new SegmentManager(segmentLoader);
     segmentsAnnouncedByMe = new ConcurrentSkipListSet<>();
@@ -159,7 +166,7 @@ public class SegmentLoadDropHandlerTest
       @Override
       public File getInfoDir()
       {
-        return infoDir;
+        return testStorageLocation.getInfoDir();
       }
 
       @Override
@@ -234,8 +241,9 @@ public class SegmentLoadDropHandlerTest
         jsonMapper,
         segmentLoaderConfig,
         announcer,
-        EasyMock.createNiceMock(DataSegmentServerAnnouncer.class),
+        Mockito.mock(DataSegmentServerAnnouncer.class),
         segmentManager,
+        segmentCacheManager,
         scheduledExecutorFactory.create(5, "SegmentLoadDropHandlerTest-[%d]"),
         new ServerTypeConfig(ServerType.HISTORICAL)
     );
@@ -270,7 +278,7 @@ public class SegmentLoadDropHandlerTest
     }
 
     Assert.assertTrue(segmentsAnnouncedByMe.contains(segment));
-    Assert.assertFalse("segment files shouldn't be deleted", segmentLoader.getSegmentsInTrash().contains(segment));
+    Assert.assertFalse("segment files shouldn't be deleted", segmentCacheManager.getSegmentsInTrash().contains(segment));
 
     segmentLoadDropHandler.stop();
   }
@@ -309,7 +317,7 @@ public class SegmentLoadDropHandlerTest
     }
 
     Assert.assertTrue(segmentsAnnouncedByMe.contains(segment));
-    Assert.assertFalse("segment files shouldn't be deleted", segmentLoader.getSegmentsInTrash().contains(segment));
+    Assert.assertFalse("segment files shouldn't be deleted", segmentCacheManager.getSegmentsInTrash().contains(segment));
 
     segmentLoadDropHandler.stop();
   }
@@ -335,10 +343,10 @@ public class SegmentLoadDropHandlerTest
     }
 
     for (DataSegment segment : segments) {
-      writeSegmentToCache(segment);
+      testStorageLocation.writeSegmentInfoToCache(segment);
     }
 
-    checkCache(segments);
+    testStorageLocation.checkInfoCache(segments);
     Assert.assertTrue(segmentManager.getDataSourceCounts().isEmpty());
     segmentLoadDropHandler.start();
     Assert.assertTrue(!segmentManager.getDataSourceCounts().isEmpty());
@@ -350,7 +358,7 @@ public class SegmentLoadDropHandlerTest
     segmentLoadDropHandler.stop();
 
     for (DataSegment segment : segments) {
-      deleteSegmentFromCache(segment);
+      testStorageLocation.deleteSegmentInfoFromCache(segment);
     }
 
     Assert.assertEquals(0, infoDir.listFiles().length);
@@ -372,52 +380,6 @@ public class SegmentLoadDropHandlerTest
     );
   }
 
-  private void writeSegmentToCache(final DataSegment segment)
-  {
-    if (!infoDir.exists()) {
-      infoDir.mkdir();
-    }
-
-    File segmentInfoCacheFile = new File(infoDir, segment.getId().toString());
-    try {
-      jsonMapper.writeValue(segmentInfoCacheFile, segment);
-    }
-    catch (IOException e) {
-      throw new RuntimeException(e);
-    }
-
-    Assert.assertTrue(segmentInfoCacheFile.exists());
-  }
-
-  private void deleteSegmentFromCache(final DataSegment segment)
-  {
-    File segmentInfoCacheFile = new File(infoDir, segment.getId().toString());
-    if (segmentInfoCacheFile.exists()) {
-      segmentInfoCacheFile.delete();
-    }
-
-    Assert.assertTrue(!segmentInfoCacheFile.exists());
-  }
-
-  private void checkCache(Set<DataSegment> expectedSegments)
-  {
-    Assert.assertTrue(infoDir.exists());
-    File[] files = infoDir.listFiles();
-
-    Set<DataSegment> segmentsInFiles = Arrays
-        .stream(files)
-        .map(file -> {
-          try {
-            return jsonMapper.readValue(file, DataSegment.class);
-          }
-          catch (IOException e) {
-            throw new RuntimeException(e);
-          }
-        })
-        .collect(Collectors.toSet());
-    Assert.assertEquals(expectedSegments, segmentsInFiles);
-  }
-
   @Test
   public void testStartStop() throws Exception
   {
@@ -449,7 +411,10 @@ public class SegmentLoadDropHandlerTest
             return 50;
           }
         },
-        announcer, EasyMock.createNiceMock(DataSegmentServerAnnouncer.class), segmentManager,
+        announcer,
+        Mockito.mock(DataSegmentServerAnnouncer.class),
+        segmentManager,
+        segmentCacheManager,
         new ServerTypeConfig(ServerType.HISTORICAL)
     );
 
@@ -463,10 +428,10 @@ public class SegmentLoadDropHandlerTest
     }
 
     for (DataSegment segment : segments) {
-      writeSegmentToCache(segment);
+      testStorageLocation.writeSegmentInfoToCache(segment);
     }
 
-    checkCache(segments);
+    testStorageLocation.checkInfoCache(segments);
     Assert.assertTrue(segmentManager.getDataSourceCounts().isEmpty());
 
     handler.start();
@@ -479,7 +444,7 @@ public class SegmentLoadDropHandlerTest
     handler.stop();
 
     for (DataSegment segment : segments) {
-      deleteSegmentFromCache(segment);
+      testStorageLocation.deleteSegmentInfoFromCache(segment);
     }
 
     Assert.assertEquals(0, infoDir.listFiles().length);
@@ -522,4 +487,47 @@ public class SegmentLoadDropHandlerTest
 
     segmentLoadDropHandler.stop();
   }
+
+  @Test(timeout = 60_000L)
+  public void testProcessBatchDuplicateLoadRequestsWhenFirstRequestFailsSecondRequestShouldSucceed() throws Exception
+  {
+    final SegmentManager segmentManager = Mockito.mock(SegmentManager.class);
+    Mockito.when(segmentManager.loadSegment(ArgumentMatchers.any(), ArgumentMatchers.anyBoolean(), ArgumentMatchers.any()))
+           .thenThrow(new RuntimeException("segment loading failure test"))
+           .thenReturn(true);
+    final SegmentLoadDropHandler segmentLoadDropHandler = new SegmentLoadDropHandler(
+        jsonMapper,
+        segmentLoaderConfig,
+        announcer,
+        Mockito.mock(DataSegmentServerAnnouncer.class),
+        segmentManager,
+        segmentCacheManager,
+        scheduledExecutorFactory.create(5, "SegmentLoadDropHandlerTest-[%d]"),
+        new ServerTypeConfig(ServerType.HISTORICAL)
+    );
+
+    segmentLoadDropHandler.start();
+
+    DataSegment segment1 = makeSegment("batchtest1", "1", Intervals.of("P1d/2011-04-01"));
+
+    List<DataSegmentChangeRequest> batch = ImmutableList.of(new SegmentChangeRequestLoad(segment1));
+
+    ListenableFuture<List<DataSegmentChangeRequestAndStatus>> future = segmentLoadDropHandler
+        .processBatch(batch);
+
+    for (Runnable runnable : scheduledRunnable) {
+      runnable.run();
+    }
+    List<DataSegmentChangeRequestAndStatus> result = future.get();
+    Assert.assertEquals(STATE.FAILED, result.get(0).getStatus().getState());
+
+    future = segmentLoadDropHandler.processBatch(batch);
+    for (Runnable runnable : scheduledRunnable) {
+      runnable.run();
+    }
+    result = future.get();
+    Assert.assertEquals(STATE.SUCCESS, result.get(0).getStatus().getState());
+
+    segmentLoadDropHandler.stop();
+  }
 }
