@@ -42,6 +42,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
 import java.util.concurrent.Executors;
@@ -232,12 +233,27 @@ public class BalanceSegmentsTest
     mockCoordinator(coordinator);
 
     BalancerStrategy strategy = EasyMock.createMock(BalancerStrategy.class);
-    EasyMock.expect(strategy.pickSegmentToMove(ImmutableList.of(new ServerHolder(druidServer2, peon2, false)), broadcastDatasources))
-            .andReturn(new BalancerSegmentHolder(druidServer2, segment3))
-            .andReturn(new BalancerSegmentHolder(druidServer2, segment4));
-    EasyMock.expect(strategy.pickSegmentToMove(EasyMock.anyObject(), EasyMock.anyObject()))
-            .andReturn(new BalancerSegmentHolder(druidServer1, segment1))
-            .andReturn(new BalancerSegmentHolder(druidServer1, segment2));
+    EasyMock.expect(
+        strategy.pickSegmentsToMove(
+            ImmutableList.of(
+                new ServerHolder(druidServer2, peon2, false)
+            ),
+            broadcastDatasources,
+            1,
+            100
+        )
+    ).andReturn(
+        ImmutableList.of(
+            new BalancerSegmentHolder(druidServer2, segment3),
+            new BalancerSegmentHolder(druidServer2, segment4)
+        ).iterator()
+    );
+
+    EasyMock.expect(strategy.pickSegmentsToMove(EasyMock.anyObject(), EasyMock.anyObject(), EasyMock.anyInt(), EasyMock.anyInt()))
+            .andReturn(
+                ImmutableList.of(
+                    new BalancerSegmentHolder(druidServer1, segment1),
+                    new BalancerSegmentHolder(druidServer1, segment2)).iterator());
 
     EasyMock.expect(strategy.findNewSegmentHomeBalancer(EasyMock.anyObject(), EasyMock.anyObject()))
             .andReturn(new ServerHolder(druidServer3, peon3))
@@ -300,11 +316,14 @@ public class BalanceSegmentsTest
     mockCoordinator(coordinator);
 
     BalancerStrategy strategy = EasyMock.createMock(BalancerStrategy.class);
-    EasyMock.expect(strategy.pickSegmentToMove(EasyMock.anyObject(), EasyMock.anyObject()))
-            .andReturn(new BalancerSegmentHolder(druidServer1, segment1))
-            .andReturn(new BalancerSegmentHolder(druidServer1, segment2))
-            .andReturn(new BalancerSegmentHolder(druidServer2, segment3))
-            .andReturn(new BalancerSegmentHolder(druidServer2, segment4));
+    EasyMock.expect(strategy.pickSegmentsToMove(EasyMock.anyObject(), EasyMock.anyObject(), EasyMock.anyInt(), EasyMock.anyInt()))
+            .andReturn(ImmutableList.of(new BalancerSegmentHolder(druidServer1, segment1)).iterator());
+    EasyMock.expect(strategy.pickSegmentsToMove(EasyMock.anyObject(), EasyMock.anyObject(), EasyMock.anyInt(), EasyMock.anyInt()))
+            .andReturn(
+                ImmutableList.of(
+                    new BalancerSegmentHolder(druidServer1, segment2),
+                    new BalancerSegmentHolder(druidServer2, segment3),
+                    new BalancerSegmentHolder(druidServer2, segment4)).iterator());
 
     EasyMock.expect(strategy.findNewSegmentHomeBalancer(EasyMock.anyObject(), EasyMock.anyObject()))
             .andReturn(new ServerHolder(druidServer3, peon3))
@@ -349,8 +368,8 @@ public class BalanceSegmentsTest
     mockCoordinator(coordinator);
 
     BalancerStrategy strategy = EasyMock.createMock(BalancerStrategy.class);
-    EasyMock.expect(strategy.pickSegmentToMove(EasyMock.anyObject(), EasyMock.anyObject()))
-            .andReturn(new BalancerSegmentHolder(druidServer1, segment1))
+    EasyMock.expect(strategy.pickSegmentsToMove(EasyMock.anyObject(), EasyMock.anyObject(), EasyMock.anyInt(), EasyMock.anyInt()))
+            .andReturn(ImmutableList.of(new BalancerSegmentHolder(druidServer1, segment1)).iterator())
             .anyTimes();
     EasyMock.expect(strategy.findNewSegmentHomeBalancer(EasyMock.anyObject(), EasyMock.anyObject())).andAnswer(() -> {
       List<ServerHolder> holders = (List<ServerHolder>) EasyMock.getCurrentArguments()[1];
@@ -384,8 +403,8 @@ public class BalanceSegmentsTest
 
     ServerHolder holder2 = new ServerHolder(druidServer2, peon2, false);
     BalancerStrategy strategy = EasyMock.createMock(BalancerStrategy.class);
-    EasyMock.expect(strategy.pickSegmentToMove(EasyMock.anyObject(), EasyMock.anyObject()))
-            .andReturn(new BalancerSegmentHolder(druidServer1, segment1))
+    EasyMock.expect(strategy.pickSegmentsToMove(EasyMock.anyObject(), EasyMock.anyObject(), EasyMock.anyInt(), EasyMock.anyInt()))
+            .andReturn(ImmutableList.of(new BalancerSegmentHolder(druidServer1, segment1)).iterator())
             .once();
     EasyMock.expect(strategy.findNewSegmentHomeBalancer(EasyMock.anyObject(), EasyMock.anyObject()))
             .andReturn(holder2)
@@ -527,6 +546,102 @@ public class BalanceSegmentsTest
     Assert.assertTrue(params.getCoordinatorStats().getTieredStat("movedCount", "normal") > 0);
   }
 
+  /**
+   * Testing that the dynamic coordinator config value, percentOfSegmentsToConsiderPerMove, is honored when calling
+   * out to pickSegmentToMove. This config limits the number of segments that are considered when looking for a segment
+   * to move.
+   */
+  @Test
+  public void testThatDynamicConfigIsHonoredWhenPickingSegmentToMove()
+  {
+    mockDruidServer(druidServer1, "1", "normal", 50L, 100L, Arrays.asList(segment1, segment2));
+    mockDruidServer(druidServer2, "2", "normal", 30L, 100L, Arrays.asList(segment3, segment4));
+    mockDruidServer(druidServer3, "3", "normal", 0L, 100L, Collections.emptyList());
+
+    EasyMock.replay(druidServer4);
+
+    mockCoordinator(coordinator);
+
+    BalancerStrategy strategy = EasyMock.createMock(BalancerStrategy.class);
+
+    // The first call for decommissioning servers
+    EasyMock.expect(
+        strategy.pickSegmentsToMove(
+            ImmutableList.of(),
+            broadcastDatasources,
+            1,
+            40
+        )
+    )
+            .andReturn(Collections.emptyIterator());
+
+    // The second call for the single non decommissioning server move
+    EasyMock.expect(
+        strategy.pickSegmentsToMove(
+            ImmutableList.of(
+                new ServerHolder(druidServer3, peon3, false),
+                new ServerHolder(druidServer2, peon2, false),
+                new ServerHolder(druidServer1, peon1, false)
+            ),
+            broadcastDatasources,
+            1,
+            40
+        )
+    )
+            .andReturn(ImmutableList.of(new BalancerSegmentHolder(druidServer2, segment3)).iterator());
+
+    EasyMock.expect(strategy.findNewSegmentHomeBalancer(EasyMock.anyObject(), EasyMock.anyObject()))
+            .andReturn(new ServerHolder(druidServer3, peon3))
+            .anyTimes();
+    EasyMock.replay(strategy);
+
+    DruidCoordinatorRuntimeParams params = defaultRuntimeParamsBuilder(
+        ImmutableList.of(druidServer1, druidServer2, druidServer3),
+        ImmutableList.of(peon1, peon2, peon3),
+        ImmutableList.of(false, false, false)
+    )
+        .withDynamicConfigs(
+            CoordinatorDynamicConfig.builder()
+                                    .withMaxSegmentsToMove(1)
+                                    .withPercentOfSegmentsToConsiderPerMove(40)
+                                    .build()
+        )
+        .withBalancerStrategy(strategy)
+        .withBroadcastDatasources(broadcastDatasources)
+        .build();
+
+    params = new BalanceSegmentsTester(coordinator).run(params);
+    Assert.assertEquals(1L, params.getCoordinatorStats().getTieredStat("movedCount", "normal"));
+    Assert.assertThat(
+        peon3.getSegmentsToLoad(),
+        Matchers.is(Matchers.equalTo(ImmutableSet.of(segment3)))
+    );
+  }
+
+  @Test
+  public void testUseBatchedSegmentSampler()
+  {
+    mockDruidServer(druidServer1, "1", "normal", 30L, 100L, segments);
+    mockDruidServer(druidServer2, "2", "normal", 0L, 100L, Collections.emptyList());
+    mockDruidServer(druidServer3, "3", "normal", 0L, 100L, Collections.emptyList());
+    mockDruidServer(druidServer4, "4", "normal", 0L, 100L, Collections.emptyList());
+
+    mockCoordinator(coordinator);
+
+    DruidCoordinatorRuntimeParams params = defaultRuntimeParamsBuilder(druidServers, peons)
+        .withDynamicConfigs(
+            CoordinatorDynamicConfig.builder()
+                                    .withMaxSegmentsToMove(2)
+                                    .withUseBatchedSegmentSampler(true)
+                                    .build()
+        )
+        .withBroadcastDatasources(broadcastDatasources)
+        .build();
+
+    params = new BalanceSegmentsTester(coordinator).run(params);
+    Assert.assertEquals(2L, params.getCoordinatorStats().getTieredStat("movedCount", "normal"));
+  }
+
   private DruidCoordinatorRuntimeParams.Builder defaultRuntimeParamsBuilder(
       List<ImmutableDruidServer> druidServers,
       List<LoadQueuePeon> peons
@@ -637,9 +752,14 @@ public class BalanceSegmentsTest
     }
 
     @Override
-    public BalancerSegmentHolder pickSegmentToMove(List<ServerHolder> serverHolders, Set<String> broadcastDatasources)
+    public Iterator<BalancerSegmentHolder> pickSegmentsToMove(
+        List<ServerHolder> serverHolders,
+        Set<String> broadcastDatasources,
+        int numberOfSegments,
+        double percentOfSegmentsToConsider
+    )
     {
-      return pickOrder.get(pickCounter.getAndIncrement() % pickOrder.size());
+      return pickOrder.iterator();
     }
 
     @Override
@@ -661,10 +781,20 @@ public class BalanceSegmentsTest
 
     // either decommissioning servers list or acitve ones (ie servers list is [2] or [1, 3])
     BalancerStrategy strategy = EasyMock.createMock(BalancerStrategy.class);
-    EasyMock.expect(strategy.pickSegmentToMove(ImmutableList.of(new ServerHolder(druidServer2, peon2, true)), broadcastDatasources))
-            .andReturn(new BalancerSegmentHolder(druidServer2, segment2));
-    EasyMock.expect(strategy.pickSegmentToMove(EasyMock.anyObject(), EasyMock.anyObject()))
-            .andReturn(new BalancerSegmentHolder(druidServer1, segment1));
+    EasyMock.expect(
+        strategy.pickSegmentsToMove(
+            ImmutableList.of(
+                new ServerHolder(druidServer2, peon2, true)
+            ),
+            broadcastDatasources,
+            1,
+            100
+        )
+    ).andReturn(
+        ImmutableList.of(new BalancerSegmentHolder(druidServer2, segment2)).iterator()
+    );
+    EasyMock.expect(strategy.pickSegmentsToMove(EasyMock.anyObject(), EasyMock.anyObject(), EasyMock.anyInt(), EasyMock.anyDouble()))
+            .andReturn(ImmutableList.of(new BalancerSegmentHolder(druidServer1, segment1)).iterator());
     EasyMock.expect(strategy.findNewSegmentHomeBalancer(EasyMock.anyObject(), EasyMock.anyObject()))
             .andReturn(new ServerHolder(druidServer3, peon3))
             .anyTimes();
