@@ -38,6 +38,7 @@ import org.apache.druid.client.SingleServerInventoryView;
 import org.apache.druid.common.config.JacksonConfigManager;
 import org.apache.druid.curator.CuratorTestBase;
 import org.apache.druid.curator.CuratorUtils;
+import org.apache.druid.curator.ZkEnablementConfig;
 import org.apache.druid.curator.discovery.NoopServiceAnnouncer;
 import org.apache.druid.discovery.DruidLeaderSelector;
 import org.apache.druid.jackson.DefaultObjectMapper;
@@ -51,6 +52,9 @@ import org.apache.druid.metadata.SegmentsMetadataManager;
 import org.apache.druid.server.DruidNode;
 import org.apache.druid.server.coordination.DruidServerMetadata;
 import org.apache.druid.server.coordination.ServerType;
+import org.apache.druid.server.coordinator.duty.CoordinatorCustomDuty;
+import org.apache.druid.server.coordinator.duty.CoordinatorCustomDutyGroup;
+import org.apache.druid.server.coordinator.duty.CoordinatorCustomDutyGroups;
 import org.apache.druid.server.coordinator.rules.ForeverBroadcastDistributionRule;
 import org.apache.druid.server.coordinator.rules.ForeverLoadRule;
 import org.apache.druid.server.coordinator.rules.IntervalLoadRule;
@@ -140,8 +144,18 @@ public class DruidCoordinatorTest extends CuratorTestBase
         new Duration(COORDINATOR_PERIOD),
         null,
         null,
+        null,
         new Duration(COORDINATOR_PERIOD),
         null,
+        null,
+        null,
+        null,
+        null,
+        null,
+        null,
+        null,
+        null,
+        null,
         10,
         new Duration("PT0s")
     );
@@ -188,7 +202,7 @@ public class DruidCoordinatorTest extends CuratorTestBase
         segmentsMetadataManager,
         serverInventoryView,
         metadataRuleManager,
-        curator,
+        () -> curator,
         serviceEmitter,
         scheduledExecutorFactory,
         null,
@@ -211,10 +225,13 @@ public class DruidCoordinatorTest extends CuratorTestBase
         druidNode,
         loadManagementPeons,
         null,
+        new HashSet<>(),
+        new CoordinatorCustomDutyGroups(ImmutableSet.of()),
         new CostBalancerStrategyFactory(),
         EasyMock.createNiceMock(LookupCoordinatorManager.class),
         new TestDruidLeaderSelector(),
-        null
+        null,
+        ZkEnablementConfig.ENABLED
     );
   }
 
@@ -410,6 +427,24 @@ public class DruidCoordinatorTest extends CuratorTestBase
     // The load rules asks for 2 replicas, therefore 1 replica should still be pending
     Assert.assertEquals(1L, underRepliicationCountsPerDataSource.getLong(dataSource));
 
+
+    Map<String, Object2LongMap<String>> underReplicationCountsPerDataSourcePerTierUsingClusterView =
+        coordinator.computeUnderReplicationCountsPerDataSourcePerTierUsingClusterView();
+    Assert.assertNotNull(underReplicationCountsPerDataSourcePerTier);
+    Assert.assertEquals(1, underReplicationCountsPerDataSourcePerTier.size());
+
+    Object2LongMap<String> underRepliicationCountsPerDataSourceUsingClusterView =
+        underReplicationCountsPerDataSourcePerTierUsingClusterView.get(tier);
+    Assert.assertNotNull(underRepliicationCountsPerDataSourceUsingClusterView);
+    Assert.assertEquals(1, underRepliicationCountsPerDataSourceUsingClusterView.size());
+    //noinspection deprecation
+    Assert.assertNotNull(underRepliicationCountsPerDataSourceUsingClusterView.get(dataSource));
+    // Simulated the adding of segment to druidServer during SegmentChangeRequestLoad event
+    // The load rules asks for 2 replicas, but only 1 historical server in cluster. Since computing using cluster view
+    // the segments are replicated as many times as they can be given state of cluster, therefore should not be
+    // under-replicated.
+    Assert.assertEquals(0L, underRepliicationCountsPerDataSourceUsingClusterView.getLong(dataSource));
+
     coordinator.stop();
     leaderUnannouncerLatch.await();
 
@@ -497,6 +532,12 @@ public class DruidCoordinatorTest extends CuratorTestBase
     Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTier.get(hotTierName).getLong(dataSource));
     Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTier.get(coldTierName).getLong(dataSource));
 
+    Map<String, Object2LongMap<String>> underReplicationCountsPerDataSourcePerTierUsingClusterView =
+        coordinator.computeUnderReplicationCountsPerDataSourcePerTierUsingClusterView();
+    Assert.assertEquals(2, underReplicationCountsPerDataSourcePerTierUsingClusterView.size());
+    Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTierUsingClusterView.get(hotTierName).getLong(dataSource));
+    Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTierUsingClusterView.get(coldTierName).getLong(dataSource));
+
     coordinator.stop();
     leaderUnannouncerLatch.await();
 
@@ -658,6 +699,14 @@ public class DruidCoordinatorTest extends CuratorTestBase
     Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTier.get(tierName1).getLong(dataSource));
     Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTier.get(tierName2).getLong(dataSource));
 
+    Map<String, Object2LongMap<String>> underReplicationCountsPerDataSourcePerTierUsingClusterView =
+        coordinator.computeUnderReplicationCountsPerDataSourcePerTierUsingClusterView();
+    Assert.assertEquals(4, underReplicationCountsPerDataSourcePerTierUsingClusterView.size());
+    Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTierUsingClusterView.get(hotTierName).getLong(dataSource));
+    Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTierUsingClusterView.get(coldTierName).getLong(dataSource));
+    Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTierUsingClusterView.get(tierName1).getLong(dataSource));
+    Assert.assertEquals(0L, underReplicationCountsPerDataSourcePerTierUsingClusterView.get(tierName2).getLong(dataSource));
+
     coordinator.stop();
     leaderUnannouncerLatch.await();
 
@@ -692,7 +741,7 @@ public class DruidCoordinatorTest extends CuratorTestBase
         null,
         null,
         null,
-        null,
+        () -> null,
         null,
         scheduledExecutorFactory,
         null,
@@ -701,12 +750,15 @@ public class DruidCoordinatorTest extends CuratorTestBase
         null,
         null,
         null,
+        new CoordinatorCustomDutyGroups(ImmutableSet.of()),
         null,
         null,
-        null
+        null,
+        null,
+        ZkEnablementConfig.ENABLED
     );
 
-    DruidCoordinator.DutiesRunnable duty = c.new DutiesRunnable(Collections.emptyList(), 0);
+    DruidCoordinator.DutiesRunnable duty = c.new DutiesRunnable(Collections.emptyList(), 0, "TEST");
     // before initialization
     Assert.assertEquals(0, c.getCachedBalancerThreadNumber());
     Assert.assertNull(c.getBalancerExec());
@@ -734,6 +786,129 @@ public class DruidCoordinatorTest extends CuratorTestBase
     Assert.assertFalse(firstExec == thirdExec);
   }
 
+  @Test(timeout = 3000)
+  public void testCoordinatorCustomDutyGroupsRunAsExpected() throws Exception
+  {
+    // Some nessesary setup to start the Coordinator
+    JacksonConfigManager configManager = EasyMock.createNiceMock(JacksonConfigManager.class);
+    EasyMock.expect(
+        configManager.watch(
+            EasyMock.eq(CoordinatorDynamicConfig.CONFIG_KEY),
+            EasyMock.anyObject(Class.class),
+            EasyMock.anyObject()
+        )
+    ).andReturn(new AtomicReference(CoordinatorDynamicConfig.builder().build())).anyTimes();
+    EasyMock.expect(
+        configManager.watch(
+            EasyMock.eq(CoordinatorCompactionConfig.CONFIG_KEY),
+            EasyMock.anyObject(Class.class),
+            EasyMock.anyObject()
+        )
+    ).andReturn(new AtomicReference(CoordinatorCompactionConfig.empty())).anyTimes();
+    EasyMock.replay(configManager);
+    EasyMock.expect(segmentsMetadataManager.isPollingDatabasePeriodically()).andReturn(true).anyTimes();
+    DruidDataSource dataSource = new DruidDataSource("dataSource1", Collections.emptyMap());
+    DataSegment dataSegment = new DataSegment(
+        "dataSource1",
+        Intervals.of("2010-01-01/P1D"),
+        "v1",
+        null,
+        null,
+        null,
+        null,
+        0x9,
+        0
+    );
+    dataSource.addSegment(dataSegment);
+    DataSourcesSnapshot dataSourcesSnapshot =
+        new DataSourcesSnapshot(ImmutableMap.of(dataSource.getName(), dataSource.toImmutableDruidDataSource()));
+    EasyMock
+        .expect(segmentsMetadataManager.getSnapshotOfDataSourcesWithAllUsedSegments())
+        .andReturn(dataSourcesSnapshot)
+        .anyTimes();
+    EasyMock.replay(segmentsMetadataManager);
+    EasyMock.expect(serverInventoryView.isStarted()).andReturn(true).anyTimes();
+    EasyMock.replay(serverInventoryView);
+
+    // Create CoordinatorCustomDutyGroups
+    // We will have two groups and each group has one duty
+    CountDownLatch latch1 = new CountDownLatch(1);
+    CoordinatorCustomDuty duty1 = new CoordinatorCustomDuty() {
+      @Override
+      public DruidCoordinatorRuntimeParams run(DruidCoordinatorRuntimeParams params)
+      {
+        latch1.countDown();
+        return params;
+      }
+    };
+    CoordinatorCustomDutyGroup group1 = new CoordinatorCustomDutyGroup("group1", Duration.standardSeconds(1), ImmutableList.of(duty1));
+
+    CountDownLatch latch2 = new CountDownLatch(1);
+    CoordinatorCustomDuty duty2 = new CoordinatorCustomDuty() {
+      @Override
+      public DruidCoordinatorRuntimeParams run(DruidCoordinatorRuntimeParams params)
+      {
+        latch2.countDown();
+        return params;
+      }
+    };
+    CoordinatorCustomDutyGroup group2 = new CoordinatorCustomDutyGroup("group2", Duration.standardSeconds(1), ImmutableList.of(duty2));
+    CoordinatorCustomDutyGroups groups = new CoordinatorCustomDutyGroups(ImmutableSet.of(group1, group2));
+
+    coordinator = new DruidCoordinator(
+        druidCoordinatorConfig,
+        new ZkPathsConfig()
+        {
+
+          @Override
+          public String getBase()
+          {
+            return "druid";
+          }
+        },
+        configManager,
+        segmentsMetadataManager,
+        serverInventoryView,
+        metadataRuleManager,
+        () -> curator,
+        serviceEmitter,
+        scheduledExecutorFactory,
+        null,
+        null,
+        new NoopServiceAnnouncer()
+        {
+          @Override
+          public void announce(DruidNode node)
+          {
+            // count down when this coordinator becomes the leader
+            leaderAnnouncerLatch.countDown();
+          }
+
+          @Override
+          public void unannounce(DruidNode node)
+          {
+            leaderUnannouncerLatch.countDown();
+          }
+        },
+        druidNode,
+        loadManagementPeons,
+        null,
+        new HashSet<>(),
+        groups,
+        new CostBalancerStrategyFactory(),
+        EasyMock.createNiceMock(LookupCoordinatorManager.class),
+        new TestDruidLeaderSelector(),
+        null,
+        ZkEnablementConfig.ENABLED
+    );
+    coordinator.start();
+
+    // Wait until group 1 duty ran for latch1 to countdown
+    latch1.await();
+    // Wait until group 2 duty ran for latch2 to countdown
+    latch2.await();
+  }
+
   private CountDownLatch createCountDownLatchAndSetPathChildrenCacheListenerWithLatch(int latchCount,
                                                                                       PathChildrenCache pathChildrenCache,
                                                                                       Map<String, DataSegment> segments,
