@@ -48,7 +48,10 @@ import org.apache.druid.server.initialization.jetty.JettyServerModule;
 import org.apache.druid.server.initialization.jetty.ServletFilterHolder;
 import org.apache.druid.server.security.AuthTestUtils;
 import org.apache.druid.server.security.AuthorizerMapper;
+import org.apache.druid.server.security.CustomCheckX509TrustManager;
+import org.apache.druid.server.security.TLSCertificateChecker;
 import org.eclipse.jetty.server.Server;
+import org.eclipse.jetty.util.ssl.SslContextFactory;
 import org.jboss.netty.handler.codec.http.HttpMethod;
 import org.joda.time.Duration;
 import org.junit.Assert;
@@ -56,7 +59,12 @@ import org.junit.Ignore;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
+import org.mockito.ArgumentCaptor;
+import org.mockito.ArgumentMatchers;
+import org.mockito.Mockito;
 
+import javax.net.ssl.SSLEngine;
+import javax.net.ssl.X509ExtendedTrustManager;
 import javax.servlet.DispatcherType;
 import javax.servlet.Filter;
 import javax.servlet.http.HttpServletResponse;
@@ -72,11 +80,12 @@ import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.nio.file.Path;
+import java.security.cert.X509Certificate;
 import java.util.EnumSet;
 import java.util.Locale;
 import java.util.Map;
 import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.Executor;
+import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.TimeUnit;
@@ -268,7 +277,7 @@ public class JettyTest extends BaseJettyTest
   public void testTimeouts() throws Exception
   {
     // test for request timeouts properly not locking up all threads
-    final Executor executor = Executors.newFixedThreadPool(100);
+    final ExecutorService executor = Executors.newFixedThreadPool(100);
     final AtomicLong count = new AtomicLong(0);
     final CountDownLatch latch = new CountDownLatch(1000);
     for (int i = 0; i < 10000; i++) {
@@ -507,6 +516,44 @@ public class JettyTest extends BaseJettyTest
     Assert.assertEquals(0, jsm.getActiveConnections());
   }
 
+  @Test
+  public void testCustomCheckX509TrustManagerSetEndpointIdentificationAlgorithmToNullWithValidateServerHostnamesSetToFalse() throws Exception
+  {
+    SslContextFactory.Server server = injector.getInstance(SslContextFactory.Server.class);
+    server.setEndpointIdentificationAlgorithm("HTTPS");
+    server.start();
+    SSLEngine sslEngine = server.newSSLEngine();
+    X509ExtendedTrustManager mockX509ExtendedTrustManager = Mockito.mock(X509ExtendedTrustManager.class);
+    TLSCertificateChecker mockTLSCertificateChecker = Mockito.mock(TLSCertificateChecker.class);
+    X509Certificate mockX509Certificate = Mockito.mock(X509Certificate.class);
+    String authType = "testAuthType";
+    X509Certificate[] chain = new X509Certificate[]{mockX509Certificate};
+
+    // The EndpointIdentificationAlgorithm should not be null as we set it to HTTPS earlier
+    Assert.assertNotNull(sslEngine.getSSLParameters().getEndpointIdentificationAlgorithm());
+
+    CustomCheckX509TrustManager customCheckX509TrustManager = new CustomCheckX509TrustManager(
+        mockX509ExtendedTrustManager,
+        mockTLSCertificateChecker,
+        false
+    );
+
+    customCheckX509TrustManager.checkServerTrusted(chain, authType, sslEngine);
+
+    ArgumentCaptor<SSLEngine> captor = ArgumentCaptor.forClass(SSLEngine.class);
+    Mockito.verify(mockTLSCertificateChecker).checkServer(
+        ArgumentMatchers.eq(chain),
+        ArgumentMatchers.eq(authType),
+        captor.capture(),
+        ArgumentMatchers.eq(mockX509ExtendedTrustManager)
+    );
+    SSLEngine transformedSSLEngine = captor.getValue();
+    // The EndpointIdentificationAlgorithm should be null or empty Stringas the CustomCheckX509TrustManager
+    // has validateServerHostnames set to false
+    String endpointIdentificationAlgorithm = transformedSSLEngine.getSSLParameters().getEndpointIdentificationAlgorithm();
+    Assert.assertTrue(endpointIdentificationAlgorithm == null || endpointIdentificationAlgorithm.isEmpty());
+  }
+
   private void waitForJettyServerModuleActiveConnectionsZero(JettyServerModule jsm) throws InterruptedException
   {
     // it can take a bit to close the connection, so maybe sleep for a while and hope it closes
