@@ -19,6 +19,7 @@
 
 package org.apache.druid.sql.calcite.expression;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Iterables;
@@ -47,9 +48,11 @@ import org.apache.calcite.sql.type.SqlOperandTypeInference;
 import org.apache.calcite.sql.type.SqlReturnTypeInference;
 import org.apache.calcite.sql.type.SqlTypeFamily;
 import org.apache.calcite.sql.type.SqlTypeName;
+import org.apache.calcite.sql.type.SqlTypeTransforms;
 import org.apache.calcite.util.Static;
 import org.apache.druid.java.util.common.IAE;
 import org.apache.druid.java.util.common.ISE;
+import org.apache.druid.java.util.common.StringUtils;
 import org.apache.druid.query.aggregation.PostAggregator;
 import org.apache.druid.query.aggregation.post.FieldAccessPostAggregator;
 import org.apache.druid.segment.column.RowSignature;
@@ -253,11 +256,12 @@ public class OperatorConversions
     }
 
     /**
-     * Sets the return type of the operator to "typeName", marked as non-nullable.
+     * Sets the return type of the operator to "typeName", marked as non-nullable. If this method is used it implies the
+     * operator should never, ever, return null.
      *
-     * One of {@link #returnTypeNonNull}, {@link #returnTypeNullable}, or
-     * {@link #returnTypeInference(SqlReturnTypeInference)} must be used before calling {@link #build()}. These methods
-     * cannot be mixed; you must call exactly one.
+     * One of {@link #returnTypeNonNull}, {@link #returnTypeNullable}, {@link #returnTypeCascadeNullable(SqlTypeName)}
+     * {@link #returnTypeNullableArray}, or {@link #returnTypeInference(SqlReturnTypeInference)} must be used before
+     * calling {@link #build()}. These methods cannot be mixed; you must call exactly one.
      */
     public OperatorBuilder returnTypeNonNull(final SqlTypeName typeName)
     {
@@ -272,9 +276,9 @@ public class OperatorConversions
     /**
      * Sets the return type of the operator to "typeName", marked as nullable.
      *
-     * One of {@link #returnTypeNonNull}, {@link #returnTypeNullable}, or
-     * {@link #returnTypeInference(SqlReturnTypeInference)} must be used before calling {@link #build()}. These methods
-     * cannot be mixed; you must call exactly one.
+     * One of {@link #returnTypeNonNull}, {@link #returnTypeNullable}, {@link #returnTypeCascadeNullable(SqlTypeName)}
+     * {@link #returnTypeNullableArray}, or {@link #returnTypeInference(SqlReturnTypeInference)} must be used before
+     * calling {@link #build()}. These methods cannot be mixed; you must call exactly one.
      */
     public OperatorBuilder returnTypeNullable(final SqlTypeName typeName)
     {
@@ -286,12 +290,44 @@ public class OperatorConversions
       return this;
     }
 
+    /**
+     * Sets the return type of the operator to "typeName", marked as nullable if any of its operands are nullable.
+     *
+     * One of {@link #returnTypeNonNull}, {@link #returnTypeNullable}, {@link #returnTypeCascadeNullable(SqlTypeName)}
+     * {@link #returnTypeNullableArray}, or {@link #returnTypeInference(SqlReturnTypeInference)} must be used before
+     * calling {@link #build()}. These methods cannot be mixed; you must call exactly one.
+     */
+    public OperatorBuilder returnTypeCascadeNullable(final SqlTypeName typeName)
+    {
+      Preconditions.checkState(this.returnTypeInference == null, "Cannot set return type multiple times");
+      this.returnTypeInference = ReturnTypes.cascade(ReturnTypes.explicit(typeName), SqlTypeTransforms.TO_NULLABLE);
+      return this;
+    }
+
+    /**
+     * Sets the return type of the operator to an array type with elements of "typeName", marked as nullable.
+     *
+     * One of {@link #returnTypeNonNull}, {@link #returnTypeNullable}, {@link #returnTypeCascadeNullable(SqlTypeName)}
+     * {@link #returnTypeNullableArray}, or {@link #returnTypeInference(SqlReturnTypeInference)} must be used before
+     * calling {@link #build()}. These methods cannot be mixed; you must call exactly one.
+     */
+    public OperatorBuilder returnTypeNullableArray(final SqlTypeName elementTypeName)
+    {
+      Preconditions.checkState(this.returnTypeInference == null, "Cannot set return type multiple times");
+
+      this.returnTypeInference = ReturnTypes.explicit(
+          factory -> Calcites.createSqlArrayTypeWithNullability(factory, elementTypeName, true)
+      );
+      return this;
+    }
+
+
     /**
      * Provides customized return type inference logic.
      *
-     * One of {@link #returnTypeNonNull}, {@link #returnTypeNullable}, or
-     * {@link #returnTypeInference(SqlReturnTypeInference)} must be used before calling {@link #build()}. These methods
-     * cannot be mixed; you must call exactly one.
+     * One of {@link #returnTypeNonNull}, {@link #returnTypeNullable}, {@link #returnTypeCascadeNullable(SqlTypeName)}
+     * {@link #returnTypeNullableArray}, or {@link #returnTypeInference(SqlReturnTypeInference)} must be used before
+     * calling {@link #build()}. These methods cannot be mixed; you must call exactly one.
      */
     public OperatorBuilder returnTypeInference(final SqlReturnTypeInference returnTypeInference)
     {
@@ -499,13 +535,15 @@ public class OperatorConversions
    * Operand type checker that is used in 'simple' situations: there are a particular number of operands, with
    * particular types, some of which may be optional or nullable, and some of which may be required to be literals.
    */
-  private static class DefaultOperandTypeChecker implements SqlOperandTypeChecker
+  @VisibleForTesting
+  static class DefaultOperandTypeChecker implements SqlOperandTypeChecker
   {
     private final List<SqlTypeFamily> operandTypes;
     private final int requiredOperands;
     private final IntSet nullableOperands;
     private final IntSet literalOperands;
 
+    @VisibleForTesting
     DefaultOperandTypeChecker(
         final List<SqlTypeFamily> operandTypes,
         final int requiredOperands,
@@ -612,4 +650,43 @@ public class OperatorConversions
       return false;
     }
   }
+
+  public static DirectOperatorConversion druidUnaryLongFn(String sqlOperator, String druidFunctionName)
+  {
+    return new DirectOperatorConversion(
+        operatorBuilder(sqlOperator)
+            .requiredOperands(1)
+            .operandTypes(SqlTypeFamily.NUMERIC)
+            .returnTypeNullable(SqlTypeName.BIGINT)
+            .functionCategory(SqlFunctionCategory.NUMERIC)
+            .build(),
+        druidFunctionName
+    );
+  }
+
+  public static DirectOperatorConversion druidBinaryLongFn(String sqlOperator, String druidFunctionName)
+  {
+    return new DirectOperatorConversion(
+        operatorBuilder(sqlOperator)
+            .requiredOperands(2)
+            .operandTypes(SqlTypeFamily.NUMERIC, SqlTypeFamily.NUMERIC)
+            .returnTypeNullable(SqlTypeName.BIGINT)
+            .functionCategory(SqlFunctionCategory.NUMERIC)
+            .build(),
+        druidFunctionName
+    );
+  }
+
+  public static DirectOperatorConversion druidUnaryDoubleFn(String sqlOperator, String druidFunctionName)
+  {
+    return new DirectOperatorConversion(
+        operatorBuilder(StringUtils.toUpperCase(sqlOperator))
+            .requiredOperands(1)
+            .operandTypes(SqlTypeFamily.NUMERIC)
+            .returnTypeNullable(SqlTypeName.DOUBLE)
+            .functionCategory(SqlFunctionCategory.NUMERIC)
+            .build(),
+        druidFunctionName
+    );
+  }
 }
