@@ -30,7 +30,9 @@ import java.util.Objects;
 public class PlannerConfig
 {
   public static final String CTX_KEY_USE_APPROXIMATE_COUNT_DISTINCT = "useApproximateCountDistinct";
+  public static final String CTX_KEY_USE_GROUPING_SET_FOR_EXACT_DISTINCT = "useGroupingSetForExactDistinct";
   public static final String CTX_KEY_USE_APPROXIMATE_TOPN = "useApproximateTopN";
+  public static final String CTX_COMPUTE_INNER_JOIN_COST_AS_FILTER = "computeInnerJoinCostAsFilter";
 
   @JsonProperty
   private Period metadataRefreshPeriod = new Period("PT1M");
@@ -59,6 +61,12 @@ public class PlannerConfig
   @JsonProperty
   private long metadataSegmentPollPeriod = 60000;
 
+  @JsonProperty
+  private boolean useGroupingSetForExactDistinct = false;
+
+  @JsonProperty
+  private boolean computeInnerJoinCostAsFilter = true;
+
   public long getMetadataSegmentPollPeriod()
   {
     return metadataSegmentPollPeriod;
@@ -86,6 +94,11 @@ public class PlannerConfig
     return useApproximateCountDistinct;
   }
 
+  public boolean isUseGroupingSetForExactDistinct()
+  {
+    return useGroupingSetForExactDistinct;
+  }
+
   public boolean isUseApproximateTopN()
   {
     return useApproximateTopN;
@@ -111,6 +124,11 @@ public class PlannerConfig
     return serializeComplexValues;
   }
 
+  public boolean isComputeInnerJoinCostAsFilter()
+  {
+    return computeInnerJoinCostAsFilter;
+  }
+
   public PlannerConfig withOverrides(final Map<String, Object> context)
   {
     if (context == null) {
@@ -125,6 +143,11 @@ public class PlannerConfig
         CTX_KEY_USE_APPROXIMATE_COUNT_DISTINCT,
         isUseApproximateCountDistinct()
     );
+    newConfig.useGroupingSetForExactDistinct = getContextBoolean(
+        context,
+        CTX_KEY_USE_GROUPING_SET_FOR_EXACT_DISTINCT,
+        isUseGroupingSetForExactDistinct()
+    );
     newConfig.useApproximateTopN = getContextBoolean(
         context,
         CTX_KEY_USE_APPROXIMATE_TOPN,
@@ -136,6 +159,9 @@ public class PlannerConfig
     newConfig.metadataSegmentCacheEnable = isMetadataSegmentCacheEnable();
     newConfig.metadataSegmentPollPeriod = getMetadataSegmentPollPeriod();
     newConfig.serializeComplexValues = shouldSerializeComplexValues();
+    newConfig.computeInnerJoinCostAsFilter = getContextBoolean(context,
+                                                               CTX_COMPUTE_INNER_JOIN_COST_AS_FILTER,
+                                                               computeInnerJoinCostAsFilter);
     return newConfig;
   }
 
