@@ -19,6 +19,7 @@
 
 package org.apache.druid.sql.calcite.schema;
 
+import com.fasterxml.jackson.core.JsonProcessingException;
 import com.fasterxml.jackson.core.type.TypeReference;
 import com.fasterxml.jackson.databind.JavaType;
 import com.fasterxml.jackson.databind.ObjectMapper;
@@ -28,6 +29,7 @@ import com.google.common.collect.FluentIterable;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Maps;
+import com.google.common.collect.Sets;
 import com.google.common.net.HostAndPort;
 import com.google.common.util.concurrent.Futures;
 import com.google.inject.Inject;
@@ -49,6 +51,7 @@ import org.apache.druid.client.JsonParserIterator;
 import org.apache.druid.client.TimelineServerView;
 import org.apache.druid.client.coordinator.Coordinator;
 import org.apache.druid.client.indexing.IndexingService;
+import org.apache.druid.common.config.NullHandling;
 import org.apache.druid.discovery.DataNodeService;
 import org.apache.druid.discovery.DiscoveryDruidNode;
 import org.apache.druid.discovery.DruidLeaderClient;
@@ -88,7 +91,6 @@ import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
@@ -142,7 +144,7 @@ public class SystemSchema extends AbstractSchema
       .add("is_available", ValueType.LONG)
       .add("is_realtime", ValueType.LONG)
       .add("is_overshadowed", ValueType.LONG)
-      .add("shardSpec", ValueType.STRING)
+      .add("shard_spec", ValueType.STRING)
       .add("dimensions", ValueType.STRING)
       .add("metrics", ValueType.STRING)
       .add("last_compaction_state", ValueType.STRING)
@@ -158,6 +160,7 @@ public class SystemSchema extends AbstractSchema
       .add("tier", ValueType.STRING)
       .add("curr_size", ValueType.LONG)
       .add("max_size", ValueType.LONG)
+      .add("is_leader", ValueType.LONG)
       .build();
 
   static final RowSignature SERVER_SEGMENTS_SIGNATURE = RowSignature
@@ -213,8 +216,8 @@ public class SystemSchema extends AbstractSchema
   {
     Preconditions.checkNotNull(serverView, "serverView");
     this.tableMap = ImmutableMap.of(
-        SEGMENTS_TABLE, new SegmentsTable(druidSchema, metadataView, authorizerMapper),
-        SERVERS_TABLE, new ServersTable(druidNodeDiscoveryProvider, serverInventoryView, authorizerMapper),
+        SEGMENTS_TABLE, new SegmentsTable(druidSchema, metadataView, jsonMapper, authorizerMapper),
+        SERVERS_TABLE, new ServersTable(druidNodeDiscoveryProvider, serverInventoryView, authorizerMapper, overlordDruidLeaderClient, coordinatorDruidLeaderClient),
         SERVER_SEGMENTS_TABLE, new ServerSegmentsTable(serverView, authorizerMapper),
         TASKS_TABLE, new TasksTable(overlordDruidLeaderClient, jsonMapper, authorizerMapper),
         SUPERVISOR_TABLE, new SupervisorsTable(overlordDruidLeaderClient, jsonMapper, authorizerMapper)
@@ -233,17 +236,20 @@ public class SystemSchema extends AbstractSchema
   static class SegmentsTable extends AbstractTable implements ScannableTable
   {
     private final DruidSchema druidSchema;
+    private final ObjectMapper jsonMapper;
     private final AuthorizerMapper authorizerMapper;
     private final MetadataSegmentView metadataView;
 
     public SegmentsTable(
         DruidSchema druidSchemna,
         MetadataSegmentView metadataView,
+        ObjectMapper jsonMapper,
         AuthorizerMapper authorizerMapper
     )
     {
       this.druidSchema = druidSchemna;
       this.metadataView = metadataView;
+      this.jsonMapper = jsonMapper;
       this.authorizerMapper = authorizerMapper;
     }
 
@@ -281,7 +287,7 @@ public class SystemSchema extends AbstractSchema
       // Coordinator.
       final Iterator<SegmentWithOvershadowedStatus> metadataStoreSegments = metadataView.getPublishedSegments();
 
-      final Set<SegmentId> segmentsAlreadySeen = new HashSet<>();
+      final Set<SegmentId> segmentsAlreadySeen = Sets.newHashSetWithExpectedSize(druidSchema.getTotalSegments());
 
       final FluentIterable<Object[]> publishedSegments = FluentIterable
           .from(() -> getAuthorizedPublishedSegments(metadataStoreSegments, root))
@@ -296,25 +302,30 @@ public class SystemSchema extends AbstractSchema
               isAvailable = partialSegmentData.isAvailable();
               isRealtime = partialSegmentData.isRealtime();
             }
-            return new Object[]{
-                segment.getId(),
-                segment.getDataSource(),
-                segment.getInterval().getStart().toString(),
-                segment.getInterval().getEnd().toString(),
-                segment.getSize(),
-                segment.getVersion(),
-                (long) segment.getShardSpec().getPartitionNum(),
-                numReplicas,
-                numRows,
-                IS_PUBLISHED_TRUE, //is_published is true for published segments
-                isAvailable,
-                isRealtime,
-                val.isOvershadowed() ? IS_OVERSHADOWED_TRUE : IS_OVERSHADOWED_FALSE,
-                segment.getShardSpec(),
-                segment.getDimensions(),
-                segment.getMetrics(),
-                segment.getLastCompactionState()
-            };
+            try {
+              return new Object[]{
+                  segment.getId(),
+                  segment.getDataSource(),
+                  segment.getInterval().getStart().toString(),
+                  segment.getInterval().getEnd().toString(),
+                  segment.getSize(),
+                  segment.getVersion(),
+                  (long) segment.getShardSpec().getPartitionNum(),
+                  numReplicas,
+                  numRows,
+                  IS_PUBLISHED_TRUE, //is_published is true for published segments
+                  isAvailable,
+                  isRealtime,
+                  val.isOvershadowed() ? IS_OVERSHADOWED_TRUE : IS_OVERSHADOWED_FALSE,
+                  segment.getShardSpec() == null ? null : jsonMapper.writeValueAsString(segment.getShardSpec()),
+                  segment.getDimensions() == null ? null : jsonMapper.writeValueAsString(segment.getDimensions()),
+                  segment.getMetrics() == null ? null : jsonMapper.writeValueAsString(segment.getMetrics()),
+                  segment.getLastCompactionState() == null ? null : jsonMapper.writeValueAsString(segment.getLastCompactionState())
+              };
+            }
+            catch (JsonProcessingException e) {
+              throw new RuntimeException(e);
+            }
           });
 
       final FluentIterable<Object[]> availableSegments = FluentIterable
@@ -328,26 +339,33 @@ public class SystemSchema extends AbstractSchema
             }
             final PartialSegmentData partialSegmentData = partialSegmentDataMap.get(val.getKey());
             final long numReplicas = partialSegmentData == null ? 0L : partialSegmentData.getNumReplicas();
-            return new Object[]{
-                val.getKey(),
-                val.getKey().getDataSource(),
-                val.getKey().getInterval().getStart().toString(),
-                val.getKey().getInterval().getEnd().toString(),
-                val.getValue().getSegment().getSize(),
-                val.getKey().getVersion(),
-                (long) val.getValue().getSegment().getShardSpec().getPartitionNum(),
-                numReplicas,
-                val.getValue().getNumRows(),
-                IS_PUBLISHED_FALSE, // is_published is false for unpublished segments
-                // is_available is assumed to be always true for segments announced by historicals or realtime tasks
-                IS_AVAILABLE_TRUE,
-                val.getValue().isRealtime(),
-                IS_OVERSHADOWED_FALSE, // there is an assumption here that unpublished segments are never overshadowed
-                val.getValue().getSegment().getShardSpec(),
-                val.getValue().getSegment().getDimensions(),
-                val.getValue().getSegment().getMetrics(),
-                null // unpublished segments from realtime tasks will not be compacted yet
-            };
+            try {
+              return new Object[]{
+                  val.getKey(),
+                  val.getKey().getDataSource(),
+                  val.getKey().getInterval().getStart().toString(),
+                  val.getKey().getInterval().getEnd().toString(),
+                  val.getValue().getSegment().getSize(),
+                  val.getKey().getVersion(),
+                  (long) val.getValue().getSegment().getShardSpec().getPartitionNum(),
+                  numReplicas,
+                  val.getValue().getNumRows(),
+                  IS_PUBLISHED_FALSE,
+                  // is_published is false for unpublished segments
+                  // is_available is assumed to be always true for segments announced by historicals or realtime tasks
+                  IS_AVAILABLE_TRUE,
+                  val.getValue().isRealtime(),
+                  IS_OVERSHADOWED_FALSE,
+                  // there is an assumption here that unpublished segments are never overshadowed
+                  val.getValue().getSegment().getShardSpec() == null ? null : jsonMapper.writeValueAsString(val.getValue().getSegment().getShardSpec()),
+                  val.getValue().getSegment().getDimensions() == null ? null : jsonMapper.writeValueAsString(val.getValue().getSegment().getDimensions()),
+                  val.getValue().getSegment().getMetrics() == null ? null : jsonMapper.writeValueAsString(val.getValue().getSegment().getMetrics()),
+                  null // unpublished segments from realtime tasks will not be compacted yet
+              };
+            }
+            catch (JsonProcessingException e) {
+              throw new RuntimeException(e);
+            }
           });
 
       final Iterable<Object[]> allSegments = Iterables.unmodifiableIterable(
@@ -463,16 +481,22 @@ public class SystemSchema extends AbstractSchema
     private final AuthorizerMapper authorizerMapper;
     private final DruidNodeDiscoveryProvider druidNodeDiscoveryProvider;
     private final InventoryView serverInventoryView;
+    private final DruidLeaderClient overlordLeaderClient;
+    private final DruidLeaderClient coordinatorLeaderClient;
 
     public ServersTable(
         DruidNodeDiscoveryProvider druidNodeDiscoveryProvider,
         InventoryView serverInventoryView,
-        AuthorizerMapper authorizerMapper
+        AuthorizerMapper authorizerMapper,
+        DruidLeaderClient overlordLeaderClient,
+        DruidLeaderClient coordinatorLeaderClient
     )
     {
       this.authorizerMapper = authorizerMapper;
       this.druidNodeDiscoveryProvider = druidNodeDiscoveryProvider;
       this.serverInventoryView = serverInventoryView;
+      this.overlordLeaderClient = overlordLeaderClient;
+      this.coordinatorLeaderClient = coordinatorLeaderClient;
     }
 
     @Override
@@ -497,17 +521,30 @@ public class SystemSchema extends AbstractSchema
       );
       checkStateReadAccessForServers(authenticationResult, authorizerMapper);
 
+      String tmpCoordinatorLeader = "";
+      String tmpOverlordLeader = "";
+      try {
+        tmpCoordinatorLeader = coordinatorLeaderClient.findCurrentLeader();
+        tmpOverlordLeader = overlordLeaderClient.findCurrentLeader();
+      }
+      catch (ISE ignored) {
+        // no reason to kill the results if something is sad and there are no leaders
+      }
+      final String coordinatorLeader = tmpCoordinatorLeader;
+      final String overlordLeader = tmpOverlordLeader;
+
       final FluentIterable<Object[]> results = FluentIterable
           .from(() -> druidServers)
           .transform((DiscoveryDruidNode discoveryDruidNode) -> {
             //noinspection ConstantConditions
             final boolean isDiscoverableDataServer = isDiscoverableDataServer(discoveryDruidNode);
+            final NodeRole serverRole = discoveryDruidNode.getNodeRole();
 
             if (isDiscoverableDataServer) {
               final DruidServer druidServer = serverInventoryView.getInventoryValue(
                   discoveryDruidNode.getDruidNode().getHostAndPortToUse()
               );
-              if (druidServer != null || discoveryDruidNode.getNodeRole().equals(NodeRole.HISTORICAL)) {
+              if (druidServer != null || NodeRole.HISTORICAL.equals(serverRole)) {
                 // Build a row for the data server if that server is in the server view, or the node type is historical.
                 // The historicals are usually supposed to be found in the server view. If some historicals are
                 // missing, it could mean that there are some problems in them to announce themselves. We just fill
@@ -516,6 +553,16 @@ public class SystemSchema extends AbstractSchema
               } else {
                 return buildRowForNonDataServer(discoveryDruidNode);
               }
+            } else if (NodeRole.COORDINATOR.equals(serverRole)) {
+              return buildRowForNonDataServerWithLeadership(
+                  discoveryDruidNode,
+                  coordinatorLeader.contains(discoveryDruidNode.getDruidNode().getHostAndPortToUse())
+              );
+            } else if (NodeRole.OVERLORD.equals(serverRole)) {
+              return buildRowForNonDataServerWithLeadership(
+                  discoveryDruidNode,
+                  overlordLeader.contains(discoveryDruidNode.getDruidNode().getHostAndPortToUse())
+              );
             } else {
               return buildRowForNonDataServer(discoveryDruidNode);
             }
@@ -523,6 +570,7 @@ public class SystemSchema extends AbstractSchema
       return Linq4j.asEnumerable(results);
     }
 
+
     /**
      * Returns a row for all node types which don't serve data. The returned row contains only static information.
      */
@@ -537,7 +585,27 @@ public class SystemSchema extends AbstractSchema
           StringUtils.toLowerCase(discoveryDruidNode.getNodeRole().toString()),
           null,
           UNKNOWN_SIZE,
-          UNKNOWN_SIZE
+          UNKNOWN_SIZE,
+          NullHandling.defaultLongValue()
+      };
+    }
+
+    /**
+     * Returns a row for all node types which don't serve data. The returned row contains only static information.
+     */
+    private static Object[] buildRowForNonDataServerWithLeadership(DiscoveryDruidNode discoveryDruidNode, boolean isLeader)
+    {
+      final DruidNode node = discoveryDruidNode.getDruidNode();
+      return new Object[]{
+          node.getHostAndPortToUse(),
+          node.getHost(),
+          (long) node.getPlaintextPort(),
+          (long) node.getTlsPort(),
+          StringUtils.toLowerCase(discoveryDruidNode.getNodeRole().toString()),
+          null,
+          UNKNOWN_SIZE,
+          UNKNOWN_SIZE,
+          isLeader ? 1L : 0L
       };
     }
 
@@ -570,7 +638,8 @@ public class SystemSchema extends AbstractSchema
           StringUtils.toLowerCase(discoveryDruidNode.getNodeRole().toString()),
           druidServerToUse.getTier(),
           currentSize,
-          druidServerToUse.getMaxSize()
+          druidServerToUse.getMaxSize(),
+          NullHandling.defaultLongValue()
       };
     }
 
