@@ -40,6 +40,7 @@ import org.apache.druid.client.ImmutableDruidServer;
 import org.apache.druid.client.InventoryView;
 import org.apache.druid.client.ServerInventoryView;
 import org.apache.druid.client.TimelineServerView;
+import org.apache.druid.common.config.NullHandling;
 import org.apache.druid.data.input.InputRow;
 import org.apache.druid.discovery.DataNodeService;
 import org.apache.druid.discovery.DiscoveryDruidNode;
@@ -89,13 +90,11 @@ import org.apache.druid.sql.calcite.util.CalciteTestBase;
 import org.apache.druid.sql.calcite.util.CalciteTests;
 import org.apache.druid.sql.calcite.util.SpecificSegmentsQuerySegmentWalker;
 import org.apache.druid.sql.calcite.util.TestServerInventoryView;
-import org.apache.druid.sql.calcite.view.NoopViewManager;
 import org.apache.druid.timeline.CompactionState;
 import org.apache.druid.timeline.DataSegment;
 import org.apache.druid.timeline.SegmentId;
 import org.apache.druid.timeline.SegmentWithOvershadowedStatus;
 import org.apache.druid.timeline.partition.NumberedShardSpec;
-import org.apache.druid.timeline.partition.ShardSpec;
 import org.easymock.EasyMock;
 import org.jboss.netty.handler.codec.http.DefaultHttpResponse;
 import org.jboss.netty.handler.codec.http.HttpMethod;
@@ -146,6 +145,8 @@ public class SystemSchemaTest extends CalciteTestBase
   private SystemSchema schema;
   private SpecificSegmentsQuerySegmentWalker walker;
   private DruidLeaderClient client;
+  private DruidLeaderClient coordinatorClient;
+  private DruidLeaderClient overlordClient;
   private TimelineServerView serverView;
   private ObjectMapper mapper;
   private StringFullResponseHolder responseHolder;
@@ -180,6 +181,8 @@ public class SystemSchemaTest extends CalciteTestBase
   {
     serverView = EasyMock.createNiceMock(TimelineServerView.class);
     client = EasyMock.createMock(DruidLeaderClient.class);
+    coordinatorClient = EasyMock.createMock(DruidLeaderClient.class);
+    overlordClient = EasyMock.createMock(DruidLeaderClient.class);
     mapper = TestHelper.makeJsonMapper();
     responseHolder = EasyMock.createMock(StringFullResponseHolder.class);
     responseHandler = EasyMock.createMockBuilder(BytesAccumulatingResponseHandler.class)
@@ -252,7 +255,6 @@ public class SystemSchemaTest extends CalciteTestBase
         new SegmentManager(EasyMock.createMock(SegmentLoader.class)),
         new MapJoinableFactory(ImmutableSet.of(), ImmutableMap.of()),
         PLANNER_CONFIG_DEFAULT,
-        new NoopViewManager(),
         new NoopEscalator()
     );
     druidSchema.start();
@@ -275,7 +277,8 @@ public class SystemSchemaTest extends CalciteTestBase
 
   private final CompactionState expectedCompactionState = new CompactionState(
       new DynamicPartitionsSpec(null, null),
-      Collections.singletonMap("test", "map")
+      Collections.singletonMap("test", "map"),
+      Collections.singletonMap("test2", "map2")
   );
 
   private final DataSegment publishedCompactedSegment1 = new DataSegment(
@@ -379,12 +382,24 @@ public class SystemSchemaTest extends CalciteTestBase
       ImmutableMap.of()
   );
 
+  private final DiscoveryDruidNode coordinator2 = new DiscoveryDruidNode(
+      new DruidNode("s1", "localhost", false, 8181, null, true, false),
+      NodeRole.COORDINATOR,
+      ImmutableMap.of()
+  );
+
   private final DiscoveryDruidNode overlord = new DiscoveryDruidNode(
       new DruidNode("s2", "localhost", false, 8090, null, true, false),
       NodeRole.OVERLORD,
       ImmutableMap.of()
   );
 
+  private final DiscoveryDruidNode overlord2 = new DiscoveryDruidNode(
+      new DruidNode("s2", "localhost", false, 8190, null, true, false),
+      NodeRole.OVERLORD,
+      ImmutableMap.of()
+  );
+
   private final DiscoveryDruidNode broker1 = new DiscoveryDruidNode(
       new DruidNode("s3", "localhost", false, 8082, null, true, false),
       NodeRole.BROKER,
@@ -517,15 +532,15 @@ public class SystemSchemaTest extends CalciteTestBase
     final SystemSchema.ServersTable serversTable = (SystemSchema.ServersTable) schema.getTableMap().get("servers");
     final RelDataType serverRowType = serversTable.getRowType(new JavaTypeFactoryImpl());
     final List<RelDataTypeField> serverFields = serverRowType.getFieldList();
-    Assert.assertEquals(8, serverFields.size());
+    Assert.assertEquals(9, serverFields.size());
     Assert.assertEquals("server", serverFields.get(0).getName());
     Assert.assertEquals(SqlTypeName.VARCHAR, serverFields.get(0).getType().getSqlTypeName());
   }
 
   @Test
-  public void testSegmentsTable()
+  public void testSegmentsTable() throws Exception
   {
-    final SegmentsTable segmentsTable = new SegmentsTable(druidSchema, metadataView, authMapper);
+    final SegmentsTable segmentsTable = new SegmentsTable(druidSchema, metadataView, new ObjectMapper(), authMapper);
     final Set<SegmentWithOvershadowedStatus> publishedSegments = new HashSet<>(Arrays.asList(
         new SegmentWithOvershadowedStatus(publishedCompactedSegment1, true),
         new SegmentWithOvershadowedStatus(publishedCompactedSegment2, false),
@@ -706,7 +721,7 @@ public class SystemSchemaTest extends CalciteTestBase
       long isRealtime,
       long isOvershadowed,
       CompactionState compactionState
-  )
+  ) throws Exception
   {
     Assert.assertEquals(segmentId, row[0].toString());
     SegmentId id = Iterables.get(SegmentId.iterateAllPossibleParsings(segmentId), 0);
@@ -722,7 +737,11 @@ public class SystemSchemaTest extends CalciteTestBase
     Assert.assertEquals(isAvailable, row[10]);
     Assert.assertEquals(isRealtime, row[11]);
     Assert.assertEquals(isOvershadowed, row[12]);
-    Assert.assertEquals(compactionState, row[16]);
+    if (compactionState == null) {
+      Assert.assertNull(row[16]);
+    } else {
+      Assert.assertEquals(mapper.writeValueAsString(compactionState), row[16]);
+    }
   }
 
   @Test
@@ -733,7 +752,9 @@ public class SystemSchemaTest extends CalciteTestBase
                                                      .withConstructor(
                                                          druidNodeDiscoveryProvider,
                                                          serverInventoryView,
-                                                         authMapper
+                                                         authMapper,
+                                                         overlordClient,
+                                                         coordinatorClient
                                                      )
                                                      .createMock();
     EasyMock.replay(serversTable);
@@ -766,8 +787,8 @@ public class SystemSchemaTest extends CalciteTestBase
             .once();
     EasyMock.expect(druidNodeDiscoveryProvider.getForNodeRole(NodeRole.PEON)).andReturn(peonNodeDiscovery).once();
 
-    EasyMock.expect(coordinatorNodeDiscovery.getAllNodes()).andReturn(ImmutableList.of(coordinator)).once();
-    EasyMock.expect(overlordNodeDiscovery.getAllNodes()).andReturn(ImmutableList.of(overlord)).once();
+    EasyMock.expect(coordinatorNodeDiscovery.getAllNodes()).andReturn(ImmutableList.of(coordinator, coordinator2)).once();
+    EasyMock.expect(overlordNodeDiscovery.getAllNodes()).andReturn(ImmutableList.of(overlord, overlord2)).once();
     EasyMock.expect(brokerNodeDiscovery.getAllNodes())
             .andReturn(ImmutableList.of(broker1, broker2, brokerWithBroadcastSegments))
             .once();
@@ -779,6 +800,9 @@ public class SystemSchemaTest extends CalciteTestBase
     EasyMock.expect(peonNodeDiscovery.getAllNodes()).andReturn(ImmutableList.of(peon1, peon2)).once();
     EasyMock.expect(indexerNodeDiscovery.getAllNodes()).andReturn(ImmutableList.of(indexer)).once();
 
+    EasyMock.expect(coordinatorClient.findCurrentLeader()).andReturn(coordinator.getDruidNode().getHostAndPortToUse()).once();
+    EasyMock.expect(overlordClient.findCurrentLeader()).andReturn(overlord.getDruidNode().getHostAndPortToUse()).once();
+
     final List<DruidServer> servers = new ArrayList<>();
     servers.add(mockDataServer(historical1.getDruidNode().getHostAndPortToUse(), 200L, 1000L, "tier"));
     servers.add(mockDataServer(historical2.getDruidNode().getHostAndPortToUse(), 400L, 1000L, "tier"));
@@ -792,7 +816,7 @@ public class SystemSchemaTest extends CalciteTestBase
             .andReturn(null)
             .once();
 
-    EasyMock.replay(druidNodeDiscoveryProvider, serverInventoryView);
+    EasyMock.replay(druidNodeDiscoveryProvider, serverInventoryView, coordinatorClient, overlordClient);
     EasyMock.replay(servers.toArray(new Object[0]));
     EasyMock.replay(
         coordinatorNodeDiscovery,
@@ -836,6 +860,7 @@ public class SystemSchemaTest extends CalciteTestBase
     rows.sort((Object[] row1, Object[] row2) -> ((Comparable) row1[0]).compareTo(row2[0]));
 
     final List<Object[]> expectedRows = new ArrayList<>();
+    final Long nonLeader = NullHandling.defaultLongValue();
     expectedRows.add(
         createExpectedRow(
             "brokerHost:8082",
@@ -845,7 +870,8 @@ public class SystemSchemaTest extends CalciteTestBase
             NodeRole.BROKER,
             null,
             0L,
-            0L
+            0L,
+            nonLeader
         )
     );
     expectedRows.add(
@@ -857,19 +883,60 @@ public class SystemSchemaTest extends CalciteTestBase
             NodeRole.BROKER,
             "tier",
             0L,
-            1000L
+            1000L,
+            nonLeader
         )
     );
     expectedRows.add(
-        createExpectedRow("histHost:8083", "histHost", 8083, -1, NodeRole.HISTORICAL, "tier", 400L, 1000L)
+        createExpectedRow(
+            "histHost:8083",
+            "histHost",
+            8083,
+            -1,
+            NodeRole.HISTORICAL,
+            "tier",
+            400L,
+            1000L,
+            nonLeader
+        )
     );
     expectedRows.add(
-        createExpectedRow("indexerHost:8091", "indexerHost", 8091, -1, NodeRole.INDEXER, "tier", 0L, 1000L)
+        createExpectedRow(
+            "indexerHost:8091",
+            "indexerHost",
+            8091,
+            -1,
+            NodeRole.INDEXER,
+            "tier",
+            0L,
+            1000L,
+            nonLeader
+        )
     );
     expectedRows.add(
-        createExpectedRow("lameHost:8083", "lameHost", 8083, -1, NodeRole.HISTORICAL, "tier", 0L, 1000L)
+        createExpectedRow(
+            "lameHost:8083",
+            "lameHost",
+            8083,
+            -1,
+            NodeRole.HISTORICAL,
+            "tier",
+            0L,
+            1000L,
+            nonLeader
+        )
     );
-    expectedRows.add(createExpectedRow("localhost:8080", "localhost", 8080, -1, NodeRole.PEON, "tier", 0L, 1000L));
+    expectedRows.add(createExpectedRow(
+        "localhost:8080",
+        "localhost",
+        8080,
+        -1,
+        NodeRole.PEON,
+        "tier",
+        0L,
+        1000L,
+        nonLeader
+    ));
     expectedRows.add(
         createExpectedRow(
             "localhost:8081",
@@ -879,7 +946,8 @@ public class SystemSchemaTest extends CalciteTestBase
             NodeRole.COORDINATOR,
             null,
             0L,
-            0L
+            0L,
+            1L
         )
     );
     expectedRows.add(
@@ -891,11 +959,22 @@ public class SystemSchemaTest extends CalciteTestBase
             NodeRole.BROKER,
             null,
             0L,
-            0L
+            0L,
+            nonLeader
         )
     );
     expectedRows.add(
-        createExpectedRow("localhost:8083", "localhost", 8083, -1, NodeRole.HISTORICAL, "tier", 200L, 1000L)
+        createExpectedRow(
+            "localhost:8083",
+            "localhost",
+            8083,
+            -1,
+            NodeRole.HISTORICAL,
+            "tier",
+            200L,
+            1000L,
+            nonLeader
+        )
     );
     expectedRows.add(
         createExpectedRow(
@@ -906,6 +985,33 @@ public class SystemSchemaTest extends CalciteTestBase
             NodeRole.OVERLORD,
             null,
             0L,
+            0L,
+            1L
+        )
+    );
+    expectedRows.add(
+        createExpectedRow(
+            "localhost:8181",
+            "localhost",
+            8181,
+            -1,
+            NodeRole.COORDINATOR,
+            null,
+            0L,
+            0L,
+            0L
+        )
+    );
+    expectedRows.add(
+        createExpectedRow(
+            "localhost:8190",
+            "localhost",
+            8190,
+            -1,
+            NodeRole.OVERLORD,
+            null,
+            0L,
+            0L,
             0L
         )
     );
@@ -918,7 +1024,8 @@ public class SystemSchemaTest extends CalciteTestBase
             NodeRole.ROUTER,
             null,
             0L,
-            0L
+            0L,
+            nonLeader
         )
     );
     expectedRows.add(
@@ -930,10 +1037,21 @@ public class SystemSchemaTest extends CalciteTestBase
             NodeRole.MIDDLE_MANAGER,
             null,
             0L,
-            0L
+            0L,
+            nonLeader
         )
     );
-    expectedRows.add(createExpectedRow("peonHost:8080", "peonHost", 8080, -1, NodeRole.PEON, "tier", 0L, 1000L));
+    expectedRows.add(createExpectedRow(
+        "peonHost:8080",
+        "peonHost",
+        8080,
+        -1,
+        NodeRole.PEON,
+        "tier",
+        0L,
+        1000L,
+        nonLeader
+    ));
     Assert.assertEquals(expectedRows.size(), rows.size());
     for (int i = 0; i < rows.size(); i++) {
       Assert.assertArrayEquals(expectedRows.get(i), rows.get(i));
@@ -963,7 +1081,8 @@ public class SystemSchemaTest extends CalciteTestBase
       NodeRole nodeRole,
       @Nullable String tier,
       @Nullable Long currSize,
-      @Nullable Long maxSize
+      @Nullable Long maxSize,
+      @Nullable Long isLeader
   )
   {
     return new Object[]{
@@ -974,7 +1093,8 @@ public class SystemSchemaTest extends CalciteTestBase
         StringUtils.toLowerCase(nodeRole.toString()),
         tier,
         currSize,
-        maxSize
+        maxSize,
+        isLeader
     };
   }
 
@@ -1289,12 +1409,6 @@ public class SystemSchemaTest extends CalciteTestBase
           case STRING:
             if (signature.getColumnName(i).equals("segment_id")) {
               expectedClass = SegmentId.class;
-            } else if (signature.getColumnName(i).equals("shardSpec")) {
-              expectedClass = ShardSpec.class;
-            } else if (signature.getColumnName(i).equals("last_compaction_state")) {
-              expectedClass = CompactionState.class;
-            } else if (signature.getColumnName(i).equals("dimensions") || signature.getColumnName(i).equals("metrics")) {
-              expectedClass = List.class;
             } else {
               expectedClass = String.class;
             }
