@@ -35,6 +35,7 @@ import org.apache.druid.client.BrokerSegmentWatcherConfig;
 import org.apache.druid.client.DruidServer;
 import org.apache.druid.client.ServerInventoryView;
 import org.apache.druid.data.input.InputRow;
+import org.apache.druid.data.input.MapBasedInputRow;
 import org.apache.druid.data.input.impl.DimensionSchema;
 import org.apache.druid.data.input.impl.DimensionsSpec;
 import org.apache.druid.data.input.impl.DoubleDimensionSchema;
@@ -118,6 +119,7 @@ import org.apache.druid.sql.calcite.schema.InformationSchema;
 import org.apache.druid.sql.calcite.schema.LookupSchema;
 import org.apache.druid.sql.calcite.schema.MetadataSegmentView;
 import org.apache.druid.sql.calcite.schema.SystemSchema;
+import org.apache.druid.sql.calcite.schema.ViewSchema;
 import org.apache.druid.sql.calcite.view.DruidViewMacroFactory;
 import org.apache.druid.sql.calcite.view.NoopViewManager;
 import org.apache.druid.sql.calcite.view.ViewManager;
@@ -129,6 +131,7 @@ import org.joda.time.Duration;
 import org.joda.time.chrono.ISOChronology;
 
 import javax.annotation.Nullable;
+
 import java.io.File;
 import java.lang.reflect.InvocationTargetException;
 import java.util.ArrayList;
@@ -159,10 +162,12 @@ public class CalciteTests
   public static final String SOME_DATASOURCE = "some_datasource";
   public static final String SOME_DATSOURCE_ESCAPED = "some\\_datasource";
   public static final String SOMEXDATASOURCE = "somexdatasource";
+  public static final String USERVISITDATASOURCE = "visits";
   public static final String DRUID_SCHEMA_NAME = "druid";
   public static final String INFORMATION_SCHEMA_NAME = "INFORMATION_SCHEMA";
   public static final String SYSTEM_SCHEMA_NAME = "sys";
   public static final String LOOKUP_SCHEMA_NAME = "lookup";
+  public static final String VIEW_SCHEMA_NAME = "view";
 
   public static final String TEST_SUPERUSER_NAME = "testSuperuser";
   public static final AuthorizerMapper TEST_AUTHORIZER_MAPPER = new AuthorizerMapper(null)
@@ -177,6 +182,8 @@ public class CalciteTests
 
         if (resource.getType() == ResourceType.DATASOURCE && resource.getName().equals(FORBIDDEN_DATASOURCE)) {
           return new Access(false);
+        } else if (resource.getType() == ResourceType.VIEW && resource.getName().equals("forbiddenView")) {
+          return new Access(false);
         } else {
           return Access.OK;
         }
@@ -361,6 +368,15 @@ public class CalciteTests
       .withRollup(false)
       .build();
 
+  private static final List<String> USER_VISIT_DIMS = ImmutableList.of("user", "country", "city");
+  private static final IncrementalIndexSchema INDEX_SCHEMA_USER_VISIT = new IncrementalIndexSchema.Builder()
+      .withMetrics(
+          new CountAggregatorFactory("cnt")
+      )
+      .withRollup(false)
+      .withMinTimestamp(DateTimes.of("2020-12-31").getMillis())
+      .build();
+
   public static final List<ImmutableMap<String, Object>> RAW_ROWS1 = ImmutableList.of(
       ImmutableMap.<String, Object>builder()
           .put("t", "2000-01-01")
@@ -601,7 +617,8 @@ public class CalciteTests
 
 
   public static final List<InputRow> FORBIDDEN_ROWS = ImmutableList.of(
-      createRow("2000-01-01", "forbidden", "abcd", 9999.0)
+      createRow("2000-01-01", "forbidden", "abcd", 9999.0),
+      createRow("2000-01-02", "forbidden", "a", 1234.0)
   );
 
   // Hi, I'm Troy McClure. You may remember these rows from such benchmarks generator schemas as basic and expression
@@ -642,6 +659,33 @@ public class CalciteTests
       )
   );
 
+  private static List<InputRow> USER_VISIT_ROWS = ImmutableList.of(
+      toRow(
+          "2021-01-01T01:00:00Z",
+          USER_VISIT_DIMS,
+          ImmutableMap.of("user", "alice", "country", "canada", "city", "A")
+      ),
+      toRow(
+          "2021-01-01T02:00:00Z",
+          USER_VISIT_DIMS,
+          ImmutableMap.of("user", "alice", "country", "canada", "city", "B")
+      ),
+      toRow("2021-01-01T03:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "bob", "country", "canada", "city", "A")),
+      toRow("2021-01-01T04:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "alice", "country", "India", "city", "Y")),
+      toRow(
+          "2021-01-02T01:00:00Z",
+          USER_VISIT_DIMS,
+          ImmutableMap.of("user", "alice", "country", "canada", "city", "A")
+      ),
+      toRow("2021-01-02T02:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "bob", "country", "canada", "city", "A")),
+      toRow("2021-01-02T03:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "foo", "country", "canada", "city", "B")),
+      toRow("2021-01-02T04:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "bar", "country", "canada", "city", "B")),
+      toRow("2021-01-02T05:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "alice", "country", "India", "city", "X")),
+      toRow("2021-01-02T06:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "bob", "country", "India", "city", "X")),
+      toRow("2021-01-02T07:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "foo", "country", "India", "city", "X")),
+      toRow("2021-01-03T01:00:00Z", USER_VISIT_DIMS, ImmutableMap.of("user", "foo", "country", "USA", "city", "M"))
+  );
+
   private static final InlineDataSource JOINABLE_BACKING_DATA = InlineDataSource.fromIterable(
       RAW_ROWS1_WITH_NUMERIC_DIMS.stream().map(x -> new Object[]{
           x.get("dim1"),
@@ -851,6 +895,14 @@ public class CalciteTests
         .rows(RAW_ROWS1_X)
         .buildMMappedIndex();
 
+    final QueryableIndex userVisitIndex = IndexBuilder
+        .create()
+        .tmpDir(new File(tmpDir, "8"))
+        .segmentWriteOutMediumFactory(OffHeapMemorySegmentWriteOutMediumFactory.instance())
+        .schema(INDEX_SCHEMA)
+        .rows(USER_VISIT_ROWS)
+        .buildMMappedIndex();
+
 
     return new SpecificSegmentsQuerySegmentWalker(
         conglomerate,
@@ -938,9 +990,23 @@ public class CalciteTests
                    .size(0)
                    .build(),
         indexNumericDims
+    ).add(
+        DataSegment.builder()
+                   .dataSource(USERVISITDATASOURCE)
+                   .interval(userVisitIndex.getDataInterval())
+                   .version("1")
+                   .shardSpec(new LinearShardSpec(0))
+                   .size(0)
+                   .build(),
+        userVisitIndex
     );
   }
 
+  private static MapBasedInputRow toRow(String time, List<String> dimensions, Map<String, Object> event)
+  {
+    return new MapBasedInputRow(DateTimes.ISO_DATE_OPTIONAL_TIME.parse(time), dimensions, event);
+  }
+
   public static ExprMacroTable createExprMacroTable()
   {
     final List<ExprMacroTable.ExprMacro> exprMacros = new ArrayList<>();
@@ -998,22 +1064,45 @@ public class CalciteTests
   )
   {
 
-    final DruidNode coordinatorNode = new DruidNode("test", "dummy", false, 8080, null, true, false);
+    final DruidNode coordinatorNode = new DruidNode("test-coordinator", "dummy", false, 8081, null, true, false);
     FakeDruidNodeDiscoveryProvider provider = new FakeDruidNodeDiscoveryProvider(
         ImmutableMap.of(
             NodeRole.COORDINATOR, new FakeDruidNodeDiscovery(ImmutableMap.of(NodeRole.COORDINATOR, coordinatorNode))
         )
     );
 
+    final DruidNode overlordNode = new DruidNode("test-overlord", "dummy", false, 8090, null, true, false);
+    FakeDruidNodeDiscoveryProvider overlordProvider = new FakeDruidNodeDiscoveryProvider(
+        ImmutableMap.of(
+            NodeRole.OVERLORD, new FakeDruidNodeDiscovery(ImmutableMap.of(NodeRole.OVERLORD, coordinatorNode))
+        )
+    );
+
     final DruidLeaderClient druidLeaderClient = new DruidLeaderClient(
         new FakeHttpClient(),
         provider,
         NodeRole.COORDINATOR,
-        "/simple/leader",
-        () -> {
-          throw new UnsupportedOperationException();
-        }
-    );
+        "/simple/leader"
+    ) {
+      @Override
+      public String findCurrentLeader()
+      {
+        return coordinatorNode.getHostAndPortToUse();
+      }
+    };
+
+    final DruidLeaderClient overlordLeaderClient = new DruidLeaderClient(
+        new FakeHttpClient(),
+        overlordProvider,
+        NodeRole.OVERLORD,
+        "/simple/leader"
+    ) {
+      @Override
+      public String findCurrentLeader()
+      {
+        return overlordNode.getHostAndPortToUse();
+      }
+    };
 
     return new SystemSchema(
         druidSchema,
@@ -1027,7 +1116,7 @@ public class CalciteTests
         new FakeServerInventoryView(),
         authorizerMapper,
         druidLeaderClient,
-        druidLeaderClient,
+        overlordLeaderClient,
         provider,
         getJsonMapper()
     );
@@ -1046,7 +1135,11 @@ public class CalciteTests
 
     SchemaPlus rootSchema = CalciteSchema.createRootSchema(false, false).plus();
     InformationSchema informationSchema =
-        new InformationSchema(rootSchema, authorizerMapper, CalciteTests.DRUID_SCHEMA_NAME);
+        new InformationSchema(
+            rootSchema,
+            authorizerMapper,
+            CalciteTests.DRUID_SCHEMA_NAME
+        );
     LookupSchema lookupSchema = CalciteTests.createMockLookupSchema();
     rootSchema.add(CalciteTests.DRUID_SCHEMA_NAME, druidSchema);
     rootSchema.add(CalciteTests.INFORMATION_SCHEMA_NAME, informationSchema);
@@ -1068,12 +1161,17 @@ public class CalciteTests
         CalciteTests.createMockSystemSchema(druidSchema, walker, plannerConfig, authorizerMapper);
     SchemaPlus rootSchema = CalciteSchema.createRootSchema(false, false).plus();
     InformationSchema informationSchema =
-        new InformationSchema(rootSchema, authorizerMapper, CalciteTests.DRUID_SCHEMA_NAME);
+        new InformationSchema(
+            rootSchema,
+            authorizerMapper,
+            CalciteTests.DRUID_SCHEMA_NAME
+        );
     LookupSchema lookupSchema = CalciteTests.createMockLookupSchema();
     rootSchema.add(CalciteTests.DRUID_SCHEMA_NAME, druidSchema);
     rootSchema.add(CalciteTests.INFORMATION_SCHEMA_NAME, informationSchema);
     rootSchema.add(CalciteTests.SYSTEM_SCHEMA_NAME, systemSchema);
     rootSchema.add(CalciteTests.LOOKUP_SCHEMA_NAME, lookupSchema);
+    rootSchema.add(CalciteTests.VIEW_SCHEMA_NAME, new ViewSchema(viewManager));
     return rootSchema;
   }
 
@@ -1121,7 +1219,6 @@ public class CalciteTests
         },
         createDefaultJoinableFactory(),
         plannerConfig,
-        viewManager,
         TEST_AUTHENTICATOR_ESCALATOR
     );
 
