@@ -19,16 +19,10 @@
 const common = require('./jest.common.config');
 
 module.exports = Object.assign(common, {
-  "moduleNameMapper": {
-    "\\.s?css$": "identity-obj-proxy"
+  moduleNameMapper: {
+    '\\.s?css$': 'identity-obj-proxy',
   },
-  "snapshotSerializers": [
-    "enzyme-to-json/serializer"
-  ],
-  "setupFilesAfterEnv": [
-    "<rootDir>src/setup-tests.ts"
-  ],
-  "testMatch": [
-    "**/src/**/?(*.)+(spec).(ts|tsx)"
-  ],
+  snapshotSerializers: ['enzyme-to-json/serializer'],
+  setupFilesAfterEnv: ['<rootDir>src/setup-tests.ts'],
+  testMatch: ['**/src/**/?(*.)+(spec).(ts|tsx)'],
 });
