@@ -19,13 +19,13 @@
 import { shallow } from 'enzyme';
 import React from 'react';
 
-import { Capabilities } from '../../utils/capabilities';
+import { Capabilities } from '../../utils';
 
 import { HeaderBar } from './header-bar';
 
 describe('header bar', () => {
   it('matches snapshot', () => {
-    const headerBar = shallow(<HeaderBar active={'load-data'} capabilities={Capabilities.FULL} />);
+    const headerBar = shallow(<HeaderBar active="load-data" capabilities={Capabilities.FULL} />);
     expect(headerBar).toMatchSnapshot();
   });
 });
