@@ -23,7 +23,7 @@ import { Loader } from './loader';
 
 describe('loader', () => {
   it('matches snapshot', () => {
-    const loader = <Loader loadingText={'test'} />;
+    const loader = <Loader loadingText="test" />;
     const { container } = render(loader);
     expect(container.firstChild).toMatchSnapshot();
   });
