@@ -23,7 +23,7 @@ import { ShowHistory } from './show-history';
 
 describe('show history', () => {
   it('matches snapshot', () => {
-    const showJson = <ShowHistory endpoint={'test'} downloadFilename={'test'} />;
+    const showJson = <ShowHistory endpoint="test" downloadFilename="test" />;
     const { container } = render(showJson);
     expect(container.firstChild).toMatchSnapshot();
   });
