@@ -24,7 +24,7 @@ import { ShowLog } from './show-log';
 describe('show log', () => {
   it('describe show log', () => {
     const showLog = (
-      <ShowLog status={'RUNNING'} endpoint={'/druid/index/test/log'} downloadFilename={'test'} />
+      <ShowLog status="RUNNING" endpoint="/druid/index/test/log" downloadFilename="test" />
     );
     const { container } = render(showLog);
     expect(container.firstChild).toMatchSnapshot();
