@@ -17,10 +17,11 @@
  */
 
 import 'core-js/stable';
+
 import { configure } from 'enzyme';
 import enzymeAdapterReact16 from 'enzyme-adapter-react-16';
 
-import { UrlBaser } from './singletons/url-baser';
+import { UrlBaser } from './singletons';
 
 configure({ adapter: new (enzymeAdapterReact16 as any)() });
 
