@@ -19,7 +19,7 @@
 import { render } from '@testing-library/react';
 import React from 'react';
 
-import { Capabilities } from '../../../utils/capabilities';
+import { Capabilities } from '../../../utils';
 
 import { SegmentsCard } from './segments-card';
 
