@@ -19,7 +19,7 @@
 import { shallow } from 'enzyme';
 import React from 'react';
 
-import { Capabilities } from '../../utils/capabilities';
+import { Capabilities } from '../../utils';
 
 import { IngestionView } from './ingestion-view';
 
@@ -27,12 +27,11 @@ describe('tasks view', () => {
   it('matches snapshot', () => {
     const taskView = shallow(
       <IngestionView
-        openDialog={'test'}
-        taskGroupId={'test'}
-        datasourceId={'datasource'}
+        openDialog="test"
+        taskGroupId="test"
+        datasourceId="datasource"
         goToDatasource={() => {}}
         goToQuery={() => {}}
-        goToMiddleManager={() => {}}
         goToLoadData={() => {}}
         capabilities={Capabilities.FULL}
       />,
