@@ -16,23 +16,23 @@
  * limitations under the License.
  */
 
-import { HTMLSelect, ITreeNode, Menu, MenuItem, Popover, Position, Tree } from '@blueprintjs/core';
+import { HTMLSelect, Menu, MenuItem, Position, Tree, TreeNodeInfo } from '@blueprintjs/core';
 import { IconNames } from '@blueprintjs/icons';
+import { Popover2 } from '@blueprintjs/popover2';
 import {
-  SqlAlias,
   SqlComparison,
   SqlExpression,
   SqlFunction,
   SqlJoinPart,
   SqlQuery,
   SqlRef,
+  SqlTableRef,
 } from 'druid-query-toolkit';
 import React, { ChangeEvent } from 'react';
 
 import { Loader } from '../../../components';
 import { Deferred } from '../../../components/deferred/deferred';
-import { copyAndAlert, groupBy, prettyPrintSql } from '../../../utils';
-import { ColumnMetadata } from '../../../utils/column-metadata';
+import { ColumnMetadata, copyAndAlert, groupBy, oneOf, prettyPrintSql } from '../../../utils';
 import { dataTypeToIcon } from '../query-utils';
 
 import { NumberMenuItems, StringMenuItems, TimeMenuItems } from './column-tree-menu';
@@ -42,6 +42,15 @@ import './column-tree.scss';
 const LAST_DAY = SqlExpression.parse(`__time >= CURRENT_TIMESTAMP - INTERVAL '1' DAY`);
 const COUNT_STAR = SqlFunction.COUNT_STAR.as('Count');
 
+function getCountExpression(columnNames: string[]): SqlExpression {
+  for (const columnName of columnNames) {
+    if (columnName === 'count' || columnName === '__count') {
+      return SqlFunction.simple('SUM', [SqlRef.column(columnName)]).as('Count');
+    }
+  }
+  return COUNT_STAR;
+}
+
 const STRING_QUERY = SqlQuery.parse(`SELECT
   ?
 FROM ?
@@ -63,39 +72,40 @@ interface HandleColumnClickOptions {
   onQueryChange: (query: SqlQuery, run: boolean) => void;
 }
 
-function handleColumnClick(options: HandleColumnClickOptions): void {
+function handleColumnShow(options: HandleColumnClickOptions): void {
   const { columnSchema, columnTable, columnName, columnType, parsedQuery, onQueryChange } = options;
 
-  let query: SqlQuery;
-  const columnRef = SqlRef.column(columnName);
-  if (columnSchema === 'druid') {
-    if (columnType === 'TIMESTAMP') {
-      query = TIME_QUERY.fillPlaceholders([columnRef, SqlRef.table(columnTable)]) as SqlQuery;
-    } else {
-      query = STRING_QUERY.fillPlaceholders([columnRef, SqlRef.table(columnTable)]) as SqlQuery;
-    }
-  } else {
-    query = STRING_QUERY.fillPlaceholders([
-      columnRef,
-      SqlRef.table(columnTable, columnSchema),
-    ]) as SqlQuery;
-  }
-
+  let from: SqlExpression;
   let where: SqlExpression | undefined;
-  let aggregates: SqlAlias[] = [];
+  let aggregates: SqlExpression[] = [];
   if (parsedQuery && parsedQuery.getFirstTableName() === columnTable) {
+    from = parsedQuery.getFirstFromExpression()!;
     where = parsedQuery.getWhereExpression();
     aggregates = parsedQuery.getAggregateSelectExpressions();
   } else if (columnSchema === 'druid') {
+    from = SqlTableRef.create(columnTable);
     where = LAST_DAY;
+  } else {
+    from = SqlTableRef.create(columnTable, columnSchema);
   }
+
   if (!aggregates.length) {
     aggregates.push(COUNT_STAR);
   }
 
+  const columnRef = SqlRef.column(columnName);
+  let query: SqlQuery;
+  if (columnSchema === 'druid' && columnType === 'TIMESTAMP') {
+    query = TIME_QUERY.fillPlaceholders([columnRef, from]) as SqlQuery;
+  } else {
+    query = STRING_QUERY.fillPlaceholders([columnRef, from]) as SqlQuery;
+  }
+
   let newSelectExpressions = query.selectExpressions;
-  for (const aggregate of aggregates) {
-    newSelectExpressions = newSelectExpressions.addLast(aggregate);
+  if (newSelectExpressions) {
+    for (const aggregate of aggregates) {
+      newSelectExpressions = newSelectExpressions.append(aggregate);
+    }
   }
 
   onQueryChange(
@@ -115,8 +125,8 @@ export interface ColumnTreeProps {
 
 export interface ColumnTreeState {
   prevColumnMetadata?: readonly ColumnMetadata[];
-  columnTree?: ITreeNode[];
-  currentSchemaSubtree?: ITreeNode[];
+  columnTree?: TreeNodeInfo[];
+  currentSchemaSubtree?: TreeNodeInfo[];
   selectedTreeIndex: number;
 }
 
@@ -127,11 +137,11 @@ export function getJoinColumns(parsedQuery: SqlQuery, _table: string) {
     const firstOnExpression = parsedQuery.fromClause.joinParts.first().onExpression;
     if (firstOnExpression instanceof SqlComparison && firstOnExpression.op === '=') {
       const { lhs, rhs } = firstOnExpression;
-      if (lhs instanceof SqlRef && lhs.namespace === 'lookup') {
-        lookupColumn = lhs.column;
+      if (lhs instanceof SqlRef && lhs.getNamespace() === 'lookup') {
+        lookupColumn = lhs.getColumn();
       }
       if (rhs instanceof SqlRef) {
-        originalTableColumn = rhs.column;
+        originalTableColumn = rhs.getColumn();
       }
     }
   }
@@ -150,49 +160,82 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
       const columnTree = groupBy(
         columnMetadata,
         r => r.TABLE_SCHEMA,
-        (metadata, schemaName): ITreeNode => ({
+        (metadata, schemaName): TreeNodeInfo => ({
           id: schemaName,
           label: schemaName,
           childNodes: groupBy(
             metadata,
             r => r.TABLE_NAME,
-            (metadata, tableName): ITreeNode => ({
+            (metadata, tableName): TreeNodeInfo => ({
               id: tableName,
               icon: IconNames.TH,
               label: (
-                <Popover
-                  boundary={'window'}
+                <Popover2
                   position={Position.RIGHT}
                   content={
                     <Deferred
                       content={() => {
                         const parsedQuery = props.getParsedQuery();
-                        const tableRef = SqlRef.table(tableName).as();
+                        const tableRef = SqlTableRef.create(tableName);
                         const prettyTableRef = prettyPrintSql(tableRef);
+                        const countExpression = getCountExpression(
+                          metadata.map(child => child.COLUMN_NAME),
+                        );
+
+                        const getQueryOnTable = () => {
+                          return SqlQuery.create(
+                            SqlTableRef.create(
+                              tableName,
+                              schemaName === 'druid' ? undefined : schemaName,
+                            ),
+                          );
+                        };
+
+                        const getWhere = (defaultToAllTime = false) => {
+                          if (parsedQuery && parsedQuery.getFirstTableName() === tableName) {
+                            return parsedQuery.getWhereExpression();
+                          } else if (schemaName === 'druid') {
+                            return defaultToAllTime ? undefined : LAST_DAY;
+                          } else {
+                            return;
+                          }
+                        };
+
                         return (
                           <Menu>
                             <MenuItem
                               icon={IconNames.FULLSCREEN}
-                              text={`SELECT ... FROM ${tableName}`}
+                              text={`SELECT ...columns... FROM ${tableName}`}
                               onClick={() => {
-                                const tableRef = SqlRef.table(
-                                  tableName,
-                                  schemaName === 'druid' ? undefined : schemaName,
-                                );
-
-                                let where: SqlExpression | undefined;
-                                if (parsedQuery && parsedQuery.getFirstTableName() === tableName) {
-                                  where = parsedQuery.getWhereExpression();
-                                } else if (schemaName === 'druid') {
-                                  where = LAST_DAY;
-                                }
-
                                 onQueryChange(
-                                  SqlQuery.create(tableRef)
+                                  getQueryOnTable()
                                     .changeSelectExpressions(
-                                      metadata.map(child => SqlRef.column(child.COLUMN_NAME).as()),
+                                      metadata.map(child => SqlRef.column(child.COLUMN_NAME)),
                                     )
-                                    .changeWhereExpression(where),
+                                    .changeWhereExpression(getWhere()),
+                                  true,
+                                );
+                              }}
+                            />
+                            <MenuItem
+                              icon={IconNames.FULLSCREEN}
+                              text={`SELECT * FROM ${tableName}`}
+                              onClick={() => {
+                                onQueryChange(
+                                  getQueryOnTable().changeWhereExpression(getWhere()),
+                                  true,
+                                );
+                              }}
+                            />
+                            <MenuItem
+                              icon={IconNames.FULLSCREEN}
+                              text={`SELECT ${countExpression} FROM ${tableName}`}
+                              onClick={() => {
+                                onQueryChange(
+                                  getQueryOnTable()
+                                    .changeSelect(0, countExpression)
+                                    .changeGroupByExpressions([])
+                                    .changeWhereExpression(getWhere(true)),
                                   true,
                                 );
                               }}
@@ -217,7 +260,7 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                               >
                                 <MenuItem
                                   icon={IconNames.LEFT_JOIN}
-                                  text={`Left join`}
+                                  text="Left join"
                                   onClick={() => {
                                     const { lookupColumn, originalTableColumn } = getJoinColumns(
                                       parsedQuery,
@@ -229,7 +272,7 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                                         .addJoin(
                                           SqlJoinPart.create(
                                             'LEFT',
-                                            SqlRef.column(tableName, schemaName).upgrade(),
+                                            SqlTableRef.create(tableName, schemaName),
                                             SqlRef.column(lookupColumn, tableName, 'lookup').equal(
                                               SqlRef.column(
                                                 originalTableColumn,
@@ -244,7 +287,7 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                                 />
                                 <MenuItem
                                   icon={IconNames.INNER_JOIN}
-                                  text={`Inner join`}
+                                  text="Inner join"
                                   onClick={() => {
                                     const { lookupColumn, originalTableColumn } = getJoinColumns(
                                       parsedQuery,
@@ -254,7 +297,7 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                                       parsedQuery.addJoin(
                                         SqlJoinPart.create(
                                           'INNER',
-                                          SqlRef.column(tableName, schemaName).upgrade(),
+                                          SqlTableRef.create(tableName, schemaName),
                                           SqlRef.column(lookupColumn, tableName, 'lookup').equal(
                                             SqlRef.column(
                                               originalTableColumn,
@@ -274,7 +317,7 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                               parsedQuery.getJoins()[0].table.toString() === tableName && (
                                 <MenuItem
                                   icon={IconNames.EXCHANGE}
-                                  text={`Remove join`}
+                                  text="Remove join"
                                   onClick={() => onQueryChange(parsedQuery.removeAllJoins())}
                                 />
                               )}
@@ -283,9 +326,9 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                               parsedQuery.getFirstTableName() === tableName && (
                                 <MenuItem
                                   icon={IconNames.FUNCTION}
-                                  text={`Aggregate COUNT(*)`}
+                                  text="Aggregate COUNT(*)"
                                   onClick={() =>
-                                    onQueryChange(parsedQuery.addSelectExpression(COUNT_STAR), true)
+                                    onQueryChange(parsedQuery.addSelect(COUNT_STAR), true)
                                   }
                                 />
                               )}
@@ -305,20 +348,18 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                     />
                   }
                 >
-                  <div>{tableName}</div>
-                </Popover>
+                  {tableName}
+                </Popover2>
               ),
               childNodes: metadata
                 .map(
-                  (columnData): ITreeNode => ({
+                  (columnData): TreeNodeInfo => ({
                     id: columnData.COLUMN_NAME,
                     icon: dataTypeToIcon(columnData.DATA_TYPE),
                     label: (
-                      <Popover
-                        boundary={'window'}
+                      <Popover2
                         position={Position.RIGHT}
                         autoFocus={false}
-                        targetClassName={'bp3-popover-open'}
                         content={
                           <Deferred
                             content={() => {
@@ -329,7 +370,7 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                                     icon={IconNames.FULLSCREEN}
                                     text={`Show: ${columnData.COLUMN_NAME}`}
                                     onClick={() => {
-                                      handleColumnClick({
+                                      handleColumnShow({
                                         columnSchema: schemaName,
                                         columnTable: tableName,
                                         columnName: columnData.COLUMN_NAME,
@@ -340,8 +381,7 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                                     }}
                                   />
                                   {parsedQuery &&
-                                    (columnData.DATA_TYPE === 'BIGINT' ||
-                                      columnData.DATA_TYPE === 'FLOAT') && (
+                                    oneOf(columnData.DATA_TYPE, 'BIGINT', 'FLOAT', 'DOUBLE') && (
                                       <NumberMenuItems
                                         table={tableName}
                                         schema={schemaName}
@@ -384,15 +424,13 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
                           />
                         }
                       >
-                        <div>{columnData.COLUMN_NAME}</div>
-                      </Popover>
+                        {columnData.COLUMN_NAME}
+                      </Popover2>
                     ),
                   }),
                 )
                 .sort((a, b) =>
-                  String(a.id)
-                    .toLowerCase()
-                    .localeCompare(String(b.id).toLowerCase()),
+                  String(a.id).toLowerCase().localeCompare(String(b.id).toLowerCase()),
                 ),
             }),
           ),
@@ -444,7 +482,7 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
     };
   }
 
-  renderSchemaSelector() {
+  private renderSchemaSelector() {
     const { columnTree, selectedTreeIndex } = this.state;
     if (!columnTree) return null;
 
@@ -466,13 +504,12 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
     );
   }
 
-  private handleSchemaSelectorChange = (e: ChangeEvent<HTMLSelectElement>): void => {
+  private readonly handleSchemaSelectorChange = (e: ChangeEvent<HTMLSelectElement>): void => {
     const { columnTree } = this.state;
+    if (!columnTree) return;
 
     const selectedTreeIndex = Number(e.target.value);
 
-    if (!columnTree) return;
-
     const currentSchemaSubtree =
       columnTree[selectedTreeIndex > -1 ? selectedTreeIndex : 0].childNodes;
 
@@ -482,24 +519,16 @@ export class ColumnTree extends React.PureComponent<ColumnTreeProps, ColumnTreeS
     });
   };
 
-  private handleNodeCollapse = (nodeData: ITreeNode) => {
+  private readonly handleNodeCollapse = (nodeData: TreeNodeInfo) => {
     nodeData.isExpanded = false;
-    this.bounceState();
+    this.forceUpdate();
   };
 
-  private handleNodeExpand = (nodeData: ITreeNode) => {
+  private readonly handleNodeExpand = (nodeData: TreeNodeInfo) => {
     nodeData.isExpanded = true;
-    this.bounceState();
+    this.forceUpdate();
   };
 
-  bounceState() {
-    const { columnTree } = this.state;
-    if (!columnTree) return;
-    this.setState(prevState => ({
-      columnTree: prevState.columnTree ? prevState.columnTree.slice() : undefined,
-    }));
-  }
-
   render(): JSX.Element | null {
     const { columnMetadataLoading } = this.props;
     const { currentSchemaSubtree } = this.state;
