@@ -21,7 +21,7 @@ import React from 'react';
 
 import { QueryInput } from './query-input';
 
-describe('query input', () => {
+describe('QueryInput', () => {
   it('matches snapshot', () => {
     const sqlControl = (
       <QueryInput queryString="hello world" onQueryStringChange={() => {}} runeMode={false} />
@@ -33,7 +33,7 @@ describe('query input', () => {
 
   it('correctly formats helper HTML', () => {
     expect(
-      QueryInput.completerToHtml({
+      QueryInput.makeDocHtml({
         caption: 'COUNT',
         syntax: 'COUNT(*)',
         description: 'Counts the number of things',
