@@ -19,14 +19,14 @@
 import { shallow } from 'enzyme';
 import React from 'react';
 
-import { Capabilities } from '../../utils/capabilities';
+import { Capabilities } from '../../utils';
 import { SegmentsView } from '../segments-view/segments-view';
 
 describe('segments-view', () => {
   it('matches snapshot', () => {
     const segmentsView = shallow(
       <SegmentsView
-        datasource={'test'}
+        datasource="test"
         onlyUnavailable={false}
         goToQuery={() => {}}
         capabilities={Capabilities.FULL}
